/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class AddOnScriptAnalyser;

#ifndef __AddOnScriptAnalyser_h__
#define __AddOnScriptAnalyser_h__

#include "ScriptAnalyser.h"
#include "HFileModifier.h"
#include "CCFileModifier.h"

#define ERROR_ADDON_SCRIPT_LIMITED_POSITION "Must find 'after', 'before' or 'at the end'"
#define ERROR_ADDON_SCRIPT_POSITION "Must find 'after', 'before', 'at the beginning' or 'at the end'"
#define ERROR_ADDON_SCRIPT_PROTECTION "Must find 'public', 'protected' or 'private'"
#define ERROR_ADDON_SCRIPT_VIRTUALITY "Must find 'no', 'virtual' or 'pure virtual'"
#define ERROR_ADDON_SCRIPT_IN "Must find 'in .h' or 'in .cc'"
#define ERROR_ADDON_SCRIPT_INVALID_END "Invalid end line"
#define ERROR_ADDON_SCRIPT_UNKNOWN "Unknown command"

#define ERROR_ADDON_SCRIPT_INI_FILE "Must find files to complete (example:'*.ini')"


/**
   Main class for "addon". This program enables you to add code in source easily, this added code can also be removed.
   @author David FAUTHOUX
 */
class AddOnScriptAnalyser:public ScriptAnalyser {
 protected:
  ///
  bool verbose;
  ///Addon name
  const String *name;
  ///
  const String *author;
  ///Modifiying class name
  const String *nameClass;
  ///Mark to add before the added code (so it can be removed)
  List< const String * > mark;
  ///Mark to add before lines in Makefile and in .addon
  List< const String * > markMakefile;

  ///
  void messageErr(const String& m) const throw();
  ///
  void message(const String& m) const throw();
  ///
  void message(const String& m0,const String& m1) const throw();
  
  ///
  const String *getString(const String& before) throw(Exception);
  ///
  List<const String *> *getPureBlock(const String& begin,const String& end,bool checkEnds=true) throw(Exception);
  ///
  void checkNotEnd(const String& s) throw(Exception);
  ///
  List<const String *> *getPureBlockWithoutBegin(const String& begin,const String& end,bool checkEnds=true) throw(Exception);

  ///
  void getBlockAddClass() throw(Exception);
  ///
  void getBlockAddParameter(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockAddFunction(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockAddDefine(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockAddInclude(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockAddInEnum(HFileModifier& hFile) throw(Exception);
  ///
  void getBlockAddInheritance(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockAddSubclass(HFileModifier& hFile) throw(Exception);
  ///
  void getBlockAddRoughly(HFileModifier& hFile,CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockCompleteFunction(CCFileModifier& ccFile) throw(Exception);
  ///
  void getBlockCompleteClass() throw(Exception);
  ///
  void getBlockCompleteIniFiles() throw(Exception);

  ///
  void supprBlockCompleteIniFiles() throw(Exception);


  ///
  void checkNeeded(const String& name,const String& author) throw(Exception);

  ///
  void getBlockNeededAddon() throw(Exception);

  ///
  void includeScript(const String& file,List<const String *> param,bool suppress) throw(Exception);
  ///
  void getBlockIncludeScript(bool suppress=false) throw(Exception);

  ///
  void getVersionNeeded() throw(Exception);

  ///
  bool isIf(const String& s) const throw();
  ///
  void cleanTheIfs(bool all) throw(Exception);

  ///
  void replaceWord(const String& from,const String& to) throw();

  ///
  void replaceParam(List<const String*> param,const String& name,const String& author) throw (Exception);

  ///  
  void completeAddonFile() throw(Exception);
  ///
  void warningNeeded() throw();
 

  ///
  void run() throw(Exception);
  ///
  void unrun() throw(Exception);

 public:
  ///
  AddOnScriptAnalyser(const String& nameFile,bool verbose) throw(Exception);
  ///
  ~AddOnScriptAnalyser() throw();
  ///
  void analyse() throw(Exception);
  ///
  void analyse(List<const String*>& param,const String& name,const String& author) throw(Exception);

  ///
  void suppress() throw(Exception);
  ///
  void suppress(List<const String*>& param,const String& name,const String& author) throw(Exception);

  ///Gives the list of addons added (using .addon)
  static void listAddons() throw();

  ///
  bool checkIfAlready(const String& f) throw();

};

#endif

