/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Background.h"

//les fichiers gif peuvent avoir plusieurs couches
// couche 0 = fond
// i>0 [ couche i = par-dessus] applaties

void Background::aplatir(GifIn& gif,int jDecalShadow,int iDecalShadow) throw()
{
  gif.place();
  gif.walk();
  data=gif.get()->getData();
  int jSize=gif.get()->nbLine();
  int iSize=gif.get()->nbColumn();
  underMask=new UnderMaskMatrix(jSize,iSize);
  
  if(gif.walk()) {
    PixelMatrix *mask=gif.get()->getMask();
    PixelMatrix *d=gif.get()->getData();
    for(int j=0;j<jSize;j++)
      for(int i=0;i<iSize;i++)
	if((*mask)[j][i]==PIXEL_MASK_FULL || (*mask)[j][i]==PIXEL_MASK_ONLY_SHADOW) {
	  (*data)[j][i]=(*d)[j][i];
	  (*underMask)[j][i]=0;
	} else (*underMask)[j][i]=1;
    delete gif.get();
  }
  while(gif.walk()) {
    PixelMatrix *mask=gif.get()->getMask();
    PixelMatrix *d=gif.get()->getData();
    for(int j=0;j<jSize;j++)
      for(int i=0;i<iSize;i++)
	if((*mask)[j][i]==PIXEL_MASK_FULL || (*mask)[j][i]==PIXEL_MASK_ONLY_SHADOW) {
	  (*data)[j][i]=(*d)[j][i];
	  (*underMask)[j][i]=0;
	  if(j+jDecalShadow<jSize && i+iDecalShadow<iSize)
	    (*data)[j+jDecalShadow][i+iDecalShadow]=
	      palettePass((*data)[j+jDecalShadow][i+iDecalShadow] | PIXEL_DATA_ONLY_SHADOW)
	      | PIXEL_DATA_ONLY_SHADOW;
	}
    delete gif.get();
  }
  
}

Background::Background(const String *nameFile,GraphicMode& graphicMode,
		       int jDecalShadow,int iDecalShadow)
  throw (Exception)
{
  Tji sizeScreen;

  winScreen=new Window(graphicMode);
  sizeScreen.j=graphicMode.nbLine();
  sizeScreen.i=graphicMode.nbColumn();
  winScreen->set(0,0,sizeScreen.j,sizeScreen.i);

  if(nameFile) {

    GifIn gif(*nameFile);

    gif.place();
    gif.walk();
    size.j=gif.get()->nbLine();
    size.i=gif.get()->nbColumn();

    aplatir(gif,jDecalShadow,iDecalShadow);
    winData=new Window(*data);
    winData->set(0,0,sizeScreen.j,sizeScreen.i);

    windowPipe=new WindowPipe(*winScreen,*winData,*underMask);

  }
  else {
    size.j=graphicMode.nbLine();
    size.i=graphicMode.nbColumn();

    data=new PixelMatrix(size.j,size.i);
    underMask=new UnderMaskMatrix(size.j,size.i);
    for(int j=0;j<size.j;j++)
      for(int i=0;i<size.i;i++) {
	(*data)[j][i]=toPixel(0,0,0,0);
	(*underMask)[j][i]=1;
      }
    winData=new Window(*data);
    winData->set(0,0,sizeScreen.j,sizeScreen.i);

    windowPipe=new WindowPipe(*winScreen,*winData,*underMask);
  }
}

Background::~Background() throw()
{
  delete winScreen;
  delete winData;
  delete windowPipe;
  delete underMask;
  delete data;
}

void Background::paint() throw()
{
  windowPipe->paintWindowFrom();
}

WindowPipe *Background::getWindowPipe() throw()
{
  return windowPipe;
}

int Background::nbLine() const throw()
{
  return size.j;
}

int Background::nbColumn() const throw()
{
  return size.i;
}
