/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */


#include <stdlib.h>
#include "Blower.h"

#define DEFAULT_SCREEN_BLOW 0

#define DEFAULT_TRIANGLE_J 20
#define DEFAULT_TRIANGLE_I 32
#define DEFAULT_RETURN 40
#define DEFAULT_ROTATION_MINI 3
#define DEFAULT_ROTATION_MAXI 18
#define DEFAULT_PARAM1 200
#define DEFAULT_PARAM2 4

#define DEFAULT_MAX_TENSION 300

Blower::Blower(Ini& ini,int jSize,int iSize) throw()
  :Redirector(ini.getWithDefault("[Slow]",0,
				 "screen blow",0,
				 DEFAULT_SCREEN_BLOW)),
   exploser(iSize,jSize,
	    ini.getWithDefault("[Blower]",0,"triangle i",0,DEFAULT_TRIANGLE_I),
	    ini.getWithDefault("[Blower]",0,"triangle j",0,DEFAULT_TRIANGLE_J)),
   buffer(jSize,iSize),
   explode(false),
   reverse(false),
   count(0),
   maxCount(ini.getWithDefault("[Blower]",0,"returns after",0,DEFAULT_RETURN)),
   jSizeDiv2(jSize/2),
   iSizeDiv2(iSize/2),
   minRotation(ini.getWithDefault("[Blower]",0,"rotation mini",0,DEFAULT_ROTATION_MINI)),
   maxRotation(ini.getWithDefault("[Blower]",0,"rotation maxi",0,DEFAULT_ROTATION_MAXI)),
   param1(ini.getWithDefault("[Blower]",0,"first parameter",0,DEFAULT_PARAM1)),
   param2(ini.getWithDefault("[Blower]",0,"second parameter",0,DEFAULT_PARAM2)),
   tension(0),
   maxTension(ini.getWithDefault("[Blower]",0,"tension max",0,DEFAULT_MAX_TENSION))
{
  randRotation=maxRotation-minRotation;
  if(randRotation<=0) randRotation=1;

  if(param1==0) param1=1;


  coefRotation=new Vector[exploser.getNbTriangles()];

  globalCoef=new int[exploser.getNbTriangles()];
  
  calcCoefRotation();
  calcGlobalCoef();
}
 

Blower::~Blower() throw()
{
  delete[] coefRotation;
  delete[] globalCoef;
}

PixelMatrix *Blower::pipeIt(PixelMatrix& from) throw()
{
  if(!explode) return &from;

  buffer=toPixel(0,0,0,0);
    
  for (int i=0;i<exploser.getNbTriangles();i++) {
    Triangle3D t=exploser.getTriangle(i);

    Point2D triangle[3];
    
    bool fillIt=true;
    for (int j=0;j<3 && fillIt;j++) {
      if (t[j][2]+256<=10)
	fillIt=false;

      int projX=int((t[j][0]*256)/(t[j][2]+256))+iSizeDiv2;
      int projY=int((t[j][1]*256)/(t[j][2]+256))+jSizeDiv2;
	
      triangle[j].setX(projX);
      triangle[j].setY(projY);
      triangle[j].setS(t[j].getS());
      triangle[j].setT(t[j].getT());
    }
    
    if (fillIt) {
      Triangle tri(triangle[0],triangle[1],triangle[2]);
      tri.fill(buffer,from);
    }
  }

  return &buffer;
}

int Blower::virtualWalk() throw()
{
  tension--;
  if(tension<0) tension=0;

  if (!explode) return 1;

  for (int i=0;i<exploser.getNbTriangles();i++) {
    Vector rotation;
    rotation=Vector(globalCoef[i]*coefRotation[i][0],
		    globalCoef[i]*coefRotation[i][1],
		    globalCoef[i]*coefRotation[i][2]);
    
    Vector deplacement=Vector(exploser.getOrigin(i)).normalize();
      
    deplacement=deplacement+Vector(0,0,-globalCoef[i]/3);
    
    exploser.setRotation(i,rotation);
    exploser.setTranslation(i,deplacement*globalCoef[i]);
    
  }
  
  walkGlobalCoef();
  
 if(reverse) {
   if(count<0) explode=false;
   else count--;
 }
 else {
   if(count>=maxCount) reverse=true;
   else count++;
 }

  return 1;
}


void Blower::blowUp() throw()
{
  explode=true;
  reverse=false;
  count=0;
    
  calcCoefRotation();
  calcGlobalCoef();
  
  for (int i=0;i<exploser.getNbTriangles();i++) {
    Vector rotation;
    rotation=Vector(0,0,0);
    
    Vector deplacement=Vector(0,0,0);
      
    exploser.setRotation(i,rotation);
    exploser.setTranslation(i,deplacement*0);
  }

  walkGlobalCoef();
  count=2;
  t=s;
}



void Blower::calcCoefRotation() throw()
{
  for (int i=0;i<exploser.getNbTriangles();i++)
    for (int j=0;j<3;j++)
      coefRotation[i][j]=minRotation+rand()%randRotation;
}

void Blower::calcGlobalCoef() throw()
{
  for (int i=0;i<exploser.getNbTriangles();i++)
    globalCoef[i]=0;
}

void Blower::walkGlobalCoef() throw()
{
  int aux=(param1+5+(maxCount-count)*param2)/param1;
  if(reverse) {
    for (int i=0;i<exploser.getNbTriangles();i++)
      globalCoef[i]-=aux;
  }
  else {
    for (int i=0;i<exploser.getNbTriangles();i++)
      globalCoef[i]+=aux;
  }
}


void Blower::incTension(int puissance) throw()
{
  if(maxTension<0 || explode) return;
  tension+=puissance;
  if(tension>=maxTension) {
    tension=0;
    blowUp();
  }
}
