/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <limits.h>
#include "Bonhomme20.h"
#include "Bonhomme.h"
#include "Anim.h"
#include "LoopAnim.h"
#include "Grille.h"

#define DEFAULT_NB_BOMBE 1
#define DEFAULT_PUISSANCE 2
#define DEFAULT_DEP 2
#define DEFAULT_DEP_CHICKEN 4
#define DEFAULT_DEP_TELEPORT 8
#define DEFAULT_RALENTISSEMENT_WIN95 8
#define DEFAULT_TEMPS_GRIS 10
#define DEFAULT_TEMPS_CHICKEN 10
#define DEFAULT_TEMPS_FEU_CUL 10

#define DEFAULT_TEMPS_SPEED 5000

#define DEFAULT_TEMPS_INVINCIBLE 10
#define DEFAULT_TEMPS_INVISIBLE 10
#define DEFAULT_TEMPS_SURPUISSANT 10
#define DEFAULT_TEMPS_WIN95 10
#define DEFAULT_TEMPS_TRAVERSE_MUR 10
#define DEFAULT_TEMPS_COEUR 10
#define DEFAULT_TEMPS_TELEPORT 10

#define DEFAULT_ANIM_SLOW 0

#define DEFAULT_SIZE_ACTION 20
#define DEFAULT_DECAL_TOLERANT_ACTION 5
#define DEFAULT_DECAL_POSE_BOMBE 9
#define DEFAULT_DECAL_PAINT 0

//
#define DEFAULT_STRIP_UP_WALL "you are on the walls"
#define DEFAULT_STRIP_DOWN_WALL "you are on the floor"
#define DEFAULT_STRIP_CRAME "you are burned ! but you can wake up : move the quickest you can !!"
#define DEFAULT_STRIP_BRISE "now you are dead... but you can continue playing being a ghost !"
#define DEFAULT_STRIP_PICOTE_DU_PAIN_DUR "une poule sur un mur qui picote du pain dur"


Bonhomme20::Bonhomme20(Ini& ini,int numeroSection,Bonhomme& bonhomme,
		       Sounder& sounder,
		       TextPainter& textPainter,
		       StripContainer& stripContainer,
		       const PixelPalette& pixelPalette) throw(Exception)
  :iniRef(&ini),
   bonhommeRef(&bonhomme),
   grilleRef(0),
   sounderRef(&sounder),
   artificialIntelligenceContainer(new ArtificialIntelligenceContainer(ini)),
   initNbBombe(ini.getWithDefault("[Bonhomme]",numeroSection,"amount bombe",0,
				  DEFAULT_NB_BOMBE)),
   initPuissance(ini.getWithDefault("[Bonhomme]",numeroSection,"puissance",0,
				    DEFAULT_PUISSANCE)),
   depNormal(ini.getWithDefault("[Bonhomme]",numeroSection,"deplacement",0,
				DEFAULT_DEP)),
   depChicken(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "deplacement chicken",0,DEFAULT_DEP_CHICKEN)),
   depTeleport(ini.getWithDefault("[Bonhomme]",numeroSection,
				  "deplacement teleport",0,
				  DEFAULT_DEP_TELEPORT)),
   ralentissementWin95(ini.getWithDefault("[Bonhomme]",numeroSection,
					  "ralentissement win95",0,
					  DEFAULT_RALENTISSEMENT_WIN95)),
   tempsGris(ini.getWithDefault("[Bonhomme]",numeroSection,"temps gris",0,
				DEFAULT_TEMPS_GRIS)),
   tempsChicken(ini.getWithDefault("[Bonhomme]",numeroSection,
				   "temps chicken",0,DEFAULT_TEMPS_CHICKEN)),
   tempsFeuCul(ini.getWithDefault("[Bonhomme]",numeroSection,
				  "temps feu au cul",0,
				  DEFAULT_TEMPS_FEU_CUL)),
   tempsSpeed(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "temps speed",0,
				 DEFAULT_TEMPS_SPEED)),
   tempsInvincible(ini.getWithDefault("[Bonhomme]",numeroSection,
				      "temps invincible",0,
				      DEFAULT_TEMPS_INVINCIBLE)),
   tempsInvisible(ini.getWithDefault("[Bonhomme]",numeroSection,
				     "temps invisible",0,
				     DEFAULT_TEMPS_INVISIBLE)),
   tempsSurpuissant(ini.getWithDefault("[Bonhomme]",numeroSection,
				       "temps surpuissant",0,
				       DEFAULT_TEMPS_SURPUISSANT)),
   tempsWin95(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "temps win95",0,
				 DEFAULT_TEMPS_WIN95)),
   tempsTraverseMur(ini.getWithDefault("[Bonhomme]",numeroSection,
				       "temps traverse mur",0,
				       DEFAULT_TEMPS_TRAVERSE_MUR)),
   tempsCoeur(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "temps coeur",0,
				 DEFAULT_TEMPS_COEUR)),
   tempsTeleport(ini.getWithDefault("[Bonhomme]",numeroSection,
				    "temps teleport",0,
				    DEFAULT_TEMPS_TELEPORT)),
   //
   boue(new Boue(*this,
		 *new Anim(*ini.get("[Bonhomme]",numeroSection,
				    "anim trace de boue up",0),
			   0,
			   ini.getListSeparator()),
		 *new Anim(*ini.get("[Bonhomme]",numeroSection,
				    "anim trace de boue down",0),
			   0,
			   ini.getListSeparator()),
		 *new Anim(*ini.get("[Bonhomme]",numeroSection,
				    "anim trace de boue left",0),
			   0,
			   ini.getListSeparator()),
		 *new Anim(*ini.get("[Bonhomme]",numeroSection,
				    "anim trace de boue right",0),
			   0,
			   ini.getListSeparator()))),
   trou(0),
   stripContainerRef(&stripContainer),
   //
   stripUpWall(ini.getWithDefault("[Bonhomme]",numeroSection,
				  "strip up wall",0,*new String(DEFAULT_STRIP_UP_WALL))),
   stripDownWall(ini.getWithDefault("[Bonhomme]",numeroSection,
				    "strip down wall",0,*new String(DEFAULT_STRIP_DOWN_WALL))),
   stripCrame(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "strip crame",0,*new String(DEFAULT_STRIP_CRAME))),
   stripBrise(ini.getWithDefault("[Bonhomme]",numeroSection,
				 "strip brise",0,*new String(DEFAULT_STRIP_BRISE))),
   stripPicoteDuPainDur(ini.getWithDefault("[Bonhomme]",numeroSection,
					   "strip picote du pain dur",0,
					   *new String(DEFAULT_STRIP_PICOTE_DU_PAIN_DUR))),
   textPainterRef(&textPainter)


{

  int slowGame=ini.getWithDefault("[Bonhomme]",numeroSection,
				  "anim game slow",0,DEFAULT_ANIM_SLOW);

  animCenter=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					     "anim center",0),slowGame,ini.getListSeparator()));
  animUp=MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
						   "anim up",0),slowGame,ini.getListSeparator()));
  animDown=MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
						     "anim down",0),slowGame,ini.getListSeparator()));
  animLeft=MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
						     "anim left",0),slowGame,ini.getListSeparator()));
  animRight=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim right",0),slowGame,ini.getListSeparator()));
  
  animCrame=MoreMultiAnim::add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
						  "anim crame",0),slowGame,ini.getListSeparator()));
  animGris=MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
						     "anim gris",0),slowGame,ini.getListSeparator()));
  animRevit=MoreMultiAnim::add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
						  "anim revit",0),slowGame,ini.getListSeparator()));
  animBrise=MoreMultiAnim::add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
						  "anim brise",0),slowGame,ini.getListSeparator()));
  /*
  animResurrect=
    MoreMultiAnim::add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
					 "anim resurrect",0),slowGame,ini.getListSeparator()));
  */
  //temporairement, tant que c'est pas dessine
  Anim *auxAnim=new Anim(*ini.get("[Bonhomme]",numeroSection,
				  "anim brise",0),slowGame,ini.getListSeparator());
  auxAnim->revert();
  animResurrect=MoreMultiAnim::add(*auxAnim);

  animFantomeCenter=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					 "anim fantome center",0),
				 slowGame,ini.getListSeparator()));
  animFantomeUp=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim fantome up",0),slowGame,ini.getListSeparator()));
  animFantomeDown=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim fantome down",0),slowGame,ini.getListSeparator()));
  animFantomeLeft=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim fantome left",0),slowGame,ini.getListSeparator()));
  animFantomeRight=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim fantome right",0),
				     slowGame,ini.getListSeparator()));
  
  animChickenCenter=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim chicken center",0),
				     slowGame,ini.getListSeparator()));
  animChickenUp=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim chicken up",0),slowGame,ini.getListSeparator()));
  animChickenDown=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim chicken down",0),slowGame,ini.getListSeparator()));
  animChickenLeft=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim chicken left",0),slowGame,ini.getListSeparator()));
  animChickenRight=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim chicken right",0),
				 slowGame,ini.getListSeparator()));
  animChickenBrise=
    MoreMultiAnim::add(*new Anim(*ini.get("[Bonhomme]",numeroSection,
					  "anim chicken brise",0),
				 slowGame,ini.getListSeparator()));
  
  animTeleport=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim teleport",0),slowGame,ini.getListSeparator()));
  animInvincible=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim invincible",0),slowGame,ini.getListSeparator()));
  animSurpuissant=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim surpuissant",0),slowGame,ini.getListSeparator()));
  animWin95=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim win95",0),slowGame,ini.getListSeparator()));
  animTraverseMur=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim traverse mur",0),slowGame,ini.getListSeparator()));
  
  animTraceBoueUp=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim trace de boue up",0),
				     slowGame,ini.getListSeparator()));
  animTraceBoueDown=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim trace de boue down",0),
				     slowGame,ini.getListSeparator()));
  animTraceBoueLeft=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim trace de boue left",0),
				     slowGame,ini.getListSeparator()));
  animTraceBoueRight=
    MoreMultiAnim::add(*new LoopAnim(*ini.get("[Bonhomme]",numeroSection,
					      "anim trace de boue right",0),
				     slowGame,ini.getListSeparator()));
  //

  int channelReserved=sounderRef->reserveChannel();

  soundGetOptionId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
						"sound get option",
						"[Bonhomme]",numeroSection),
				   channelReserved);
  soundCrameId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
					    "sound crame",
					    "[Bonhomme]",numeroSection),
			       channelReserved);
  soundBriseId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
					    "sound brise",
					    "[Bonhomme]",numeroSection),
			       channelReserved);
  soundUpDownId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
					     "sound up down",
					     "[Bonhomme]",numeroSection),
				channelReserved);
  soundChickenDeadId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
						  "sound chicken dead",
						  "[Bonhomme]",numeroSection),
				     channelReserved);

  soundWonId=sounderRef->add(*ini.getList("[Bonhomme]",numeroSection,
					  "sound won",
					  "[Bonhomme]",numeroSection),
			     channelReserved);


  //
  
  sizeAction.j=ini.getWithDefault("[Bonhomme]",numeroSection,
				  "size action j",0,DEFAULT_SIZE_ACTION);
  sizeAction.i=ini.getWithDefault("[Bonhomme]",numeroSection,
				  "size action i",0,DEFAULT_SIZE_ACTION);

  int jDecalTolerantAction=ini.getWithDefault("[Bonhomme]",numeroSection,
					  "decal tolerant action j",0,
					  DEFAULT_DECAL_TOLERANT_ACTION);
  int iDecalTolerantAction=ini.getWithDefault("[Bonhomme]",numeroSection,
					      "decal tolerant action i",0,
					      DEFAULT_DECAL_TOLERANT_ACTION);

  int jDecalPosBombe=ini.getWithDefault("[Bonhomme]",numeroSection,
					"decal pose bombe j",0,
					DEFAULT_DECAL_POSE_BOMBE);
  int iDecalPosBombe=ini.getWithDefault("[Bonhomme]",numeroSection,
					"decal pose bombe i",0,
					DEFAULT_DECAL_POSE_BOMBE);

  //'pix' sont les points qui definissent la position du bonhomme
  //sur la grille,
  //ils ne s'incrementent uniquement quand leurs compteurs associes
  //leur commandent (passent a 0 ou a 'max')
  //pix = positions haut/gauche, bas/droite ,
  //                haut/gauche+tolerance, bas/droite-tolerance
  //                milieu (poser bombe)
  //                future position si deplacement haut/gauche,
  //                future position si deplacement bas/droite,
  //'decalPaint' decale le paint du sprite
  //   TODO : 'decalPaint' relatif a chaque sprite

  pix.init[0].j=0;
  pix.init[0].i=0;
  pix.init[1].j=sizeAction.j-1;
  pix.init[1].i=sizeAction.i-1;
  pix.init[2].j=jDecalTolerantAction;
  pix.init[2].i=iDecalTolerantAction;
  pix.init[3].j=pix.init[1].j-jDecalTolerantAction;
  pix.init[3].i=pix.init[1].i-iDecalTolerantAction;
  pix.init[4].j=jDecalPosBombe;
  pix.init[4].i=iDecalPosBombe;

  pix.init[5].j=pix.init[0].j-1;
  pix.init[5].i=pix.init[0].i-1;
  pix.init[6].j=pix.init[1].j+1;
  pix.init[6].i=pix.init[1].i+1;
  
  pix.decalPaint.j=ini.getWithDefault("[Bonhomme]",numeroSection,
				      "decal paint j",0,
				      DEFAULT_DECAL_PAINT);
  pix.decalPaint.i=ini.getWithDefault("[Bonhomme]",numeroSection,
				      "decal paint i",0,
				      DEFAULT_DECAL_PAINT);


  // positions reelles sur background
  pix.realPos.j=pix.realPos.i=0;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		      pix.realPos.i-pix.decalPaint.i);


  setPalette(pixelPalette);

  limit.pos.j=limit.pos.i=limit.size.j=limit.size.i=0;
  mult.j=mult.i=1;
  move(0,0);

  reinitAllValues();

}

Bonhomme20::~Bonhomme20() throw()
{
  delete artificialIntelligenceContainer;

  leaverList.place();
  while(leaverList.walk()) delete leaverList.get();
  //  if(trou)
  delete trou;
  delete boue;

}

void Bonhomme20::init(int j,int i,int jMult,int iMult,Grille& grille)
  throw(Exception)
  //reinit des valeurs !
{
  grilleRef=&grille;
  if(trou) delete trou;
  trou=new Trou(*iniRef,*sounderRef,*grilleRef);
  mult.j=jMult;
  mult.i=iMult;
  move(j,i);
  artificialIntelligenceContainer->init(*this,*grilleRef);
  reinitAllValues();
}

void Bonhomme20::move(int j,int i) throw()
{
  pix.realPos.j=j*mult.j;
  pix.realPos.i=i*mult.i;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		     pix.realPos.i-pix.decalPaint.i);
  for(int k=0;k<7;k++) {
    pix.cpt[k].j=pix.init[k].j;
    pix.cpt[k].i=pix.init[k].i;

    pix.pos[k].j=(pix.realPos.j+pix.init[k].j)/mult.j;
    pix.pos[k].i=(pix.realPos.i+pix.init[k].i)/mult.i;

    if(pix.realPos.j+pix.init[k].j<0) pix.pos[k].j--;
    if(pix.realPos.i+pix.init[k].i<0) pix.pos[k].i--;

    while(pix.cpt[k].j>=mult.j) pix.cpt[k].j-=mult.j;
    while(pix.cpt[k].i>=mult.i) pix.cpt[k].i-=mult.i;
    while(pix.cpt[k].j<0) pix.cpt[k].j+=mult.j;
    while(pix.cpt[k].i<0) pix.cpt[k].i+=mult.i;
  }
}

void Bonhomme20::reinitAllValues() throw()
{
  setSlow(0);

  etat=ETAT_NORMAL;
  specialFire=FIRE_NORMAL;
  nbBombe=initNbBombe;
  puissance=initPuissance;
  nbTeleport=0;
  dep=depNormal;

  cptTempsGris=0;

  invincible=invisible=surpuissant=traverseMur=
    superBombe=false;

  //%%  moving=false;
  movingDir=NO_DIR;
  firing0=firing1=false;
  nbPose=0;

  niv=1;

  blowUpDynamite=false;

  leaverList.place();while(leaverList.walk()) delete leaverList.get();
  
  MoreMultiAnim::reinit();
  CounterContainer::reset();

  if(trou) trou->reinit();
  boue->reinit();
}

void Bonhomme20::setLimit(int j,int i,int jSize,int iSize) throw()
{
  limit.pos.j=j;
  limit.pos.i=i;
  limit.size.j=jSize;
  limit.size.i=iSize;
}


void Bonhomme20::getSize(int& j,int& i) const throw()
{
  j=sizeAction.j;
  i=sizeAction.i;
}

void Bonhomme20::pixMoveUp() throw()
{
  if(pix.realPos.j<=limit.pos.j) return;

  for(int k=0;k<7;k++) {
    if(pix.cpt[k].j==0) {
      pix.cpt[k].j=mult.j-1;
      pix.pos[k].j--;
    }
    else pix.cpt[k].j--;
  }
  pix.realPos.j--;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		     pix.realPos.i-pix.decalPaint.i);
}
void Bonhomme20::pixMoveDown() throw()
{
  if(pix.realPos.j+sizeAction.j>=limit.pos.j+limit.size.j) return;

  for(int k=0;k<7;k++) {
    if(pix.cpt[k].j==mult.j-1) {
      pix.cpt[k].j=0;
      pix.pos[k].j++;
    }
    else pix.cpt[k].j++;
  }
  pix.realPos.j++;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		     pix.realPos.i-pix.decalPaint.i);
}
void Bonhomme20::pixMoveLeft() throw()
{
  if(pix.realPos.i<=limit.pos.i) return;

  for(int k=0;k<7;k++) {
    if(pix.cpt[k].i==0) {
      pix.cpt[k].i=mult.i-1;
      pix.pos[k].i--;
    }
    else pix.cpt[k].i--;
  }
  pix.realPos.i--;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		     pix.realPos.i-pix.decalPaint.i);
}
void Bonhomme20::pixMoveRight() throw()
{
  if(pix.realPos.i+sizeAction.i>=limit.pos.i+limit.size.i) return;

  for(int k=0;k<7;k++) {
    if(pix.cpt[k].i==mult.i-1) {
      pix.cpt[k].i=0;
      pix.pos[k].i++;
    }
    else pix.cpt[k].i++;
  }
  pix.realPos.i++;
  MoreMultiAnim::move(pix.realPos.j-pix.decalPaint.j,
		     pix.realPos.i-pix.decalPaint.i);
}


void Bonhomme20::moveUp() throw()
{
  //%%  moving=true;
  movingDir=DIR_UP;
  switch(etat) {
  case ETAT_NORMAL:
    MoreMultiAnim::on(animUp);
    break;
  case ETAT_CHICKEN:
    MoreMultiAnim::on(animChickenUp);
    break;
  case ETAT_FANTOME:
    MoreMultiAnim::on(animFantomeUp);
    break;
  case ETAT_TELEPORT:
    break;
  default:
    cptTempsGris--;
    //%%    moving=false;
    movingDir=NO_DIR;
    return;
  }
}


void Bonhomme20::privateMoveUp() throw()
{
  //  if(!grilleRef) return;
  bool up0=grilleRef->traversable(niv,pix.pos[5].j,pix.pos[0].i,traverseMur);
  bool up1=grilleRef->traversable(niv,pix.pos[5].j,pix.pos[1].i,traverseMur);
  if(up0 && up1) pixMoveUp();
  else if(up0) pixMoveLeft();
  else if(up1) pixMoveRight();
}

void Bonhomme20::moveDown() throw()
{
  //%%  moving=true;
  movingDir=DIR_DOWN;
  switch(etat) {
  case ETAT_NORMAL:
    MoreMultiAnim::on(animDown);
    break;
  case ETAT_CHICKEN:
    MoreMultiAnim::on(animChickenDown);
    break;
  case ETAT_FANTOME:
    MoreMultiAnim::on(animFantomeDown);
    break;
  case ETAT_TELEPORT:
    break;
  default:
    cptTempsGris--;
    //%    moving=false;
    movingDir=NO_DIR;
    return;
  }
}

void Bonhomme20::privateMoveDown() throw()
{
  //  if(!grilleRef) return;
  bool down0=grilleRef->traversable(niv,pix.pos[6].j,pix.pos[0].i,traverseMur);
  bool down1=grilleRef->traversable(niv,pix.pos[6].j,pix.pos[1].i,traverseMur);
  if(down0 && down1) pixMoveDown();
  else if(down0) pixMoveLeft();
  else if(down1) pixMoveRight();
}

void Bonhomme20::moveLeft() throw()
{
  //%%  moving=true;
  movingDir=DIR_LEFT;
  switch(etat) {
  case ETAT_NORMAL:
    MoreMultiAnim::on(animLeft);
    break;
  case ETAT_CHICKEN:
    MoreMultiAnim::on(animChickenLeft);
    break;
  case ETAT_FANTOME:
    MoreMultiAnim::on(animFantomeLeft);
    break;
  case ETAT_TELEPORT:
    break;
  default:
    cptTempsGris--;
    movingDir=NO_DIR;
    //%%    moving=false;
    return;
  }
}

void Bonhomme20::privateMoveLeft() throw()
{
  //  if(!grilleRef) return;
  bool left0=grilleRef->traversable(niv,pix.pos[0].j,pix.pos[5].i,traverseMur);
  bool left1=grilleRef->traversable(niv,pix.pos[1].j,pix.pos[5].i,traverseMur);
  if(left0 && left1) pixMoveLeft();
  else if(left0) pixMoveUp();
  else if(left1) pixMoveDown();
}

void Bonhomme20::moveRight() throw()
{
  //%%  moving=true;
  movingDir=DIR_RIGHT;
  switch(etat) {
  case ETAT_NORMAL:
    MoreMultiAnim::on(animRight);
    break;
  case ETAT_CHICKEN:
    MoreMultiAnim::on(animChickenRight);
    break;
  case ETAT_FANTOME:
    MoreMultiAnim::on(animFantomeRight);
    break;
  case ETAT_TELEPORT:
    break;
  default:
    cptTempsGris--;
    //%%    moving=false;
    movingDir=NO_DIR;
    return;
  }
}

void Bonhomme20::privateMoveRight() throw()
{
  //  if(!grilleRef) return;
  bool right0=
    grilleRef->traversable(niv,pix.pos[0].j,pix.pos[6].i,traverseMur);
  bool right1=
    grilleRef->traversable(niv,pix.pos[1].j,pix.pos[6].i,traverseMur);
  if(right0 && right1) pixMoveRight();
  else if(right0) pixMoveUp();
  else if(right1) pixMoveDown();
}

void Bonhomme20::doNotMove() throw()
{
  movingDir=NO_DIR;
  //  moving=false;
  switch(etat) {
  case ETAT_NORMAL:
    MoreMultiAnim::on(animCenter);
    break;
  case ETAT_CHICKEN:
    MoreMultiAnim::on(animChickenCenter);
    break;
  case ETAT_FANTOME:
    MoreMultiAnim::on(animFantomeCenter);
    break;
  default:
    break;
  }
}

void Bonhomme20::doNotFire0() throw()
{
  firing0=false;
}

void Bonhomme20::doNotFire1() throw()
{
  firing1=false;
}

void Bonhomme20::privateFire0() throw()
{
  blowUpDynamite=false;
  //  if(!grilleRef) return;
  if(etat!=ETAT_NORMAL && etat!=ETAT_FANTOME) return;
  if(etat!=ETAT_FANTOME && nbTeleport>0) beginOfCount(COUNT_TELEPORT);
  else {
    CaseId::Id c=CaseId::BOMBE;
    switch(specialFire) {
    case FIRE_NORMAL:
      if(nbPose>=nbBombe) return;
      c=CaseId::BOMBE;
      break;
    case FIRE_DYNAMITE:
      if(nbPose>=nbBombe) return;
      c=CaseId::DYNAMITE;
      break;
    case FIRE_MINE:
      c=CaseId::MINE;
      break;
    case FIRE_MUR:
      c=CaseId::MUR_DESTRUCTIBLE;
      break;
    default:
      break;
    }
    int auxNiv;
    if(specialFire==FIRE_MUR) auxNiv=0;
    else if(grilleRef->high(0,pix.pos[4].j,pix.pos[4].i)) auxNiv=1;
    else auxNiv=0;

    bool ok=grilleRef->add(c,auxNiv,
			   pix.pos[4].j,
			   pix.pos[4].i,
			   this);
    if(ok)
      switch(specialFire) {
      case FIRE_DYNAMITE:
      case FIRE_NORMAL:
	nbPose++;
	break;
      default:
	break;
      }
    
  }
}

void Bonhomme20::privateFire1() throw()
{
  //  if(!grilleRef) return;
  blowUpDynamite=true;
  if(etat==ETAT_FANTOME) {
    //    if(trou)
    trou->fire();
    return;
  }
  if(etat!=ETAT_NORMAL) return;

  if(nbPose>=nbBombe) return;
  int auxNiv;
  if(grilleRef->high(0,pix.pos[4].j,pix.pos[4].i)) auxNiv=1;
  else auxNiv=0;

  if(grilleRef->add(CaseId::BOMBE,
		    auxNiv,
		    pix.pos[4].j,
		    pix.pos[4].i,
		    this)) nbPose++;
}

void Bonhomme20::fire0() throw()
{
  firing0=true;
}

void Bonhomme20::fire1() throw()
{
  firing1=true;
}

Bonhomme20::DirectionId Bonhomme20::getDirection() const throw()
{
  return movingDir;
}

void Bonhomme20::decNbPose() throw()
{
  if(nbPose>0) nbPose--;
}


//

void Bonhomme20::incNbBombe() throw()
{
  nbBombe++;
}

void Bonhomme20::incPuissance() throw()
{
  puissance++;
}

void Bonhomme20::incNbTeleport() throw()
{
  nbTeleport++;
}

void Bonhomme20::setTraverseMur() throw()
{
  beginOfCount(COUNT_TRAVERSE_MUR);
}

void Bonhomme20::setSurpuissant() throw()
{
  beginOfCount(COUNT_SURPUISSANT);
}

void Bonhomme20::setFireDynamite() throw()
{
  specialFire=FIRE_DYNAMITE;
  nbTeleport=0;
}

void Bonhomme20::setFireMine() throw()
{
  specialFire=FIRE_MINE;
  nbTeleport=0;
}

void Bonhomme20::setFirePoseMur() throw()
{
  specialFire=FIRE_MUR;
  nbTeleport=0;
}

void Bonhomme20::setChicken() throw()
{
  switch(etat) {
  case ETAT_NORMAL:
  case ETAT_CRAME:
  case ETAT_BRISE:
  case ETAT_REVIT:
  case ETAT_GRIS:
  case ETAT_CHICKEN:
  case ETAT_CHICKEN_BRISE:
    beginOfCount(COUNT_CHICKEN);
    break;
  default:
    break;
  }
}

void Bonhomme20::setFeuCul() throw()
{
  beginOfCount(COUNT_FEU_CUL);
}

void Bonhomme20::setInvisible() throw()
{
  beginOfCount(COUNT_INVISIBLE);
}

void Bonhomme20::setInvincible() throw()
{
  beginOfCount(COUNT_INVINCIBLE);
}

void Bonhomme20::setSuperBombe() throw()
{
  superBombe=true;
}//bombe qui traverse les murs

void Bonhomme20::setWin95() throw()
{
  beginOfCount(COUNT_WIN95);
}
  
bool Bonhomme20::setUpNiv() throw()
{
  if(niv==1) return true;
  sounderRef->play(soundUpDownId);
  niv=1;
  printStrip(*stripUpWall);
  if(etat==ETAT_CHICKEN) 
    printStrip(*stripPicoteDuPainDur);
  return false;
}//renvoie true si pas deja en niv==1

bool Bonhomme20::setDownNiv() throw()
{
  if(niv==0) return true;
  sounderRef->play(soundUpDownId);
  niv=0;
  printStrip(*stripDownWall);
  return false;
}

void Bonhomme20::cramerIfVivant() throw()
{
  if(etat==ETAT_NORMAL || etat==ETAT_CHICKEN || etat==ETAT_REVIT) cramer();
}

void Bonhomme20::cramer() throw()
{
  if(invincible || grilleRef->getCoeur()->isOn(*this)) return;

  //suppr win95
  MoreMultiAnim::supprMore(animWin95);
  setSlow(0);

  switch(etat) {
  case ETAT_NORMAL:
  case ETAT_CHICKEN:
  case ETAT_REVIT:
    cptTempsGris=tempsGris;
    MoreMultiAnim::on(animCrame);
    etat=ETAT_CRAME;
    movingDir=NO_DIR;
    sounderRef->play(soundCrameId);
    printStrip(*stripCrame);
    break;
  case ETAT_GRIS:
    MoreMultiAnim::on(animBrise);
    etat=ETAT_BRISE;
    movingDir=NO_DIR;
    sounderRef->play(soundBriseId);
    printStrip(*stripBrise);
    break;
  default:
    break;
  }
}

bool Bonhomme20::setEtatGrisIfMove() throw()
{
  if(etat==ETAT_NORMAL && movingDir!=NO_DIR) {
    MoreMultiAnim::on(animGris);
    etat=ETAT_GRIS;
    cptTempsGris=tempsGris;
    movingDir=NO_DIR;
    //%%    moving=false;
    return true;
  }
  return false;
}//renvoie true si a bouge


bool Bonhomme20::resurrect() throw()
{
  if(etat==ETAT_FANTOME) {
    MoreMultiAnim::on(animResurrect);
    etat=ETAT_RESURRECT;
    return true;
  }
  return false;
}//renvoie true si avant, etait mort



void Bonhomme20::getPosTolerantAction(int& niv,int j[4],int i[4]) const throw()
  //position tolerance = la flamme tue
{
  niv=this->niv;
  j[0]=pix.pos[2].j;
  i[0]=pix.pos[2].i;
  j[1]=pix.pos[2].j;
  i[1]=pix.pos[3].i;
  j[2]=pix.pos[3].j;
  i[2]=pix.pos[3].i;
  j[3]=pix.pos[3].j;
  i[3]=pix.pos[2].i;
}

void Bonhomme20::getPosAction(int& niv,int j[4],int i[4]) const throw()
  //position non tolerance = la flamme ne tue pas, on prend les options
{
  niv=this->niv;
  j[0]=pix.pos[0].j;
  i[0]=pix.pos[0].i;
  j[1]=pix.pos[0].j;
  i[1]=pix.pos[1].i;
  j[2]=pix.pos[1].j;
  i[2]=pix.pos[1].i;
  j[3]=pix.pos[1].j;
  i[3]=pix.pos[0].i;
}

//
void Bonhomme20::reinit() const throw()
{
  //rien
}

//

void Bonhomme20::beginOfCount(int value) throw()
{
  switch(value) {

  case COUNT_INVISIBLE:
    CounterContainer::add(tempsInvisible,value);
    invisible=true;
    break;
  case COUNT_INVINCIBLE:
    CounterContainer::add(tempsInvincible,value);
    invincible=true;
    MoreMultiAnim::onMore(animInvincible);
    addLeaver(CaseId::ETOILE);
    break;
  case COUNT_TELEPORT:
    CounterContainer::add(tempsTeleport,value);
    setUpNiv();
    nbTeleport--;
    etat=ETAT_TELEPORT;
    dep=depTeleport;
    MoreMultiAnim::on(animTeleport);
    break;
  case COUNT_TRAVERSE_MUR:
    CounterContainer::add(tempsTraverseMur,value);
    traverseMur=true;
    MoreMultiAnim::onMore(animTraverseMur);
    break;
  case COUNT_SURPUISSANT:
    CounterContainer::add(tempsSurpuissant,value);
    if(!surpuissant) {
      nbBombeSaveSurpuissant=nbBombe;
      nbBombe=INT_MAX;
    }
    surpuissant=true;
    MoreMultiAnim::onMore(animSurpuissant);
    break;
  case COUNT_WIN95:
    CounterContainer::add(tempsWin95,value);
    MoreMultiAnim::onMore(animWin95);
    setSlow(ralentissementWin95);
    break;
  case COUNT_FEU_CUL:
    CounterContainer::add(tempsFeuCul,value);
    addLeaver(CaseId::FLAMME_CENTER);
    break;
  case COUNT_SPEED:
    CounterContainer::add(tempsSpeed,value);
    dep=depChicken;
    break;
  case COUNT_CHICKEN:
    CounterContainer::add(tempsChicken,value);
    dep=depChicken;
    MoreMultiAnim::on(animChickenCenter);
    etat=ETAT_CHICKEN;

    //suppr win95
    MoreMultiAnim::supprMore(animWin95);
    setSlow(0);

    break;
  }
}

void Bonhomme20::endOfCount(int value) throw()
{
  switch(value) {

  case COUNT_INVISIBLE:
    invisible=false;
    break;
  case COUNT_INVINCIBLE:
    invincible=false;
    MoreMultiAnim::supprMore(animInvincible);
    supprLeaver(CaseId::ETOILE);
    break;
  case COUNT_TELEPORT:
    etat=ETAT_NORMAL;
    dep=depNormal;
    MoreMultiAnim::on(animCenter);
    break;
  case COUNT_TRAVERSE_MUR:
    traverseMur=false;
    MoreMultiAnim::supprMore(animTraverseMur);
    break;
  case COUNT_SURPUISSANT:
    nbBombe=nbBombeSaveSurpuissant;
    surpuissant=false;
    MoreMultiAnim::supprMore(animSurpuissant);
    break;
  case COUNT_WIN95:
    MoreMultiAnim::supprMore(animWin95);
    setSlow(0);
    break;
  case COUNT_FEU_CUL:
    supprLeaver(CaseId::FLAMME_CENTER);
    break;
  case COUNT_SPEED:
    dep=depNormal;
    break;
  case COUNT_CHICKEN:
    if(etat==ETAT_CHICKEN) {
      dep=depNormal;
      MoreMultiAnim::on(animCenter);
      etat=ETAT_NORMAL;
    }
    break;
  }
}

//

void Bonhomme20::addLeaver(CaseId::Id c) throw()
{
  //  if(!grilleRef) return;
  leaverList.place();
  while(leaverList.walk()) if(leaverList.get()->getCase()==c) return;
  leaverList.put(new Leaver(*this,c,*grilleRef));
}

void Bonhomme20::supprLeaver(CaseId::Id c) throw()
{
  //  if(!grilleRef) return;
  leaverList.place();
  while(leaverList.walk())
    if(leaverList.get()->getCase()==c) {
      delete leaverList.get();
      leaverList.suppr();
      return;
    }
}

//

bool Bonhomme20::isChicken() const throw()
{
  if(etat==ETAT_CHICKEN) return true;
  return false;
}

Bonhomme20::IdEtat Bonhomme20::getEtat() const throw()
{
  return etat;
}

void Bonhomme20::ecraserChicken() throw()
{
  etat=ETAT_CHICKEN_BRISE;
  MoreMultiAnim::on(animChickenBrise);
  movingDir=NO_DIR;
  //%%  moving=false;
  sounderRef->play(soundChickenDeadId);
}

void Bonhomme20::testerEcrasementChicken() throw()
{
  List<Bonhomme*> *l=getBonhomme()->getPlayingAll();

  l->place();
  while(l->walk()) {
    Bonhomme20 *b=l->get()->getBonhomme20();
    if(!b->isChicken()) {
      int niv,j[4],i[4];
      b->getPosAction(niv,j,i);
      for(int k=0;k<2;k++)
	if(pix.pos[k].j>=j[0] && pix.pos[k].j<=j[2] &&
	   pix.pos[k].i>=i[0] && pix.pos[k].i<=i[2]) {
	  ecraserChicken();
	  return;
	}
    }
  }
}

//

void Bonhomme20::changeArtificialIntelligence() throw()
{
  artificialIntelligenceContainer->next();
}

const String *Bonhomme20::getArtificialIntelligenceId() throw()
{
  return artificialIntelligenceContainer->getId();
}

//

int Bonhomme20::virtualWalk() throw()
{
  artificialIntelligenceContainer->walk();

  boue->walk();
  //  if(trou)
  trou->walk();

  if(!MoreMultiAnim::virtualWalk()) {
    switch(etat) {
    case ETAT_CRAME:
      MoreMultiAnim::on(animGris);
      etat=ETAT_GRIS;
      break;
    case ETAT_REVIT:
      MoreMultiAnim::on(animCenter);
      etat=ETAT_NORMAL;
      break;
    case ETAT_BRISE:
    case ETAT_CHICKEN_BRISE:
      MoreMultiAnim::on(animFantomeCenter);
      etat=ETAT_FANTOME;
      dep=depNormal;
      setUpNiv();
      specialFire=FIRE_NORMAL;
      break;
    case ETAT_RESURRECT:
      MoreMultiAnim::on(animGris);
      etat=ETAT_GRIS;
      break;
    default:
      break;
    }
  }

  if(isChicken()) testerEcrasementChicken();

  switch(etat) {
  case ETAT_NORMAL:
  case ETAT_CHICKEN:
    if(niv==1)
      if(!grilleRef->high(0,pix.pos[0].j,pix.pos[0].i) &&
	 !grilleRef->high(0,pix.pos[1].j,pix.pos[0].i) &&
	 !grilleRef->high(0,pix.pos[0].j,pix.pos[1].i) &&
	 !grilleRef->high(0,pix.pos[1].j,pix.pos[1].i)) setDownNiv();
    break;
  default:
    break;
  }

  //

  CounterContainer::inc();

  //

  leaverList.place();
  while(leaverList.walk()) leaverList.get()->walk();

  //

  if(etat==ETAT_GRIS) {
    if(cptTempsGris<=0) {
      MoreMultiAnim::on(animRevit);
      etat=ETAT_REVIT;
    }
  }

  if(firing0) privateFire0();
  if(firing1) privateFire1();

  switch(movingDir) {
  case DIR_UP:
    for(int k=0;k<dep;k++) privateMoveUp();
    break;
  case DIR_DOWN:
    for(int k=0;k<dep;k++) privateMoveDown();
    break;
  case DIR_LEFT:
    for(int k=0;k<dep;k++) privateMoveLeft();
    break;
  case DIR_RIGHT:
    for(int k=0;k<dep;k++) privateMoveRight();
    break;
  case NO_DIR:
    break;
  }

  return 1;
}

bool Bonhomme20::capableToTakeOption(bool playSound) throw()
{
  if(etat==ETAT_NORMAL) {
    if(playSound) sounderRef->play(soundGetOptionId);
    return true;
  }
  return false;
}
 
int Bonhomme20::getPuissance() const throw()
{
  if(surpuissant) return INT_MAX;
  return puissance;
}
bool Bonhomme20::isSuperBombe() const throw()
{
  if(surpuissant) return true;
  return superBombe;
}


bool Bonhomme20::haveToBlowUpDynamite() const throw()
{
  return blowUpDynamite;
}

Bonhomme *Bonhomme20::getBonhomme() throw()
{
  return bonhommeRef;
}

const Bonhomme *Bonhomme20::getBonhomme() const throw()
{
  return bonhommeRef;
}

int Bonhomme20::getTempsCoeur() const throw()
{
  return tempsCoeur;
}

void Bonhomme20::setBoueColleeAuxPieds() throw()
{
  if(etat==ETAT_NORMAL) boue->set();
}

Trou *Bonhomme20::getTrou() throw()
{
  return trou;
}

void Bonhomme20::paint(WindowPipe& windowPipe) throw()
{
  boue->paint(windowPipe);
  //%%  if(trou)
  //%%  trou->paint(windowPipe);
  if(!invisible) MoreMultiAnim::paint(windowPipe);

  /*pour voir les points d'action:

  Array3<Pixel> a(2,1,1);
  a[1]=0;
  a[0]=16;
  for(int k=0;k<2;k++)
  windowPipe.put(a,
  pix.pos[k].j*mult.j,//+pix.cpt[k].j,
  pix.pos[k].i*mult.i);//+pix.cpt[k].i);
  a[0]=0;
  for(int k=2;k<4;k++)
  windowPipe.put(a,
  pix.pos[k].j*mult.j,//+pix.cpt[k].j,
  pix.pos[k].i*mult.i);//+pix.cpt[k].i);
  a[0]=52;
  for(int k=5;k<7;k++)
  windowPipe.put(a,
  pix.pos[k].j*mult.j,//+pix.cpt[k].j,
  pix.pos[k].i*mult.i);//+pix.cpt[k].i);
  
  a[0]=52;
  windowPipe.put(a,
		 pix.pos[4].j*mult.j,//+pix.cpt[k].j,
		 pix.pos[4].i*mult.i);//+pix.cpt[k].i);
  */
}

void Bonhomme20::playSoundWon() throw()
{
  sounderRef->play(soundWonId);
}


void Bonhomme20::setSpeed() throw()
{
  beginOfCount(COUNT_SPEED);
}

Pixel Bonhomme20::getNormalColor() const throw()
{
  return getBonhomme()->getColor();
}

void Bonhomme20::printStrip(const String& s) throw()
{
  stripContainerRef->add(s,getNormalColor());
}

void Bonhomme20::printStrip(PreStrip::Id id) throw()
{
  stripContainerRef->add(id,getNormalColor());
}

void Bonhomme20::paintText(WindowPipe& windowPipe,const String& text) throw()
{
  textPainterRef->paint(windowPipe,
			pos.j-textPainterRef->nbLine(),
			pos.i,
			text,
			getNormalColor());//true??NO
}




void Bonhomme20::paintNight(WindowPipe&windowPipe)throw()
{
  boue->paint(windowPipe);
  if(!invisible) MoreMultiAnim::paintNight(windowPipe);
}
