/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Bonhomme60;

#ifndef __Bonhomme60_h__
#define __Bonhomme60_h__

#include "List.h"
#include "MultiAnim.h"
#include "Ini.h"
#include "Sounder.h"
#include "Exception.h"
#include "Tji.h"
#include "LoopAnim.h"
#include "PageableContainer.h"
#include "TextPainter.h"
#include "Bonhomme.h"

/**
   The character when choosing. It manages the normal four animations (loop when not chosen, not loop when starting, not loop when crashed on wall, loop after), plus other and other animations... (see the stars on witch !)
   @see Bonhomme
   @see PageBonhomme
   @author David FAUTHOUX
 */
class Bonhomme60:public MultiAnim {
 private:
  ///Not callable
  void move(int j,int i) throw(); //non appelable
 protected:
  ///Reference
  Bonhomme *const bonhommeRef;

  ///Reference to play sounds
  Sounder *const sounderRef;

  ///
  int soundDepartId;
  ///
  int soundEcrasementId;

  ///Used to paint the name
  TextPainter *const textPainterRef;

  ///Id to use with MultiAnim : loop when not chosen
  int animPrincipale;
  ///Not loop when starting
  int animDepart;
  ///Not loop when crashing
  int animEcrasement;
  ///Loop after
  int animArrivee;

  ///To manage pretty useless animations
  bool moreActivated;
  ///
  PageableContainer animMore;
  ///
  PageableContainer loopAnimMore;
  ///
  List< Anim * > allMore;

  ///
  Tji posAnimDepart;
  ///
  Tji posAnimArrivee;
  ///Crashed ?
  bool ecra;


  ///
  int virtualWalk() throw();


 public:
  ///
  Bonhomme60(Ini& ini,int numeroSection,Bonhomme& bonhomme,
	     TextPainter& textPainter,
	     Sounder& sounder,
	     const PixelPalette& pixelPalette) throw(Exception);
  ///
  virtual ~Bonhomme60() throw();

  ///
  void reinit() throw();

  ///Crash it
  void ecraser() throw();

  ///Is crashed ?
  bool isEcrase() const throw();

  ///Activate the pretty useless animations on Bonhomme60
  void activateMore() throw();
  ///Unactivate the pretty useless animations on Bonhomme60
  void unactivateMore() throw();

  ///
  void paint(WindowPipe& windowPipe) throw();

  ///Reference to the container
  Bonhomme *getBonhomme() throw();
  ///
  const Bonhomme *getBonhomme() const throw();
  ///
  Pixel getNormalColor() const throw();


  ///
  void paintText(WindowPipe& windowPipe,const String& text) throw();
};

#endif

