/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <limits.h>
#include <iostream.h>
#include "BonhommeContainer.h"
#include "error.h"

#define DEFAULT_NB_PLAYER_IN_TEAM 3

#define GREY toPixel(maxGreyValue,maxGreyValue,maxGreyValue,0)

BonhommeContainer::BonhommeContainer(Ini& ini,
				     Sounder& sounder,
				     TextPainter& textPainter,
				     StripContainer& stripContainer)
  throw(Exception)
  :iniRef(&ini)
{
  cout<<"Loading Bonhommes and Players...";cout.flush();

  int auxTeamSet=ini.getWithDefault("[Global]",0,"team activate",0,0);
  if(auxTeamSet) teamSet=true;
  else teamSet=false;

  amountPlayerInTeam=ini.getWithDefault("[Global]",0,
					"amount Player in each team",0,
					DEFAULT_NB_PLAYER_IN_TEAM);

  if(amountPlayerInTeam<=0) amountPlayerInTeam=DEFAULT_NB_PLAYER_IN_TEAM;

  //

  //initialise les bonhommes
  int cptB=0;
  while(ini.isPresentSection("[Bonhomme]",cptB)) {
    try {
      cout<<"*";cout.flush();
      Bonhomme *b=new Bonhomme(ini,cptB,sounder,textPainter,stripContainer);
      listBonhomme.put(b);
    } catch(Exception) {}
    cptB++;
  }

  //  if(listBonhomme.empty()) throw Exception(ERROR_NO_VALID_BONHOMME);

  listBonhomme.place();
  cptB=0;
  while(listBonhomme.walk()) {
    listBonhommeAlone.put(listBonhomme.get());
    cptB++;
  }

  //color

  listBonhomme.place();
  while(listBonhomme.walk()) 
    listBonhomme.get()->setColor(GREY);

  
  //initialise les players
  
  int cptJ=0;
  while(cptJ<cptB) {
    Player *pl=new Player(ini,cptJ);
    listPlayer.put(pl);
    const String *refBonhomme=ini.getWithDefault("[Player]",cptJ,
					   "Bonhomme",0,"");
    pl->setBonhomme(*findBonhomme(*refBonhomme));
    cptJ++;
  }

  maxNbPlayer=cptB;

  int auxNbPlayer=ini.getWithDefault("[Global]",0,
				     "nombre de Players actuel",0,1);

  if(auxNbPlayer<1) auxNbPlayer=1;
  if(auxNbPlayer>maxNbPlayer) auxNbPlayer=maxNbPlayer;

  nbPlayer=0;
  for(int t=0;t<auxNbPlayer;t++) incNbPlayer();

  //
  cout<<"ok\n";cout.flush();
}

BonhommeContainer::~BonhommeContainer() throw()
{
  iniRef->set("[Global]",0,"nombre de Players actuel",0,nbPlayer);
  if(teamSet) iniRef->set("[Global]",0,"team activate",0,1);
  else iniRef->set("[Global]",0,"team activate",0,0);

  listBonhomme.place();
  while(listBonhomme.walk()) delete listBonhomme.get();
  listPlayer.place();
  while(listPlayer.walk()) delete listPlayer.get();
}

/*%%
void BonhommeContainer::alignLeftToRight() throw()
{
  List< Bonhomme *> list;
  listBonhommeAlone.place();
  while(listBonhommeAlone.walk()) {
    list.put(listBonhommeAlone.get());
    listBonhommeAlone.suppr();
  }

  list.place();
  while(list.walk()) {
    if(listBonhommeAlone.empty()) listBonhommeAlone.put(list.get());
    else {
      int j,i;
      list.get()->getBonhomme60()->getPos(j,i);
      int lj,li;
      listBonhommeAlone.place();
      bool placed=false;
      while(!placed && listBonhommeAlone.walk()) {
	listBonhommeAlone.get()->getBonhomme60()->getPos(lj,li);
	if(i<li) {
	  listBonhommeAlone.put(list.get());
	  placed=true;
	}
      }
      if(!placed) listBonhommeAlone.put(list.get());
    }
  }
}
*/

Bonhomme *BonhommeContainer::findBonhomme(const String& b) throw()
{
  listBonhomme.place();
  while(listBonhomme.walk())
    if(*(listBonhomme.get()->getId())==b)
      return listBonhomme.get();
  listBonhomme.place();
  listBonhomme.walk();
  return listBonhomme.get();
}

void BonhommeContainer::checkBonhomme(Player& p) throw()
{
  Bonhomme *b=p.getBonhomme();
  listBonhommeAlone.place();
  while(listBonhommeAlone.walk())
    if(listBonhommeAlone.get()==b) {
      listBonhommeAlone.suppr();
      p.setBonhomme(*b);
      b->getBonhomme60()->activateMore();
      return;
    }
  listBonhommeAlone.place();
  listBonhommeAlone.walk();
  b=listBonhommeAlone.get();
  p.setBonhomme(*b);
  b->getBonhomme60()->activateMore();

  listBonhommeAlone.suppr();
}


void BonhommeContainer::incNbPlayer() throw()
{
  if(nbPlayer>=maxNbPlayer) return;
  listPlayer.place();
  listPlayer.walk();
  for(int i=0;i<nbPlayer;i++) {
    listPlayer.walk();
  }
  nbPlayer++;
  Player *p=listPlayer.get();
  listPlayerPlaying.placeCurrentOnHead();
  listPlayerPlaying.put(p);
  listPlayerPlaying.placeCurrentOnHead();

  checkBonhomme(*p);

  //
  setBonhommePlaying();
}

void BonhommeContainer::decNbPlayer() throw()
{
  if(nbPlayer<=1) return;
  nbPlayer--;
  listPlayerPlaying.placeCurrentOnHead();

  Bonhomme *b=listPlayerPlaying.get()->getBonhomme();
  b->setColor(GREY);
  b->getBonhomme60()->unactivateMore();
  listBonhommeAlone.placeCurrentOnHead();
  listBonhommeAlone.put(b);
  listPlayerPlaying.suppr();
  setBonhommePlaying();
}

void BonhommeContainer::setBonhommePlaying() throw()
{
  listBonhommePlaying.place();
  while(listBonhommePlaying.walk()) listBonhommePlaying.suppr();

  listPlayerPlaying.place();
  while(listPlayerPlaying.walk()) {
    listBonhommePlaying.put(listPlayerPlaying.get()->getBonhomme());
  }

  setPlayingAdded();
}

void BonhommeContainer::setPlayingAdded() throw()
{
  List<Bonhomme*> l=listBonhommePlaying;

  //  int max=(maxNbPlayer+amountPlayerInTeam-1)/amountPlayerInTeam;
  int max=(nbPlayer+amountPlayerInTeam-1)/amountPlayerInTeam;

  int cpt;
  int ici=0;

  listBonhommePlaying.place();
  while(listBonhommePlaying.walk()) {
    listBonhommePlaying.get()->supprAllPlayingAdded();
    l.place();
    cpt=0;
    while(l.walk()) {
      Bonhomme *b=listBonhommePlaying.get();
      if(!teamSet) {
	if(b==l.get()) b->addPlaying(*(l.get()),true);
	else b->addPlaying(*(l.get()),false);
      }
      else {
	if(cpt==ici) {
	  b->addPlaying(*(l.get()),true);
	} else {
	  b->addPlaying(*(l.get()),false);
	}
	cpt++;
	if(cpt==max) cpt=0; 
      }
    }
    ici++;
    if(ici==max) ici=0; 
  }

  listPlayerPlaying.place();
  while(listPlayerPlaying.walk())
    listPlayerPlaying.get()->setColorToBonhomme();
}

int BonhommeContainer::getNbPlayer() const throw()
{
  return nbPlayer;
}

int BonhommeContainer::getMaxNbPlayer() const throw()
{
  return maxNbPlayer;
}


List<Bonhomme*> *BonhommeContainer::getAllBonhomme() throw()
{
  return &listBonhomme;
}

List<Bonhomme*> *BonhommeContainer::getBonhommePlaying() throw()
{
  return &listBonhommePlaying;
}

List<Player*> *BonhommeContainer::getAllPlayer() throw()
{
  return &listPlayer;
}

List<Player*> *BonhommeContainer::getPlayerPlaying() throw()
{
  return &listPlayerPlaying;
}

void BonhommeContainer::setOrUnsetTeam() throw()
{
  if(teamSet) {
    teamSet=false;
    setPlayingAdded();
  }
  else {
    teamSet=true;
    setPlayingAdded();
  }
}

bool BonhommeContainer::isTeamSet() const throw()
{
  return teamSet;
}


bool BonhommeContainer::eachPlayerHasBonhomme() throw()
{
  listPlayerPlaying.place();
  while(listPlayerPlaying.walk())
    if(!listPlayerPlaying.get()->getBonhomme()->getBonhomme60()->isEcrase())
      return false;
  return true;
}

Bonhomme *BonhommeContainer::findRight(Bonhomme *b) throw()
{
  Bonhomme *bCur=0;
  int j,i,cur;
  b->getBonhomme60()->getPos(j,i);

  cur=INT_MAX;

  listBonhommeAlone.place();
  while(listBonhommeAlone.walk()) {
    int lj,li;
    listBonhommeAlone.get()->getBonhomme60()->getPos(lj,li);
    if(li<cur && li>i) {
      cur=li;
      bCur=listBonhommeAlone.get();
    }
  }
  if(bCur) {
    listBonhommeAlone.place();
    while(listBonhommeAlone.walk())
      if(listBonhommeAlone.get()==bCur) {
	listBonhommeAlone.suppr();
	return bCur;
      }
  }
  bCur=listBonhommeAlone.get();
  listBonhommeAlone.suppr();
  return bCur;
}

Bonhomme *BonhommeContainer::findLeft(Bonhomme *b) throw()
{
  Bonhomme *bCur=0;
  int j,i,cur;
  b->getBonhomme60()->getPos(j,i);

  cur=INT_MIN;

  listBonhommeAlone.placeCurrentOnHead();
  do {
    int lj,li;
    listBonhommeAlone.get()->getBonhomme60()->getPos(lj,li);
    if(li>cur && li<i) {
      cur=li;
      bCur=listBonhommeAlone.get();
    }
  } while(listBonhommeAlone.walkHeadToQueue());
  if(bCur) {
    listBonhommeAlone.place();
    while(listBonhommeAlone.walk())
      if(listBonhommeAlone.get()==bCur) {
	listBonhommeAlone.suppr();
	return bCur;
      }
  }
  listBonhommeAlone.walkQueueToHead();
  bCur=listBonhommeAlone.get();
  listBonhommeAlone.suppr();
  return bCur;
}


void BonhommeContainer::keyAction(const Key& key) throw()
{
  if(key.isUp()) return;
  listPlayerPlaying.place();
  while(listPlayerPlaying.walk()) {
    Player *p=listPlayerPlaying.get();
    if(!p->getBonhomme()->getBonhomme60()->isEcrase()) {
      if(key.getCode()==p->getKeyRight()) {
	Bonhomme *b=p->getBonhomme();
	b->setColor(GREY);
	b->getBonhomme60()->unactivateMore();
	listBonhommeAlone.placeCurrentOnHead();
	listBonhommeAlone.put(b);

	//	listBonhommeAlone.place();
	//	listBonhommeAlone.walk();
	//	listBonhommeAlone.get();
	b=findRight(b);

	p->setBonhomme(*b);
	b->getBonhomme60()->activateMore();
	//%%	listBonhommeAlone.suppr();
	setBonhommePlaying();
      }
      if(key.getCode()==p->getKeyLeft()) {
	Bonhomme *b=p->getBonhomme();
	b->setColor(GREY);
	b->getBonhomme60()->unactivateMore();
	listBonhommeAlone.place();
	//%%!!	listBonhommeAlone.walk();
	listBonhommeAlone.put(b);

	//	listBonhommeAlone.placeCurrentOnHead();
	//	listBonhommeAlone.get();
	b=findLeft(b);

	p->setBonhomme(*b);
	b->getBonhomme60()->activateMore();
	//%%	listBonhommeAlone.suppr();
	setBonhommePlaying();
      }
      if(key.getCode()==p->getKeyFire0()) {
	p->getBonhomme()->getBonhomme60()->ecraser();
      }
      if(key.getCode()==p->getKeyFire1()) {
	p->getBonhomme()->getBonhomme20()->changeArtificialIntelligence();
      }
    }
  }
}
