/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Boue.h"
#include "Bonhomme20.h"
#include "error.h"

Boue::Boue(const Bonhomme20& b,
	   Anim& animUp,Anim& animDown,
	   Anim& animLeft,Anim& animRight) throw()
  :bonhommeRef(&b),
   aUp(add(animUp)),aDown(add(animDown)),
   aLeft(add(animLeft)),aRight(add(animRight)),
   toBePaint(false),finished(true)
{
  setUnder();
}

Boue::~Boue() throw()
{
}

void Boue::set() throw()
{
  int niv,j[4],i[4];
  bonhommeRef->getPosTolerantAction(niv,j,i);
  for(int k=0;k<4;k++) {
    pos[k].j=j[k];
    pos[k].i=i[k];
  }
  reinit();
  finished=false;
  toBePaint=false;
}

void Boue::on(int id) throw (Exception)
{
  if(id<0 || id>=length) throw Exception(ERROR_MULTIANIM_INVALID_ID);
  currentAnim=id;
  activeAnim=tab[id];
  //NON:  activeAnim->reinit();
}

int Boue::virtualWalk() throw()
{
  if(finished) return 1;

  if(!bonhommeRef->getEtat()==Bonhomme20::ETAT_NORMAL) {
    toBePaint=false;
    return 1;
  }

  int niv,j[4],i[4];
  bonhommeRef->getPosTolerantAction(niv,j,i);
  for(int k=0;k<4;k++) {
    bool doIt=true;
    int t=0;
    while(t<4 && doIt) {
      if(j[k]==pos[t].j && i[k]==pos[t].i) doIt=false;
      t++;
    }
    if(doIt) { //la nouvelle case differente de ttes les anciennes
      switch(bonhommeRef->getDirection()) {
      case Bonhomme20::DIR_UP:
	on(aUp);
	break;
      case Bonhomme20::DIR_DOWN:
	on(aDown);
	break;
      case Bonhomme20::DIR_LEFT:
	on(aLeft);
	break;
      case Bonhomme20::DIR_RIGHT:
	on(aRight);
	break;
      default:
	break;
      }
      toBePaint=true;
    }
  }
  for(int k=0;k<4;k++) {
    //actualiser les cases "pos"
    pos[k].j=j[k];
    pos[k].i=i[k];
  }
  return 1;
}

void Boue::paint(WindowPipe& windowPipe) throw()
{
  if(!toBePaint) return;
  int j,i;
  bonhommeRef->getPos(j,i);
  move(j,i);
  MultiAnim::paint(windowPipe);
  int r=
    getAnim(aUp)->walk()+
    getAnim(aDown)->walk()+
    getAnim(aLeft)->walk()+
    getAnim(aRight)->walk();
  if(r<4) finished=true;
  toBePaint=false;
}

void Boue::reinit() throw()
{
  toBePaint=false;
  finished=true;
  MultiAnim::reinit();
}
