/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class CaseBombe;

#ifndef __CaseBombe_h__
#define __CaseBombe_h__

#include "Case.h"

/**
   The bomb Case.
   It explodes on a call to 'blowable'. It produces a bomb sound on the Sounder. It is printed via the Anim class.
   @author David FAUTHOUX
   @see Sounder
   @see Bonhomme20
   @see Anim
 */
class CaseBombe:public Case {
 protected:
  ///
  int puissance;
  ///
  bool traverseMur;
  ///
  const int aniNormalId,aniFinId;
  ///
  int step; //0:Normal,1:Fin
  ///
  Bonhomme20 *const bonhomme;
  ///
  Sounder *const sounderRef;
  ///
  const int soundIdRef;
  ///
  int virtual virtualWalk() throw(); //(0=>on(FinBombe)) + ::virtualWalk
 public:
  /**
     Defines entirely the Case : the animations used, the bonhomme who has put the bomb, the Sounder wich manages sounds.
     @param aniNormal the animation while fuse is consuming
     @param aniFin the animation just before blowing
     @param poseur the bonhomme who has put the bomb
     @param sounder the Sounder wich manages sounds
     @param soundId the id for the sound to be played (given by the Sounder when sound loaded)
   */
  CaseBombe(Anim *aniNormal,//normalBombe ou mine(Loop) ou dynamite(Loop) ...
	    Anim *aniFin,//finBombe
	    Bonhomme20 *poseur,//si poseur==0, on ne reinc pas nbPose
	    Sounder& sounder,int soundId) throw();

  ///
  ~CaseBombe() throw() {}
  /**
     True when fuse is consumed (animation ended).
   */
  bool toBeBlownUp() const throw(); //true qd finBombe et walk==0

  /**
     Activates the explosion and return true.
   */
  bool virtual blowable() throw(); //step==0 => on(putFinBombe)

  ///
  bool virtual getTraverseMur() const throw();
  ///
  int virtual getPuissance() const throw();

  /**
     True just before blowing.
   */
  bool restricted() const throw();//qd fin

  /**
     Never.
   */
  bool stackable() const throw();//false

  ///
  bool virtual traversable(bool force) throw();

  ///
  virtual void paintNight(WindowPipe& windowPipe) throw();
};

#endif

