/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "CaseGestion.h"
#include "LoopAnim.h"
#include "Grille.h"
#include "error.h"

//
#include "CaseVideForce.h"
#include "CaseBombe.h"
#include "CaseBoue.h"
#include "CaseDynamite.h"
#include "CaseEscalier.h"
#include "CaseFlamme.h"
#include "CaseGestion.h"
#include "CaseId.h"
#include "CaseJoli.h"
#include "CaseMine.h"
#include "CaseMur.h"
#include "CaseMurDestructible.h"
#include "CaseMurIndestructible.h"
#include "CaseMurRefermable.h"
#include "CaseOption.h"
#include "CaseOptionArmageddon.h"
#include "CaseOptionBombe.h"
#include "CaseOptionChicken.h"
#include "CaseOptionCounter.h"
#include "CaseOptionDynamite.h"
#include "CaseOptionFemme.h"
#include "CaseOptionFeuCul.h"
#include "CaseOptionFlamme.h"
#include "CaseOptionInvincible.h"
#include "CaseOptionInvisible.h"
#include "CaseOptionMine.h"
#include "CaseOptionOuba.h"
#include "CaseOptionPhoto.h"
#include "CaseOptionPoseMur.h"
#include "CaseOptionResurrect.h"
#include "CaseOptionSpeed.h"
#include "CaseOptionSuperBombe.h"
#include "CaseOptionSurpuissance.h"
#include "CaseOptionTeleport.h"
#include "CaseOptionTraverseMur.h"
#include "CaseOptionWin95.h"

//

#define DEFAULT_SLOW_ANIMATION 0


void CaseGestion::setAnim(Ini& ini,int num,CaseId::Id animId,const String& id)
  throw(Exception)
{
  int slow=ini.getWithDefault("[Slow]",0,"Animation",0,DEFAULT_SLOW_ANIMATION);

  List<const String *> *ls=ini.getList("[Fond]",num,id,"[Animation]");
  ls->place();
  while(ls->walk()) {
    allAnim[animId].put(new Anim(*(ls->get()),slow,
				 ini.getListSeparator()));
  }
}

Anim *CaseGestion::findIt(CaseId::Id animId,bool loop,int cpt) throw()
{
  allAnim[animId].place();
  allAnim[animId].walk();
  while(cpt && allAnim[animId].walk()) cpt--;
  if(loop) return new LoopAnim(*(allAnim[animId].get()));
  return new Anim(*(allAnim[animId].get()));
}

Anim *CaseGestion::find(CaseId::Id animId,bool loop) throw(Exception)
{
  int cpt=0;
  allAnim[animId].place();
  while(allAnim[animId].walk()) cpt++;
  oldToBeKept=rand()%cpt;
  return findIt(animId,loop,oldToBeKept);
}

Anim *CaseGestion::findK(CaseId::Id animId,bool loop) throw(Exception)
{
  return findIt(animId,loop,oldToBeKept);
}

CaseGestion::CaseGestion(Ini& ini,int numeroSection,Sounder& sounder,Femme& femme)
  throw (Exception)
  :sounderRef(&sounder),
   oldToBeKept(0),
   femmeRef(&femme)

{
  setAnim(ini,numeroSection,CaseId::BOMBE,"anim bombe");
  setAnim(ini,numeroSection,CaseId::FIN_BOMBE,"anim fin bombe");
  setAnim(ini,numeroSection,CaseId::MINE,"anim mine");
  setAnim(ini,numeroSection,CaseId::DYNAMITE,"anim dynamite");

  setAnim(ini,numeroSection,CaseId::MUR_DESTRUCTIBLE,"anim mur normal");
  setAnim(ini,numeroSection,CaseId::MUR_BRISE,"anim mur brise");
  setAnim(ini,numeroSection,CaseId::MUR_TAS,"anim mur tas");
  setAnim(ini,numeroSection,CaseId::MUR_ENVOLE,"anim mur envole");

  setAnim(ini,numeroSection,CaseId::MUR_REFERMABLE,"anim mur refermable");
  setAnim(ini,numeroSection,CaseId::MUR_REFERMABLE_OPENING,"anim mur refermable opening");
  setAnim(ini,numeroSection,CaseId::MUR_REFERMABLE_CLOSING,"anim mur refermable closing");
  
  setAnim(ini,numeroSection,CaseId::FLAMME_CENTER,"anim flamme center");

  setAnim(ini,numeroSection,CaseId::FLAMME_UP,"anim flamme up");
  setAnim(ini,numeroSection,CaseId::FLAMME_DOWN,"anim flamme down");
  setAnim(ini,numeroSection,CaseId::FLAMME_LEFT,"anim flamme left");
  setAnim(ini,numeroSection,CaseId::FLAMME_RIGHT,"anim flamme right");

  setAnim(ini,numeroSection,CaseId::FLAMME_UP_BOUT,"anim flamme up bout");
  setAnim(ini,numeroSection,CaseId::FLAMME_DOWN_BOUT,"anim flamme down bout");
  setAnim(ini,numeroSection,CaseId::FLAMME_LEFT_BOUT,"anim flamme left bout");
  setAnim(ini,numeroSection,CaseId::FLAMME_RIGHT_BOUT,"anim flamme right bout");

  setAnim(ini,numeroSection,CaseId::FUMEE,"anim fumee");
  setAnim(ini,numeroSection,CaseId::ETOILE,"anim etoile");
  setAnim(ini,numeroSection,CaseId::TELEPORT,"anim teleport");

  setAnim(ini,numeroSection,CaseId::OPTION_BOMBE,"anim option bombe");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_BOMBE,"anim option crame bombe");

  setAnim(ini,numeroSection,CaseId::OPTION_FLAMME,"anim option flamme");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_FLAMME,"anim option crame flamme");

  setAnim(ini,numeroSection,CaseId::OPTION_OUBA,"anim option ouba");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_OUBA,"anim option crame ouba");

  setAnim(ini,numeroSection,CaseId::OPTION_ARMAGEDDON,"anim option armageddon");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_ARMAGEDDON,"anim option crame armageddon");

  setAnim(ini,numeroSection,CaseId::OPTION_CHICKEN,"anim option chicken");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_CHICKEN,"anim option crame chicken");

  setAnim(ini,numeroSection,CaseId::OPTION_PHOTO,"anim option photo");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_PHOTO,"anim option crame photo");

  setAnim(ini,numeroSection,CaseId::OPTION_FEU_CUL,"anim option feu au cul");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_FEU_CUL,"anim option crame feu au cul");

  setAnim(ini,numeroSection,CaseId::OPTION_SUPER_BOMBE,"anim option super bombe");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_SUPER_BOMBE,"anim option crame super bombe");

  setAnim(ini,numeroSection,CaseId::OPTION_MINE,"anim option mine");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_MINE,"anim option crame mine");

  setAnim(ini,numeroSection,CaseId::OPTION_DYNAMITE,"anim option dynamite");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_DYNAMITE,"anim option crame dynamite");

  setAnim(ini,numeroSection,CaseId::OPTION_INVISIBLE,"anim option invisible");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_INVISIBLE,"anim option crame invisible");

  setAnim(ini,numeroSection,CaseId::OPTION_INVINCIBLE,"anim option invincible");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_INVINCIBLE,"anim option crame invincible");

  setAnim(ini,numeroSection,CaseId::OPTION_POSE_MUR,"anim option pose mur");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_POSE_MUR,"anim option crame pose mur");

  setAnim(ini,numeroSection,CaseId::OPTION_SURPUISSANCE,"anim option surpuissance");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_SURPUISSANCE,"anim option crame surpuissance");

  setAnim(ini,numeroSection,CaseId::OPTION_WIN95,"anim option win95");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_WIN95,"anim option crame win95");

  setAnim(ini,numeroSection,CaseId::OPTION_TRAVERSE_MUR,"anim option traverse mur");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_TRAVERSE_MUR,"anim option crame traverse mur");

  setAnim(ini,numeroSection,CaseId::OPTION_FEMME,"anim option femme");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_FEMME,"anim option crame femme");

  setAnim(ini,numeroSection,CaseId::OPTION_TELEPORT,"anim option teleport");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_TELEPORT,"anim option crame teleport");

  setAnim(ini,numeroSection,CaseId::OPTION_RESURRECT,"anim option resurrect");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_RESURRECT,"anim option crame resurrect");

  setAnim(ini,numeroSection,CaseId::OPTION_SPEED,"anim option speed");
  setAnim(ini,numeroSection,CaseId::OPTION_CRAME_SPEED,"anim option crame speed");

  /*%%%%%
  int cpt;
  cpt=0;
  allAnim[_OPTION_LOOP].place();
  while(allAnim[_OPTION_LOOP].walk()) cpt++;
  if(cpt!=CaseId::NB_OPTION)
    throw Exception(ERROR_NEEDED_MEME_NOMBRE_OPTION_ANIM_OPTION);

  cpt=0;
  allAnim[_OPTION_CRAME].place();
  while(allAnim[_OPTION_CRAME].walk()) cpt++;
  if(cpt!=CaseId::NB_OPTION)
    throw Exception(ERROR_NEEDED_MEME_NOMBRE_OPTION_ANIM_OPTION_CRAME);
  */

  //
  soundBombeId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					    "sound bombe","[Sound]"));
  soundArmageddonId=
    sounderRef->add(*ini.getList("[Fond]",numeroSection,
				 "sound armageddon","[Sound]"));
  soundArmageddonBlowUpId=
    sounderRef->add(*ini.getList("[Fond]",numeroSection,
				 "sound armageddon blow up","[Sound]"));
  soundPhotoId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					    "sound photo","[Sound]"));
  soundFlashId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					    "sound flash","[Sound]"));
  soundChickenId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					      "sound chicken","[Sound]"));
  soundChickenEndId=
    sounderRef->add(*ini.getList("[Fond]",numeroSection,
				 "sound become chicken","[Sound]"));
  soundResurrectId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
						"sound resurrect","[Sound]"));
  soundOubaId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					   "sound ouba","[Sound]"));
  soundFemmeId=sounderRef->add(*ini.getList("[Fond]",numeroSection,
					    "sound femme","[Sound]"));
  soundWin95Id=sounderRef->add(*ini.getList("[Fond]",numeroSection,
  					    "sound win95","[Sound]"));
  //
}

CaseGestion::~CaseGestion() throw()
{
  for(int i=0;i<CaseId::NB_ALL;i++) {
    allAnim[i].place();
    while(allAnim[i].walk()) delete allAnim[i].get();
  }
  sounderRef->suppr(soundBombeId);
  sounderRef->suppr(soundArmageddonId);
  sounderRef->suppr(soundArmageddonBlowUpId);
  sounderRef->suppr(soundPhotoId);
  sounderRef->suppr(soundFlashId);
  sounderRef->suppr(soundChickenId);
  sounderRef->suppr(soundChickenEndId);
  sounderRef->suppr(soundResurrectId);
  sounderRef->suppr(soundOubaId);
  sounderRef->suppr(soundFemmeId);
  sounderRef->suppr(soundWin95Id);
}

Case *CaseGestion::get(CaseId::Id id,Grille& grille,Bonhomme20 *bonhomme)
  throw(Exception)
{
  switch(id) {


  case CaseId::VIDE:
    return 0;
  case CaseId::VIDE_FORCE:
    return new CaseVideForce();
  case CaseId::BOMBE:
    return new CaseBombe(find(id),
			 find(CaseId::FIN_BOMBE),
			 bonhomme,
			 *sounderRef,soundBombeId);
  case CaseId::MINE:
    return new CaseMine(find(id,true),
			find(CaseId::FIN_BOMBE),bonhomme,
			*sounderRef,soundBombeId);
  case CaseId::DYNAMITE:
    return new CaseDynamite(find(id,true),
			    find(CaseId::FIN_BOMBE),bonhomme,
			    *sounderRef,soundBombeId);
  case CaseId::MUR_INDESTRUCTIBLE:
    return new CaseMurIndestructible();
  case CaseId::ESCALIER:
    return new CaseEscalier();
  case CaseId::MUR_DESTRUCTIBLE: {
    Anim *a=find(id,true);
    return new CaseMurDestructible(a,
				   findK(CaseId::MUR_BRISE),
				   findK(CaseId::MUR_TAS,true),
				   findK(CaseId::MUR_ENVOLE));
  }
  case CaseId::MUR_REFERMABLE: {
    Anim *a=find(id,true);
    return new CaseMurRefermable(a,
				 findK(CaseId::MUR_REFERMABLE_OPENING),
				 findK(CaseId::MUR_REFERMABLE_CLOSING));
  }
  case CaseId::FLAMME_CENTER:
  case CaseId::FLAMME_UP:
  case CaseId::FLAMME_DOWN:
  case CaseId::FLAMME_LEFT:
  case CaseId::FLAMME_RIGHT:
  case CaseId::FLAMME_UP_BOUT:
  case CaseId::FLAMME_DOWN_BOUT:
  case CaseId::FLAMME_LEFT_BOUT:
  case CaseId::FLAMME_RIGHT_BOUT:
    return new CaseFlamme(find(id));

    
  case CaseId::OPTION_BOMBE: {
    Anim *a=find(id,true);
    return new CaseOptionBombe(a,
			       findK(CaseId::OPTION_CRAME_BOMBE));
  }
  case CaseId::OPTION_FLAMME: {
    Anim *a=find(id,true);
    return new CaseOptionFlamme(a,
				findK(CaseId::OPTION_CRAME_FLAMME));
  }
  case CaseId::OPTION_TRAVERSE_MUR: {
    Anim *a=find(id,true);
    return new CaseOptionTraverseMur(a,
				     findK(CaseId::OPTION_CRAME_TRAVERSE_MUR));
  }
  case CaseId::OPTION_DYNAMITE: {
    Anim *a=find(id,true);
    return new CaseOptionDynamite(a,
				  findK(CaseId::OPTION_CRAME_DYNAMITE));
  }
  case CaseId::OPTION_CHICKEN: {
    Anim *a=find(id,true);
    return new CaseOptionChicken(a,
				 findK(CaseId::OPTION_CRAME_CHICKEN),
				 *sounderRef,soundChickenId,soundChickenEndId);
  }
  case CaseId::OPTION_FEU_CUL: {
    Anim *a=find(id,true);
    return new CaseOptionFeuCul(a,
				findK(CaseId::OPTION_CRAME_FEU_CUL));
  }
  case CaseId::OPTION_SPEED: {
    Anim *a=find(id,true);
    return new CaseOptionSpeed(a,
			       findK(CaseId::OPTION_CRAME_SPEED));
  }
  case CaseId::OPTION_INVISIBLE: {
    Anim *a=find(id,true);
    return new CaseOptionInvisible(a,
				   findK(CaseId::OPTION_CRAME_INVISIBLE));
  }
  case CaseId::OPTION_INVINCIBLE: {
    Anim *a=find(id,true);
    return new CaseOptionInvincible(a,
				    findK(CaseId::OPTION_CRAME_INVINCIBLE));
  }
  case CaseId::OPTION_SURPUISSANCE: {
    Anim *a=find(id,true);
    return new CaseOptionSurpuissance(a,
				      findK(CaseId::OPTION_CRAME_SURPUISSANCE));
  }
  case CaseId::OPTION_MINE: {
    Anim *a=find(id,true);
    return new CaseOptionMine(a,
			      findK(CaseId::OPTION_CRAME_MINE));
  }
  case CaseId::OPTION_POSE_MUR: {
    Anim *a=find(id,true);
    return new CaseOptionPoseMur(a,
				 findK(CaseId::OPTION_CRAME_POSE_MUR));
  }
  case CaseId::OPTION_OUBA: {
    Anim *a=find(id,true);
    return new CaseOptionOuba(a,
			      findK(CaseId::OPTION_CRAME_OUBA),
			      *sounderRef,soundOubaId);
  }
  case CaseId::OPTION_PHOTO: {
    Anim *a=find(id,true);
    return new CaseOptionPhoto(a,
			       findK(CaseId::OPTION_CRAME_PHOTO),
			       grille,*sounderRef,soundPhotoId,soundFlashId);
  }
  case CaseId::OPTION_RESURRECT: {
    Anim *a=find(id,true);
    return new CaseOptionResurrect(a,
				   findK(CaseId::OPTION_CRAME_RESURRECT),
				   *sounderRef,soundResurrectId);
  }
  case CaseId::OPTION_SUPER_BOMBE: {
    Anim *a=find(id,true);
    return new CaseOptionSuperBombe(a,
				    findK(CaseId::OPTION_CRAME_SUPER_BOMBE));
  }
  case CaseId::OPTION_TELEPORT: {
    Anim *a=find(id,true);
    return new CaseOptionTeleport(a,
				  findK(CaseId::OPTION_CRAME_TELEPORT));
  }
  case CaseId::OPTION_WIN95: {
    Anim *a=find(id,true);
    return new CaseOptionWin95(a,
			       findK(CaseId::OPTION_CRAME_WIN95),
			       *sounderRef,soundWin95Id);
  }
  case CaseId::OPTION_ARMAGEDDON: {
    Anim *a=find(id,true);
    return new CaseOptionArmageddon(a,
				    findK(CaseId::OPTION_CRAME_ARMAGEDDON),
				    grille,*sounderRef,soundArmageddonId,
				    soundArmageddonBlowUpId);
  }
  case CaseId::OPTION_FEMME: {
    Anim *a=find(id,true);
    return new CaseOptionFemme(a,
			       findK(CaseId::OPTION_CRAME_FEMME),
			       grille,*sounderRef,soundFemmeId,*femmeRef);
  }

  case CaseId::SOL_FLAMME:
    return new CaseFlamme(0);

  case CaseId::SOL_OPTION_BOMBE:
    return new CaseOptionBombe(0,0);
  case CaseId::SOL_OPTION_FLAMME:
    return new CaseOptionFlamme(0,0);
  case CaseId::SOL_OPTION_TRAVERSE_MUR:
    return new CaseOptionTraverseMur(0,0);
  case CaseId::SOL_OPTION_DYNAMITE:
    return new CaseOptionDynamite(0,0);
  case CaseId::SOL_OPTION_CHICKEN:
    return new CaseOptionChicken(0,0,*sounderRef,
				 soundChickenId,soundChickenEndId);
  case CaseId::SOL_OPTION_FEU_CUL:
    return new CaseOptionFeuCul(0,0);
  case CaseId::SOL_OPTION_INVISIBLE:
    return new CaseOptionInvisible(0,0);
  case CaseId::SOL_OPTION_INVINCIBLE:
    return new CaseOptionInvincible(0,0);
  case CaseId::SOL_OPTION_SURPUISSANCE:
    return new CaseOptionSurpuissance(0,0);
  case CaseId::SOL_OPTION_MINE:
    return new CaseOptionMine(0,0);
  case CaseId::SOL_OPTION_POSE_MUR:
    return new CaseOptionPoseMur(0,0);
  case CaseId::SOL_OPTION_OUBA:
    return new CaseOptionOuba(0,0,*sounderRef,soundOubaId);
  case CaseId::SOL_OPTION_PHOTO:
    return new CaseOptionPhoto(0,0,grille,*sounderRef,
			       soundPhotoId,soundFlashId);
  case CaseId::SOL_OPTION_RESURRECT:
    return new CaseOptionResurrect(0,0,*sounderRef,soundResurrectId);
  case CaseId::SOL_OPTION_SUPER_BOMBE:
    return new CaseOptionSuperBombe(0,0);
  case CaseId::SOL_OPTION_TELEPORT:
    return new CaseOptionTeleport(0,0);
  case CaseId::SOL_OPTION_WIN95:
    return new CaseOptionWin95(0,0,*sounderRef,soundWin95Id);
  case CaseId::SOL_OPTION_ARMAGEDDON:
    return new CaseOptionArmageddon(0,0,grille,*sounderRef,soundArmageddonId,
				    soundArmageddonBlowUpId);
  case CaseId::SOL_OPTION_FEMME:
    return new CaseOptionFemme(0,0,grille,*sounderRef,soundFemmeId,*femmeRef);
  case CaseId::SOL_OPTION_SPEED:
    return new CaseOptionSpeed(0,0);

  case CaseId::SOL_BOUE:
    return new CaseBoue();

  case CaseId::FUMEE:
    return new CaseJoli(find(CaseId::FUMEE));
  case CaseId::ETOILE:
    return new CaseJoli(find(CaseId::ETOILE));
  case CaseId::TELEPORT:
    return new CaseJoli(find(CaseId::TELEPORT));

  default:
    //%%    return 0;
    throw Exception(ERROR_INVALID_CASE_ID);
  }
}

int CaseGestion::getSoundBombeId() const throw()
{
  return soundBombeId;
}
