/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include <stdio.h>
#include "File.h"
#include "error.h"

void File::open(const char *name,OpenAutorisation autorisation)
  throw (Exception)
{
  this->autorisation=autorisation;
  this->name=name;
  if(autorisation==READ_ONLY) {
    f=fopen(name,"r");
    if(f==0) throw Exception(ERROR_FILE_OPEN,name);
  }
  else {//WRITE_ONLY
    f=fopen(name,"w");
    if(f==0) throw Exception(ERROR_FILE_OPEN,name);
  }
}

void File::close() throw ()
{
  fclose(f);
}

bool File::eof() const throw ()
{
  //marche pas  if(feof(f)) return true;
  if(pos==size) return true;
  return false;
}

int File::getSize() const throw ()
{
  return size;
}

void File::write(const char *buffer,int size) throw (Exception)
{
  if(size!=(int)fwrite(buffer,1,size,f)) {
    pos=this->size;
    throw Exception(ERROR_FILE_WRITE,name);
  }
  pos+=size;
}

void File::write(const char *str) throw (Exception)
{
  int i=0;
  while(str[i]) {
    if(fwrite(str+i,1,1,f)!=1) {
      pos=size;
      throw Exception(ERROR_FILE_WRITE,name);
    }
    i++;
    pos++;
  }
  
}

void File::write(char c) throw (Exception)
{
  char buf[1]={c};
  if(fwrite(buf,1,1,f)!=1) {
    pos=size;
    throw Exception(ERROR_FILE_WRITE,name);
  }
  pos++;
}

void File::write(short c) throw (Exception)
{
  char to[sizeof(short)];
  int *i=(int *)to;
  *i=c;
  for(unsigned int k=0;k<sizeof(short);k++) write(to[k]);
} 

void File::write(int c) throw (Exception)
{
  char to[sizeof(int)];
  int *i=(int *)to;
  *i=c;
  for(unsigned int k=0;k<sizeof(int);k++) write(to[k]);
} 

void File::read(char *buffer,int size) throw (Exception)
{
  if(size!=(int)fread(buffer,1,size,f)) {
    pos=this->size;
    throw Exception(ERROR_FILE_READ,name);
  }
  pos+=size;
}

void File::read(char *& str) throw (Exception)
{
  int cpt=0;
  List<char> l;
  char c;
  do {
    read(c);
    l.put(c);
    cpt++;
  } while(c!='\n' && !eof());
  str=new char[cpt+1];
  cpt=0;
  l.place();
  while(l.walk()) {
    str[cpt]=l.get();
    cpt++;
  }
  str[cpt]=0;
}

void File::read(char& c) throw (Exception)
{
  char buf[1];
  if(fread(buf,1,1,f)!=1) {
    pos=size;
    throw Exception(ERROR_FILE_READ,name);
  }
  c=buf[0];
  pos++;
}

void File::read(short& c) throw (Exception)
{
  char from[sizeof(short)];
  for(unsigned int k=0;k<sizeof(short);k++) read(from[k]);
  int *i=(int *)from;
  c=*i;
}

void File::read(int& c) throw (Exception)
{
  char from[sizeof(int)];
  for(unsigned int k=0;k<sizeof(int);k++) read(from[k]);
  int *i=(int *)from;
  c=*i;
}

void File::seek(int offset,SeekCode code) throw (Exception)
{
  switch(code) {
  case FROM_BEGIN:
    if(fseek(f,offset,SEEK_SET)<0) throw Exception();
    pos=offset;
    break;
  case FROM_CURRENT:
    if(fseek(f,offset,SEEK_CUR)<0) throw Exception();
    pos+=offset;
    break;
  case FROM_END:
    if(fseek(f,offset,SEEK_END)<0) throw Exception();
    pos=size-offset;
    break;
  }
}

File::File(const String& name,OpenAutorisation autorisation) throw (Exception)
  :pos(0)
{
  open((const char*)name,autorisation);
  calculerSize();
}

void File::calculerSize() throw(Exception)
{
  seek(0,FROM_END);
  size=ftell(f);
  if(size<0) throw Exception();
  seek(0,FROM_BEGIN);
}

File::File(const char *name,OpenAutorisation autorisation) throw (Exception)
  :pos(0)
{
  open(name,autorisation);
  calculerSize();
}

File::~File() throw ()
{
  close();
}

void File::read(String *& s) throw(Exception)
{
  List<char> list;
  bool found=false;
  while(!eof() && !found) {
    char c;
    read(c);
    if(c=='\n') found=true;
    else list.put(c);
  }
  s=new String(list);
}


bool File::exists(const String& nameFile) throw()
{
  FILE *fe=fopen((const char*)nameFile,"r");
  if(fe==0) return false;
  return true;
}

String *File::renameCheck(const String& nameFile,const String& toAdd) throw(Exception)
{
  List<char> l;
  for(int k=0;k<nameFile.size();k++) l.put(nameFile[k]);
  String *s;
  do {
    for(int k=0;k<toAdd.size();k++) l.put(toAdd[k]);
    s=new String(l);
  }
  while(File::exists(*s));
  if(rename((const char*)nameFile,(const char*)*s)) throw Exception(ERROR_FILE_RENAME);
  return s;
}
