/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "GifIn.h"
#include "error.h"

//%%%#include "ImageCompletionScriptAnalyser.h"
#include "File.h"

//%%#define NAME_FILE_BEFORE_COMPLETION "."
//%%#define NAME_FILE_AFTER_COMPLETION ""


GifIn::GifIn(const String& nameFile) throw(Exception)
  :gif89disposal(-1),gif89inputFlag(-1),gif89delayTime(-1),gif89transparent(0),
   ZeroDataBlock(false)
   //%%   rgbBonhommeColors(new List<Pixel>)
{
  /*%%
  for(int k=0;k<GIF_NB_COLORS;k++) translucideColors[k]=false;
  for(int k=0;k<GIF_NB_COLORS;k++) bonhommeColors[k]=false;
  */

  //  try {
    /*%%%%
    String *completionFile=&(*new String(NAME_FILE_BEFORE_COMPLETION)+nameFile+*new String(NAME_FILE_AFTER_COMPLETION));
    if(File::exists(*completionFile)) {
      ImageCompletionScriptAnalyser s(*completionFile,false);
      s.analyse();

      List< int > *l;

      l=s.getTranslucideColors();
      l->place();
      while(l->walk()) translucideColors[l->get()]=true;
      delete l;

      l=s.getBonhommeColors();
      l->place();
      while(l->walk()) bonhommeColors[l->get()]=true;
      delete l;

    }
    */
    if(!load_image((const char*)nameFile)) throw Exception(ERROR_GIF_FILE);
    /*
    return;
  } catch(Exception) {
  }
  throw Exception(ERROR_GIF_FILE);
    */
}



#define MAX_LWZ_BITS     12

#define INTERLACE          0x40
#define LOCALCOLORMAP      0x80
#define BitSet(byte, bit)  (((byte) & (bit)) == (bit))

#define ReadOK(file,buffer,len) (fread(buffer, len, 1, file) != 0)
#define LM_to_uint(a,b)         (((b)<<8)|(a))




#define g_message printf



int GifIn::GetDataBlock (FILE          *fd,
			 unsigned char *buf)
{
  unsigned char count;

  if (!ReadOK (fd, &count, 1)) {
    g_message ("GIF: error in getting DataBlock size\n");
    return -1;
  }

  ZeroDataBlock = count == 0;

  if ((count != 0) && (!ReadOK (fd, buf, count))) {
    g_message ("GIF: error in reading DataBlock\n");
    return -1;
  }

  return count;
}


bool GifIn::ReadColorMap (FILE *fd,
			  int   number,
			  uchar  *buffer)
{
  int i;
  unsigned char rgb[3];

  for (i = 0; i < number; ++i) {
      if (!ReadOK (fd, rgb, sizeof (rgb))) {
	g_message ("GIF: bad colormap\n");
	return false;
      }

      buffer[i*3+0] = rgb[0];
      buffer[i*3+1] = rgb[1];
      buffer[i*3+2] = rgb[2];
      
    }

  return true;
}


bool GifIn::DoExtension (FILE *fd,
			 int   label)
{
  static uchar buf[256];
  char *str;

  switch (label) {
  case 0x01:			/* Plain Text Extension */
    str = "Plain Text Extension";
    break;
  case 0xff:			/* Application Extension */
    str = "Application Extension";
    break;
  case 0xfe:			/* Comment Extension */
    str = "Comment Extension";
    while (GetDataBlock (fd, (uchar *) buf) != 0);
    return true;
  case 0xf9:			/* Graphic Control Extension */
    str = "Graphic Control Extension";
    GetDataBlock (fd, (uchar *) buf);
    gif89disposal = (buf[0] >> 2) & 0x7;
    gif89inputFlag = (buf[0] >> 1) & 0x1;
    gif89delayTime = LM_to_uint (buf[1], buf[2]);
    if ((buf[0] & 0x1) != 0)
      gif89transparent = buf[3];
    else
      gif89transparent = -1;

    while (GetDataBlock (fd, (uchar *) buf) != 0);
    return true;
  default:
    break;
  }

  while (GetDataBlock (fd, (uchar *) buf) != 0);

  return false;
}

int GifIn::GetCode (FILE *fd,
		    int   code_size,
		    int   flag)
{
  static unsigned char buf[280];
  static int curbit, lastbit, done, last_byte;
  int i, j, ret;
  unsigned char count;

  if (flag)
    {
      curbit = 0;
      lastbit = 0;
      done = false;
      return 0;
    }

  if ((curbit + code_size) >= lastbit)
    {
      if (done)
	{
	  if (curbit >= lastbit)
	    {
	      //	      g_message ("GIF: ran off the end of by bits\n");
	      throw Exception("GIF: ran off the end of by bits");//gimp_quit ();
	    }
	  return -1;
	}
      buf[0] = buf[last_byte - 2];
      buf[1] = buf[last_byte - 1];

      if ((count = GetDataBlock (fd, &buf[2])) == 0)
	done = true;

      last_byte = 2 + count;
      curbit = (curbit - lastbit) + 16;
      lastbit = (2 + count) * 8;
    }

  ret = 0;
  for (i = curbit, j = 0; j < code_size; ++i, ++j)
    ret |= ((buf[i / 8] & (1 << (i % 8))) != 0) << j;

  curbit += code_size;

  return ret;
}

int GifIn::LWZReadByte (FILE *fd,
			int   flag,
			int   input_code_size)
{
  static int fresh = false;
  int code, incode;
  static int code_size, set_code_size;
  static int max_code, max_code_size;
  static int firstcode, oldcode;
  static int clear_code, end_code;
  static int table[2][(1 << MAX_LWZ_BITS)];
  static int stack[(1 << (MAX_LWZ_BITS)) * 2], *sp;
  register int i;

  if (flag)
    {
      set_code_size = input_code_size;
      code_size = set_code_size + 1;
      clear_code = 1 << set_code_size;
      end_code = clear_code + 1;
      max_code_size = 2 * clear_code;
      max_code = clear_code + 2;

      GetCode (fd, 0, true);

      fresh = true;

      for (i = 0; i < clear_code; ++i)
	{
	  table[0][i] = 0;
	  table[1][i] = i;
	}
      for (; i < (1 << MAX_LWZ_BITS); ++i)
	table[0][i] = table[1][0] = 0;

      sp = stack;

      return 0;
    }
  else if (fresh)
    {
      fresh = false;
      do
	{
	  firstcode = oldcode =
	    GetCode (fd, code_size, false);
	}
      while (firstcode == clear_code);
      return firstcode;
    }

  if (sp > stack)
    return *--sp;

  while ((code = GetCode (fd, code_size, false)) >= 0)
    {
      if (code == clear_code)
	{
	  for (i = 0; i < clear_code; ++i)
	    {
	      table[0][i] = 0;
	      table[1][i] = i;
	    }
	  for (; i < (1 << MAX_LWZ_BITS); ++i)
	    table[0][i] = table[1][i] = 0;
	  code_size = set_code_size + 1;
	  max_code_size = 2 * clear_code;
	  max_code = clear_code + 2;
	  sp = stack;
	  firstcode = oldcode =
	    GetCode (fd, code_size, false);
	  return firstcode;
	}
      else if (code == end_code)
	{
	  int count;
	  unsigned char buf[260];

	  if (ZeroDataBlock)
	    return -2;

	  while ((count = GetDataBlock (fd, buf)) > 0)
	    ;

	  if (count != 0) g_message ("GIF: missing EOD in data stream (common occurence)");
	  return -2;
	}

      incode = code;

      if (code >= max_code)
	{
	  *sp++ = firstcode;
	  code = oldcode;
	}

      while (code >= clear_code)
	{
	  *sp++ = table[1][code];
	  if (code == table[0][code])
	    {
	      throw Exception("GIF: circular table entry BIG ERROR");
	      //	      g_message ("GIF: circular table entry BIG ERROR\n");
	      //	      gimp_quit ();
	    }
	  code = table[0][code];
	}

      *sp++ = firstcode = table[1][code];

      if ((code = max_code) < (1 << MAX_LWZ_BITS))
	{
	  table[0][code] = oldcode;
	  table[1][code] = firstcode;
	  ++max_code;
	  if ((max_code >= max_code_size) &&
	      (max_code_size < (1 << MAX_LWZ_BITS)))
	    {
	      max_code_size *= 2;
	      ++code_size;
	    }
	}

      oldcode = incode;

      if (sp > stack)
	return *--sp;
    }
  return code;
}


bool GifIn::ReadImage (FILE *fd,
		       uchar *cmap,
		       int   ncols,
		       int   interlace,
		       int   leftpos,
		       int   toppos,
		       int screenWidth,
		       int screenHeight,
		       Sprite& sprite,
		       int jSize,
		       int iSize)
{

  int xpos = 0, ypos = 0, pass = 0;
  int v;
  uchar c;
  int len=iSize;
  int height=jSize;

  if (!ReadOK (fd, &c, 1)) {
    g_message ("GIF: EOF / read error on image data\n");
    return false;
  }

  if (LWZReadByte (fd, true, c) < 0) {
    g_message ("GIF: error while reading\n");
    return false;
  }


  while ((v = LWZReadByte (fd, false, c)) >= 0) {

    if(ypos+toppos<sprite.nbLine() && xpos+leftpos<sprite.nbColumn() && ypos+toppos>=0 && xpos+leftpos>=0) {
      if(v!=gif89transparent) {
	/*%%%%%%%
	if(translucideColors[v])
	  sprite.setLuz(ypos+toppos,xpos+leftpos);
	else
	*/
	sprite.set(cmap[v*3+0],cmap[v*3+1],cmap[v*3+2],ypos+toppos,xpos+leftpos);
	/*%%%
	if(bonhommeColors[v]) 
	  rgbBonhommeColors->put(toPixel(cmap[v*3+0],cmap[v*3+1],cmap[v*3+2],0));
	*/

      }
      //      currentData[leftpos+xpos+(ypos+toppos)*len]=v;
    }

    ++xpos;
    if (xpos == len) {
      xpos = 0;
	if (interlace) {
	  switch (pass) {
	  case 0:
	  case 1:
	    ypos += 8;
	    break;
	  case 2:
	    ypos += 4;
	    break;
	  case 3:
	    ypos += 2;
	    break;
	  }

	  if (ypos >= height) {
	    ++pass;
	    switch (pass) {
	    case 1:
	      ypos = 4;
	      break;
	    case 2:
	      ypos = 2;
	      break;
	    case 3:
	      ypos = 1;
	      break;
	    default:
	      goto fini;
	    }
	  }
	}
	else {
	  ++ypos;
	}

      }
      if (ypos >= height)
	break;
  }

 fini:
  if (LWZReadByte (fd, false, c) >= 0) g_message ("GIF: too much input data, ignoring extra...\n");

  return true;
}


bool GifIn::load_image (const char *filename)
{
  int gifScreenWidth;
  int gifScreenHeight;
  
  int gifScreenBitPixel;
  int gifScreenColorResolution;
  int gifScreenBackground;
  int gifScreenAspectRatio;

  uchar *gifScreenColorMap=new uchar[768];

  FILE *fd;


  unsigned char buf[16];
  char version[4];

  fd = fopen (filename, "r");
  if(!fd) {
    perror("ERROR:");
    g_message ("GIF: can't open \"%s\"\n", filename);
    return false;
  }

  if(!ReadOK (fd, buf, 6)) {
    g_message ("GIF: error reading magic number\n");
    fclose(fd);
    return false;
  }

  if(strncmp ((char *) buf, "GIF", 3) != 0) {
    g_message ("GIF: not a GIF file\n");
    fclose(fd);
    return false;
  }

  strncpy (version, (char *) buf + 3, 3);
  version[3] = '\0';

  if ((strcmp (version, "87a") != 0) && (strcmp (version, "89a") != 0)) {
    g_message ("GIF: bad version number, not '87a' or '89a'\n");
    fclose(fd);
    return false;
  }

  if (!ReadOK (fd, buf, 7)) {
    g_message ("GIF: failed to read screen descriptor\n");
    fclose(fd);
    return false;
  }

  gifScreenWidth = LM_to_uint (buf[0], buf[1]);
  gifScreenHeight = LM_to_uint (buf[2], buf[3]);
  gifScreenBitPixel = 2 << (buf[4] & 0x07);
  gifScreenColorResolution = (((buf[4] & 0x70) >> 3) + 1);
  gifScreenBackground = buf[5];
  gifScreenAspectRatio = buf[6];

  if (BitSet (buf[4], LOCALCOLORMAP)) {
    /* Global Colormap */
    if(!ReadColorMap (fd, gifScreenBitPixel, gifScreenColorMap)) {
      g_message ("GIF: error reading global colormap\n");
      fclose(fd);
      return false;
    }
  }


  if (gifScreenAspectRatio != 0 && gifScreenAspectRatio != 49) {
    g_message ("GIF: warning - non-square pixels\n");
  }

  //  highest_used_index = 0;

  for(;;) {
    uchar *localColorMap;
    int useGlobalColorMap;
    uchar c;
    int bitPixel;


    if (!ReadOK (fd, &c, 1)) {
      g_message ("GIF: EOF / read error on image data\n");
      fclose(fd);
      return false;
    }

    if (c == ';') {
      /* GIF terminator */
      fclose(fd);
      return true;
    }

    if (c == '!') {
      /* Extension */
      if (!ReadOK (fd, &c, 1)) {
	g_message ("GIF: OF / read error on extention function code\n");
	fclose(fd);
	return false;
      }
      DoExtension (fd, c);
      continue;
    }

    if (c != ',') {
      /* Not a valid start character */
      g_message ("GIF: bogus character 0x%02x, ignoring\n", (int) c);
      continue;
    }

    //    ++imageCount;

    if (!ReadOK (fd, buf, 9)) {
      g_message ("GIF: couldn't read left/top/width/height\n");
      fclose(fd);
      return false;
    }

    useGlobalColorMap = !BitSet (buf[8], LOCALCOLORMAP);

    bitPixel = 1 << ((buf[8] & 0x07) + 1);
    
    if(!useGlobalColorMap) {
      localColorMap=new uchar[bitPixel*3];
      if(!ReadColorMap (fd, bitPixel, localColorMap)) {
	g_message ("GIF: error reading local colormap\n");
	fclose(fd);
	return false;
      }
    }
    else {
      localColorMap=gifScreenColorMap;
      bitPixel=gifScreenBitPixel;
    }


    int jSize=LM_to_uint (buf[6], buf[7]);
    int iSize=LM_to_uint (buf[4], buf[5]);
    //    currentData=new uchar[jSize*iSize];
    //        Sprite *sprite=new Sprite(jSize,iSize);
    Sprite *sprite=new Sprite(gifScreenHeight,gifScreenWidth);


    if(!ReadImage (fd,
		   localColorMap,
		   bitPixel,
		   BitSet (buf[8], INTERLACE),
		   LM_to_uint (buf[0], buf[1]),
		   LM_to_uint (buf[2], buf[3]),
		   gifScreenWidth,
		   gifScreenHeight,
		   *sprite,jSize,iSize
		   )) {
      fclose(fd);
      return false;
    }

    if(!useGlobalColorMap) delete localColorMap;

    list.put(sprite);
    //    gifData.put(currentData);

  }
}



