/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "GraphicMode.h"
#include "display.h"
#include "mem_function.h"
#include "error.h"

GraphicMode::GraphicMode(int gapBetweenLuz) throw ()
  :PixelMatrix(graphicHeight(),graphicWidth()),actif(false)
{
  if(!enablePixelPalette(gapBetweenLuz))
    throw Exception(ERROR_CANNOT_ENABLE_PIXEL_PALETTE);
}

GraphicMode::~GraphicMode() throw ()
{
  if(actif) {
    disableGraphic();
  }
  disablePixelPalette();
}

void GraphicMode::getSize(int& j,int& i) const throw ()
{
  j=nbLine();
  i=nbColumn();
}


void GraphicMode::on() throw(Exception)
{
  if(actif) return;
  actif=true;
  if(!enableGraphic())
    throw Exception(ERROR_CANNOT_ENABLE_GRAPHIC_MODE);
}

void GraphicMode::off() throw()
{
  if(!actif) return;
  actif=false;
  disableGraphic();
}

void GraphicMode::refresh() throw()
{
  if(!actif) return;

  PixelMatrix *toBePrint=this;
  redirector.place();
  while(redirector.walk()) {
    toBePrint=redirector.get()->pipeIt(*toBePrint);
    if(!toBePrint) return;
  }
  
  
  /*
  int j=nbLine();
  int i=nbColumn();
  if(toBePrint->nbLine()<j) j=toBePrint->nbLine();
  if(toBePrint->nbColumn()<i) i=toBePrint->nbColumn();
  
  toBePrint->uncheckedCopyTo(getGraphicMem(),j,i);
  */

  graphicRefresh(*toBePrint);
}

void GraphicMode::redirect(Redirector& r) throw()
{
  redirector.placeCurrentOnHead();
  redirector.put(&r);
}

void GraphicMode::clear() throw ()
{
  memClear(*this,nbLine()*nbColumn());
}

/*
GraphicMode& GraphicMode::operator=(const Array2<Pixel>& a) throw()
{
  this->Array2<Pixel>::operator=(a);
  return *this;
}
*/

