/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <stdlib.h>
#include "Grille.h"
#include "PageGame.h"
#include "util.h"
#include "GrilleUtilCase.h"

#define DEFAULT_CASE_SIZE 20
#define DEFAULT_AMOUNT_MUR_DESTRUCTIBLE 10
#define DEFAULT_AMOUNT_MUR_REFERMABLE 0
#define DEFAULT_AMOUNT_OPTION 5

#define DEFAULT_TREMBLE_J 10
#define DEFAULT_TREMBLE_I 10

#define MAX_TEST_REMPLIR 100


#define DEFAULT_AMOUNT_ARMAGEDDON_SCREEN_BLOW_UP 2


Grille::Grille(Ini& ini,BonhommeContainer& bonhommeContainer,
	       FlashPalette& palette,Sounder& sounder,
	       Blower& blower,
	       StripContainer& stripContainer) throw()
  :PageableContainer(0),
   caseGestion(0),
   sounderRef(&sounder),
   bonhommeContainerRef(&bonhommeContainer),
   paletteRef(&palette),
   tab(0),
   trembleurRef(0),
   coeur(new Coeur(ini)),
   armageddonActived(false),
   countScreenBlow(0),
   blowerRef(&blower),
   stripContainerRef(&stripContainer),
   femme(ini),
   nightActive(false),nightSprite(0)
{
  mult.j=mult.i=1;
  trembleExplosion.j=ini.getWithDefault("[Global]",0,
					"tremble explosion j",0,
					DEFAULT_TREMBLE_J);
  trembleExplosion.i=ini.getWithDefault("[Global]",0,
					"tremble explosion i",0,
					DEFAULT_TREMBLE_I);
  trembleArmageddon.j=ini.getWithDefault("[Global]",0,
					 "tremble armageddon j",0,
					 DEFAULT_TREMBLE_J);
  trembleArmageddon.i=ini.getWithDefault("[Global]",0,
					 "tremble armageddon i",0,
					 DEFAULT_TREMBLE_I);
  maxCountScreenBlow=
    ini.getWithDefault("[Global]",0,
		       "amount armageddon to blow up screen",0,
		       DEFAULT_AMOUNT_ARMAGEDDON_SCREEN_BLOW_UP);
}

void Grille::setTrembleur(Trembleur& trembleur) throw()
{
  trembleurRef=&trembleur;
}

void Grille::destroy() throw()
{
  list.place();
  while(list.walk()) {
    delete list.get();
    list.suppr();
  }
  if(tab) {
    for(int n=0;n<nbNiv();n++)
      for(int j=0;j<nbLine();j++)
	for(int i=0;i<nbColumn();i++) {
	  List<Case*> *l=(*tab)[n][j][i];
	  delete l;
	}
    delete tab;
    tab=0;
  }

  if(caseGestion) delete caseGestion;
}
/////////////////

#define MAX_SIZE_STRING 10

void Grille::putIniCase(Ini& ini,const String& s,CaseId::Id id,bool allNiv)
  throw(Exception)
{
  List<const String *> *coord=ini.getListWithoutException("[Grille]",0,s);
  coord->place();
  while(coord->walk()) {

    List<int> listInt;//%%=new List<int>;
    if(extractInt(*(coord->get()),ini.getListSeparator(),listInt,ini.language)) {
      listInt.place();
      if(listInt.walk() && listInt.walk()) {
	listInt.place();
	listInt.walk();
	int j=listInt.get();
	listInt.walk();
	int i=listInt.get();

	if(j>=0 && j<nbLine() && i>=0 && i<nbColumn()) {
	  if(allNiv) for(int n=0;n<nbNiv();n++) add(id,n,j,i); 
	  else add(id,0,j,i);
	}
      }
    }
  }
  delete coord;
}


//////////////////////
void Grille::create(Ini& ini,int numeroSection,int jBackSize,int iBackSize) throw(Exception)
{
  destroy();

  caseGestion=0;
  caseGestion=new CaseGestion(ini,numeroSection,*sounderRef,femme);

  mult.j=ini.getWithDefault("[Fond]",numeroSection,
			   "case size j",0,DEFAULT_CASE_SIZE);
  mult.i=ini.getWithDefault("[Fond]",numeroSection,
			   "case size i",0,DEFAULT_CASE_SIZE);

  int nbBonhomme=bonhommeContainerRef->getNbPlayer();

  Ini iniGrille(*ini.get("[Fond]",numeroSection,"grille",0));

  int auxNiv=2;//nombre de niveau dans la grille, pour l'instant c'est 2, pour en mettre plus, il faut changer l'affichage des niveaux et plein de trucs, c'est dur !

  tab=new Array3<List<Case *>*>(auxNiv,jBackSize/mult.j,iBackSize/mult.i);
  for(int n=0;n<nbNiv();n++)
    for(int j=0;j<nbLine();j++)
      for(int i=0;i<nbColumn();i++) {
	(*tab)[n][j][i]=new List<Case*>;
	//%%%	add(CaseGestion::VIDE,n,j,i);
      }

  GrilleUtilCase grilleUtilCase;
  grilleUtilCase.list.place();
  while(grilleUtilCase.list.walk()) 
    putIniCase(iniGrille,
	       grilleUtilCase.list.get()->name,
	       grilleUtilCase.list.get()->id,
	       grilleUtilCase.list.get()->allNiv);

  /*%%%%%%%%%%%%%
  putIniCase(iniGrille,"mur indestructible",         CaseId::MUR_INDESTRUCTIBLE,false);
  putIniCase(iniGrille,"mur haut",                   CaseId::MUR_INDESTRUCTIBLE,true);
  putIniCase(iniGrille,"vide force",                 CaseId::VIDE_FORCE,false);
  putIniCase(iniGrille,"escalier",                   CaseId::ESCALIER,false);
  putIniCase(iniGrille,"steady flamme",              CaseId::SOL_FLAMME,false);
  putIniCase(iniGrille,"steady option bombe",        CaseId::SOL_OPTION_BOMBE,false);
  putIniCase(iniGrille,"steady option flamme",       CaseId::SOL_OPTION_FLAMME,false);
  putIniCase(iniGrille,"steady option ouba",         CaseId::SOL_OPTION_OUBA,false);
  putIniCase(iniGrille,"steady option armageddon",   CaseId::SOL_OPTION_ARMAGEDDON,false);
  putIniCase(iniGrille,"steady option chicken",      CaseId::SOL_OPTION_CHICKEN,false);
  putIniCase(iniGrille,"steady option photo",        CaseId::SOL_OPTION_PHOTO,false);
  putIniCase(iniGrille,"steady option feu au cul",   CaseId::SOL_OPTION_FEU_CUL,false);
  putIniCase(iniGrille,"steady option super bombe",  CaseId::SOL_OPTION_SUPER_BOMBE,false);
  putIniCase(iniGrille,"steady option mine",         CaseId::SOL_OPTION_MINE,false);
  putIniCase(iniGrille,"steady option dynamite",     CaseId::SOL_OPTION_DYNAMITE,false);
  putIniCase(iniGrille,"steady option invincible",   CaseId::SOL_OPTION_INVINCIBLE,false);
  putIniCase(iniGrille,"steady option invisible",    CaseId::SOL_OPTION_INVISIBLE,false);
  putIniCase(iniGrille,"steady option pose mur",     CaseId::SOL_OPTION_POSE_MUR,false);
  putIniCase(iniGrille,"steady option surpuissance", CaseId::SOL_OPTION_SURPUISSANCE,false);
  putIniCase(iniGrille,"steady option win95",        CaseId::SOL_OPTION_WIN95,false);
  putIniCase(iniGrille,"steady option traverse mur", CaseId::SOL_OPTION_TRAVERSE_MUR,false);
  putIniCase(iniGrille,"steady option femme",        CaseId::SOL_OPTION_FEMME,false);
  putIniCase(iniGrille,"steady option teleport",     CaseId::SOL_OPTION_TELEPORT,false);
  putIniCase(iniGrille,"steady option resurrect",    CaseId::SOL_OPTION_RESURRECT,false);
  putIniCase(iniGrille,"steady option speed",        CaseId::SOL_OPTION_SPEED,false);

  putIniCase(iniGrille,"boue",                       CaseId::SOL_BOUE,false);
  */

  //remplir mur, options
  List< AuxStruct * > posMur;
  remplirOption(ini,numeroSection,"amount option bombe",posMur,CaseId::OPTION_BOMBE,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option flamme",posMur,CaseId::OPTION_FLAMME,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option ouba",posMur,CaseId::OPTION_OUBA,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option armageddon",posMur,CaseId::OPTION_ARMAGEDDON,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option chicken",posMur,CaseId::OPTION_CHICKEN,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option photo",posMur,CaseId::OPTION_PHOTO,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option feu au cul",posMur,CaseId::OPTION_FEU_CUL,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option super bombe",posMur,CaseId::OPTION_SUPER_BOMBE,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option mine",posMur,CaseId::OPTION_MINE,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option dynamite",posMur,CaseId::OPTION_DYNAMITE,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option invincible",posMur,CaseId::OPTION_INVINCIBLE,nbBonhomme);

  remplirOption(ini,numeroSection,"amount option invisible",posMur,CaseId::OPTION_INVISIBLE,nbBonhomme);

  remplirOption(ini,numeroSection,"amount option pose mur",posMur,CaseId::OPTION_POSE_MUR,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option surpuissance",posMur,CaseId::OPTION_SURPUISSANCE,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option win95",posMur,CaseId::OPTION_WIN95,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option traverse mur",posMur,CaseId::OPTION_TRAVERSE_MUR,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option femme",posMur,CaseId::OPTION_FEMME,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option teleport",posMur,CaseId::OPTION_TELEPORT,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option resurrect",posMur,CaseId::OPTION_RESURRECT,nbBonhomme);
  remplirOption(ini,numeroSection,"amount option speed",posMur,CaseId::OPTION_SPEED,nbBonhomme);

  //...
  remplirMur(ini,numeroSection,posMur,nbBonhomme);

  //initPos (en faisant gaffe que si sur mur alors niv==2)
  List<Bonhomme*> *lb=bonhommeContainerRef->getAllBonhomme();
  lb->place();
  int cpt=0;
  while(lb->walk()) {
    int j=ini.getWithDefault("[Fond]",numeroSection,
			      "init pos bonhomme j",cpt,0);
    int i=ini.getWithDefault("[Fond]",numeroSection,
			      "init pos bonhomme i",cpt,0);
    validatePos(j,i);
    lb->get()->getBonhomme20()->init(j,i,mult.j,mult.i,*this);
    cpt++;
  }

  armageddonActived=false;
  //  countScreenBlow=0;

  coeur->setBonhomme(0);

  femme.reinit();
  femme.unactivate();
}


void Grille::validatePos(int& j,int& i) throw(Exception)
{
  if(!traversable(1,j,i,false)) {
    i=0;
    j=0;
    while(!traversable(1,j,i,false)) {
      i++;
      if(i>=nbColumn()) {
	i=0;
	j++;
	if(j>=nbLine()) throw Exception(ERROR_NO_VALID_POS_BONHOMME);
      }
    }
  }
}

Grille::~Grille() throw()
{
  destroy();
  delete coeur;
}


int Grille::nbLine() const throw()
{
  if(!tab) return 0;
  return tab->nbLine();
}
int Grille::nbColumn() const throw()
{
  if(!tab) return 0;
  return tab->nbColumn();
}
int Grille::nbNiv() const throw()
{
  if(!tab) return 0;
  return tab->deep();
}


int Grille::virtualWalk() throw()
{
  List<AuxStruct*> toBeBlown; //algo impec
  List<Case *> toBePutDown; 

  actionAllBonhomme();
  //non:  PageableContainer::walk();
  //on supprime qd walk==0
  list.place();
  while(list.walk()) {
    if(list.get()->isActive()) {
      Case *c=(Case *)(list.get());
      int j,i,niv;
      c->getPos(niv,j,i);
      if(c->haveToActivateUnder()) {
	List<Case*> *l=(*tab)[niv][j][i];
	l->place();
	while(l->walk()) 
	  if(l->get()==c) {
	    while(l->walk()) l->get()->activate();
	    break;
	  }
      }
      if(c->haveToUnactivateUnder()) {
	List<Case*> *l=(*tab)[niv][j][i];
	l->place();
	while(l->walk()) 
	  if(l->get()==c) {
	    while(l->walk()) l->get()->unactivate();
	    break;
	}
      }
      if(!c->walk()) {
	
	if(c->toBeBlownUp()) {
	  AuxStruct *auxStruct=new AuxStruct;
	  auxStruct->j=j;
	  auxStruct->i=i;
	  auxStruct->niv=niv;
	  auxStruct->pui=c->getPuissance();
	  auxStruct->trav=c->getTraverseMur();
	  toBeBlown.put(auxStruct);
	}
	
	list.suppr();

	List<Case*> *l=(*tab)[niv][j][i];
	l->place();
	while(l->walk()) if(l->get()==c) { l->suppr(); break; }
	
	delete c;
	
      }
      else if(c->fallable() && niv==1) {

	bool put=true;

	List<Case *> *l=(*tab)[0][j][i];
	l->place();
	while(l->walk()) {
	  if(!l->get()->stackable()) put=false;
	  if(l->get()->restricted())
	    if(!c->restricted()) put=false;
	  if(l->get()->isActive())
	    if(!(l->get()->traversable(false))) put=false;
	}

	if(put) toBePutDown.put(c);
      }
    }
  }

  toBeBlown.place();
  while(toBeBlown.walk()) {
    blowUp(toBeBlown.get()->niv,toBeBlown.get()->j,toBeBlown.get()->i,
	   toBeBlown.get()->pui,toBeBlown.get()->trav);
    delete toBeBlown.get();
  }

  toBePutDown.place();
  while(toBePutDown.walk()) {
    Case *c=toBePutDown.get();
    int j,i,niv;
    c->getPos(niv,j,i);
    c->move(0,j,i,mult.j,mult.i);
    List<Case*> *l=(*tab)[1][j][i];
    l->place();
    while(l->walk()) if(l->get()==c) { l->suppr();break; }
    l=(*tab)[0][j][i];
    l->place();
    l->put(c);
  }

  if(armageddonActived) privateArmageddon();

  if(femme.isActive()) femme.walk();
  coeur->walk();


  return 1;
}


void Grille::keyAction(const Key& key) throw()
{
  femme.work(key);
}

bool Grille::add(Case *c,int niv,int j,int i) throw()
{
  if(!c || !tab) return false;

  c->move(niv,j,i,mult.j,mult.i);

  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  if(l->walk()) {
    if(!l->get()->stackable()) return false;
    if(l->get()->restricted())
      if(!c->restricted()) return false;
  }

  l->place();
  l->put(c);
  
  list.placeCurrentOnHead();
  list.put(c);
  return true;
}

bool Grille::add(CaseId::Id id,int niv,int j,int i,Bonhomme20 *bonhomme,bool callBlowUp)
  throw(Exception)
{
  if(!caseGestion) return false;
  if(callBlowUp && (id>=CaseId::FLAMME_CENTER && id<=CaseId::FLAMME_RIGHT_BOUT))
    conditionBlowUp(niv,j,i,false);
  Case *c=caseGestion->get(id,*this,bonhomme);
  if(c) return add(c,niv,j,i);
  else return true;
}


bool Grille::traversable(int niv,int j,int i,bool force) throw()
  //toutes les cases sur ce niveau
{
  if(!tab) return true;

  if(j<0 || i<0 || j>=nbLine() || i>=nbColumn()) return false;
  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  while(l->walk())
    if(l->get()->isActive()) {
      //%%      if(l->get()->traversable(false)) return true;
      if(!(l->get()->traversable(force))) return false;
    }
  return true;
}


bool Grille::high(int niv,int j,int i) throw()
  //la case en haut de ce niveau
{
  if(!tab) return true;


  if(j<0 || i<0 || j>=nbLine() || i>=nbColumn()) return false;
  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  while(l->walk())
    if(l->get()->isActive())
      if(l->get()->high()) return true;
  return false;
}

void Grille::action(int niv,int j,int i,Bonhomme20& bonhomme) throw()
  //toutes les cases sur ce niveau
{
  if(!tab) return;

  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  while(l->walk())
    if(l->get()->isActive())
    l->get()->action(bonhomme);
}

void Grille::tolerantAction(int niv,int j,int i,Bonhomme20& bonhomme) throw()
{
  if(!tab) return;

  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  while(l->walk())
    if(l->get()->isActive())
    l->get()->tolerantAction(bonhomme);
}

void Grille::actionAllBonhomme() throw()
{
  if(!tab) return;

  List<Bonhomme*> *lb=bonhommeContainerRef->getBonhommePlaying();
  lb->place();
  while(lb->walk()) {
    int niv;
    int j[4],i[4];
    //action sur niveaux (0=>0&1 1=>1&(0 si 1 traversable))
    //tolerantAction sur les 2 niveaux
    lb->get()->getBonhomme20()->getPosTolerantAction(niv,j,i);
    for(int k=0;k<4;k++)
      for(int n=0;n<nbNiv();n++)
	tolerantAction(n,j[k],i[k],*(lb->get()->getBonhomme20()));

    lb->get()->getBonhomme20()->getPosAction(niv,j,i);
    //%%    if(niv==0)
    for(int k=0;k<4;k++)
      for(int n=0;n<nbNiv();n++)
	action(n,j[k],i[k],*(lb->get()->getBonhomme20()));
    /*%%    else
      for(int k=0;k<4;k++) {
      action(1,j[k],i[k],*(lb->get()->getBonhomme20()));
      if(traversable(0,j[k],i[k],false))
      action(0,j[k],i[k],*(lb->get()->getBonhomme20()));
      }*/
  }
}

bool Grille::conditionBlowUp(int niv,int j,int i,bool traverseMur) throw()
{
  List<Case *> *l=(*tab)[niv][j][i];
  l->place();
  while(l->walk()) {
    /*
    bool b=l->get()->traversable(false); //oui oui (non :traverseMur)
    if(l->get()->blowable())
      if(traverseMur) return true;
      else return false;
    else if(!b) return false;
    */
    if(l->get()->blowable()) {
      if(traverseMur) return true;
      else return false;
    }
    else if(!l->get()->traversable(false)) return false;
  }
  return true;
}

void Grille::blowUp(int niv,int j,int i,int puissance,bool traverseMur)
  throw() //cross blow up
{
  if(!tab) return;

  if(trembleurRef) trembleurRef->inc(trembleExplosion.j,
				     trembleExplosion.i);


  add(CaseId::FLAMME_CENTER,niv,j,i,0,false);

  conditionBlowUp(niv,j,i,false);
  if(niv==1) conditionBlowUp(0,j,i,false);

  int _j,_i,cpt_pui;

  //UP
  _j=j-1;
  _i=i;
  cpt_pui=puissance;
  while(_j>=0 && cpt_pui>0
	&& conditionBlowUp(niv,_j,_i,traverseMur)) {
    if(cpt_pui>1) add(CaseId::FLAMME_UP,niv,_j,_i,0,false);
    else add(CaseId::FLAMME_UP_BOUT,niv,_j,_i,0,false);
    _j--;
    cpt_pui--;
  }
  
  //DOWN
  _j=j+1;
  _i=i;
  cpt_pui=puissance;
  while(_j<nbLine() && cpt_pui>0
	&& conditionBlowUp(niv,_j,_i,traverseMur)) {
    if(cpt_pui>1) add(CaseId::FLAMME_DOWN,niv,_j,_i,0,false);
    else add(CaseId::FLAMME_DOWN_BOUT,niv,_j,_i,0,false);
    _j++;
    cpt_pui--;
  }

  //LEFT
  _j=j;
  _i=i-1;
  cpt_pui=puissance;
  while(_i>=0 && cpt_pui>0
	&& conditionBlowUp(niv,_j,_i,traverseMur)) {
    if(cpt_pui>1) add(CaseId::FLAMME_LEFT,niv,_j,_i,0,false);
    else add(CaseId::FLAMME_LEFT_BOUT,niv,_j,_i,0,false);
    _i--;
    cpt_pui--;
  }

  //RIGHT
  _j=j;
  _i=i+1;
  cpt_pui=puissance;
  while(_i<nbColumn() && cpt_pui>0 
	&& conditionBlowUp(niv,_j,_i,traverseMur)) {
    if(cpt_pui>1) add(CaseId::FLAMME_RIGHT,niv,_j,_i,0,false);
    else add(CaseId::FLAMME_RIGHT_BOUT,niv,_j,_i,0,false);
    _i++;
    cpt_pui--;
  }
}

void Grille::remplirOption(Ini& ini,int numeroSection,const String& iniId,List<AuxStruct *>& posMur,CaseId::Id id,int nbBonhomme) throw()
{
  bool withStar;
  int nb=ini.getWithDefault("[Fond]",numeroSection,
			    iniId,0,
			    DEFAULT_AMOUNT_OPTION,
			    withStar);
  if(withStar) nb*=nbBonhomme;
  //options
  //  for(int i=0;i<CaseId::NB_OPTION;i++) {
  for(int cpt=0;cpt<nb;cpt++) { //Option[i];cpt++) {
    AuxStruct *pos=new AuxStruct;
    int max=MAX_TEST_REMPLIR;
    do {
      pos->j=rand()%nbLine();
      pos->i=rand()%nbColumn();
      max--;
      if(!max) { cerr<<"Impossible de poser toutes les options\n";return; }
    } while(!traversable(0,pos->j,pos->i,false));
    add(id,//(CaseId::Id)(CaseId::OPTION_BEGIN+i),
	0,pos->j,pos->i);
    posMur.put(pos);
  }
  //  }
}

void Grille::remplirMur(Ini& ini,int numeroSection,List<AuxStruct *>& posMur,int nbBonhomme) throw()
{
  bool withStar;
  int nbMurDestructible=ini.getWithDefault("[Fond]",numeroSection,
				       "amount mur destructible",0,
				       DEFAULT_AMOUNT_MUR_DESTRUCTIBLE,
				       withStar);
  if(withStar) nbMurDestructible*=nbBonhomme;
  int nbMurRefermable=ini.getWithDefault("[Fond]",numeroSection,
				     "amount mur refermable",0,
				     DEFAULT_AMOUNT_MUR_REFERMABLE,
				     withStar);
  if(withStar) nbMurRefermable*=nbBonhomme;

  //  List<AuxStruct *> posMur;

  posMur.place();
  int cptD=0,cptR=0;
  while(cptD+cptR<nbMurDestructible+nbMurRefermable) {
    int j,i;
    if(posMur.walk()) {
      j=posMur.get()->j;
      i=posMur.get()->i;
    }
    else {
      int max=MAX_TEST_REMPLIR;
      do {
	j=rand()%nbLine();
	i=rand()%nbColumn();
	max--;
	if(!max) { cerr<<"Impossible de poser tous les murs\n"; }
      } while(max && !traversable(0,j,i,false));
      if(!max) break;
    }

    if(cptD==nbMurDestructible) { 
      if(add(CaseId::MUR_REFERMABLE,0,j,i)) cptR++;
    }
    else if(cptR==nbMurRefermable) {
      if(add(CaseId::MUR_DESTRUCTIBLE,0,j,i)) cptD++;
    }
    else if(rand()%2) {
      if(add(CaseId::MUR_DESTRUCTIBLE,0,j,i)) cptD++;
    }
    else {
      if(add(CaseId::MUR_REFERMABLE,0,j,i)) cptR++;
    }
  }

  posMur.place();
  while(posMur.walk()) delete posMur.get();

}

void Grille::armageddon() throw()
{
  armageddonActived=true;
  countScreenBlow++;
  if(countScreenBlow==maxCountScreenBlow) {
    countScreenBlow=0;
    blowerRef->blowUp();
  }
}

void Grille::privateArmageddon() throw()
{
  armageddonActived=false;
  if(trembleurRef) trembleurRef->inc(trembleArmageddon.j,
				     trembleArmageddon.i);

  for(int j=0;j<nbLine();j++)
    for(int i=0;i<nbColumn();i++) {
      add(CaseId::FLAMME_CENTER,1,j,i,0,false);
      conditionBlowUp(0,j,i,false);
      conditionBlowUp(1,j,i,false);
    }

}

void Grille::photo() throw()
{
  paletteRef->flashToOther();
}

void Grille::runFemme(Bonhomme20& bonhomme) throw()
{
  coeur->setBonhomme(&bonhomme);
}

const Coeur *Grille::getCoeur() const throw()
{
  return coeur;
}

Trembleur *Grille::getTrembleur() throw()
{
  return trembleurRef;
}


void Grille::paint(WindowPipe& windowPipe) throw()
{
  if(nightActive) {
    /*
    if(nightSprite==0) {
      int jSize,iSize;
      int jPos,iPos;
      windowPipe.getTo(jPos,iPos,jSize,iSize);
      nightSprite=new Sprite(jSize,iSize);
      for(int kj=0;kj<nightSprite->nbLine();kj++)
	for(int ki=0;ki<nightSprite->nbColumn();ki++)
	  nightSprite->setWithoutShadow(0,0,0,kj,ki);
    }
    windowPipe.put(*nightSprite,0,0);
    */

    //the animations under the Bonhommes
    List<Pageable *> listUp;
    list.place();
    while(list.walk())
      if(list.get()->isActive()) {
	if(((Case*)(list.get()))->paintedUp()) listUp.put(list.get());
	else list.get()->paintNight(windowPipe);
      }
    //the Bonhommes
    List<Bonhomme*> *l=bonhommeContainerRef->getBonhommePlaying();
    l->place();
    while(l->walk())
      l->get()->getBonhomme20()->paintNight(windowPipe);
    //the animations on the Bonhommes
    listUp.place();
    while(listUp.walk()) listUp.get()->paintNight(windowPipe);
    return;
  }

  //the animations under the Bonhommes
  
  List<Pageable *> listUp;

  list.place();
  while(list.walk())
    if(list.get()->isActive()) {
      if(((Case*)(list.get()))->paintedUp()) listUp.put(list.get());
      else list.get()->paint(windowPipe);
    }

  //the Bonhommes

  List<Bonhomme*> *l=bonhommeContainerRef->getBonhommePlaying();
  l->place();
  while(l->walk())
    l->get()->getBonhomme20()->paint(windowPipe);

  //the animations on the Bonhommes

  listUp.place();
  while(listUp.walk()) listUp.get()->paint(windowPipe);

  //other things

  if(femme.isActive()) femme.paint(windowPipe);
  if(coeur->isActive()) coeur->paint(windowPipe);
}

void Grille::begin() throw()
{
  if(!caseGestion) return;
  sounderRef->play(caseGestion->getSoundBombeId());
  paletteRef->flashToColor();
}


void Grille::setNight(bool nightActive)throw()
{
  this->nightActive=nightActive;
}

bool Grille::isInNight()throw()
{
  return nightActive;
}

