/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Grille;

#ifndef __Grille_h__
#define __Grille_h__

#include "PageableContainer.h"
//#include "Case.h"
#include "CaseGestion.h"
#include "Key.h"
#include "Array3.h"
#include "Background.h"
#include "BonhommeContainer.h"
#include "FlashPalette.h"
#include "Sounder.h"
#include "Coeur.h"
#include "Trembleur.h"
#include "Blower.h"
#include "Tji.h"
#include "StripContainer.h"

#include "Femme.h"

// gestion complete des cases du jeu.
// gestion : tableau du niveaux, chaque niveau contient une liste de
// cases empilees selon les regles des cases
// (regles definies par les methodes 'stackable', 'restricted',
// 'high', etc... dans 'Case')

// Important : les 'Bonhomme20' ne gere pas l'interaction forcee avec la grille
// (par exemple, la prise d'option);
// au contraire, c'est 'Grille' et plus precisement 'Case',
// qui appellent les methodes du bonhomme,
// ce sont ces classes qui gerent ces interactions, et non 'Bonhomme20'.

/**
   Manages the square array of the game (Case).
   Manages the two levels, the options appearing, the destruction of animations when finished, the Femme (woman), the stacks of the Case.
   Manages also the interactions from Case to Bonhomme20 (in fact, the Case call the Bonhomme20 methods).
   @see Case
   @author David FAUTHOUX
 */
class Grille:public PageableContainer {
 private:
  ///
  struct AuxStruct {
    ///
    int j;
    ///
    int i;
    ///
    int niv;
    ///
    int pui;
    ///
    bool trav;
  };

 protected:

  ///Id used with CounterContainer
  enum IdCount {
    ///
    COUNT_ARMAGEDDON,
    ///
    COUNT_FEMME,
    ///
    COUNT_PHOTO,
    ///
    COUNT_CHICKEN
  };

  ///Grille creates a CaseGestion to load animations
  CaseGestion *caseGestion;
  ///Reference
  Sounder *const sounderRef;

  ///Reference
  BonhommeContainer *const bonhommeContainerRef;

  ///Reference (used for OptionPhoto)
  FlashPalette *const paletteRef;

  ///Each level contains a list of Case stacked
  Array3< List< Case * > * > *tab; //niv,j,i

  ///
  Trembleur *trembleurRef;

  ///hart
  Coeur *coeur; //permanent

  ///
  bool armageddonActived;
  ///
  int countScreenBlow;
  ///
  int maxCountScreenBlow;

  ///
  Blower *blowerRef;

  ///
  StripContainer *stripContainerRef;

  ///
  Tji mult;

  ///
  //  int nbMurDestructible;
  ///
  //  int nbMurRefermable;
  ///
  //  int nbOption[CaseId::NB_OPTION];

  ///
  Tji trembleExplosion;
  ///
  Tji trembleArmageddon;


  ///
  Femme femme;


  //

  ///
  void privateArmageddon() throw();

  ///
  bool add(Case *c,int j,int i,int niv) throw();
  //ATTENTION ! Case delete dans le destructor de Grille ici

  ///
  void blowUp(int niv,int j,int i,int puissance,bool traverseMur)
    throw(); //cross blow up

  ///Interacts Case with Bonhomme20
  void action(int niv,int j,int i,Bonhomme20& bonhomme) throw();
  ///
  void tolerantAction(int niv,int j,int i,Bonhomme20& bonhomme) throw();
  ///
  void actionAllBonhomme() throw();

  ///Initialises the Grille from .grille of Fond
  void remplirOption(Ini& ini,int numeroSection,const String& iniId,List< AuxStruct * >& posMur,CaseId::Id id,int nbBonhomme) throw();
  ///Initialises the Grille from .grille of Fond
  void remplirMur(Ini& ini,int numeroSection,List< AuxStruct * >& posMur,int nbBonhomme) throw();

  ///
  int virtual virtualWalk() throw();
  ///Checks if the Bonhomme are in the array
  void validatePos(int& j,int& i) throw(Exception);
  ///
  void destroy() throw();

  ///
  bool conditionBlowUp(int niv,int j,int i,bool traverseMur) throw();

  ///
  void endOfCount(int value) throw();



 public:
  ///
  Grille(Ini& ini,BonhommeContainer& bonhommeContainer,FlashPalette& palette,
	 Sounder& sounder,
	 Blower& blower,
	 StripContainer& stripContainer) throw();
  ///
  virtual ~Grille() throw();

  ///
  void setTrembleur(Trembleur& trembleur) throw();

  ///
  void putIniCase(Ini& ini,const String& s,CaseId::Id id,bool allNiv) throw(Exception);
  ///
  void create(Ini& ini,int numeroSection,int jBackSize,int iBackSize) throw(Exception);

  //

  ///
  int nbLine() const throw();
  ///
  int nbColumn() const throw();
  ///
  int nbNiv() const throw();

  ///Passes the key to the Femme to manage the Femme messages (see MessageKeeper)
  void virtual keyAction(const Key& key) throw();



  ///Use it to add a case to the Grille (id, level, vertical pos, horizontal pos, maybe Bonhomme is needed - example CaseMine). 'callBlowUp' == false forbids the effect of a flame on another 'Case'.
  bool add(CaseId::Id id,int niv,int j,int i,Bonhomme20 *bonhomme=0,bool callBlowUp=true)
    throw(Exception);

  ///If force==true, breaks the destructible walls
  bool traversable(int niv,int j,int i,bool force) throw();
  ///Level up ?
  bool high(int niv,int j,int i) throw();

  ///
  void armageddon() throw();

  ///
  void runFemme(Bonhomme20& bonhomme) throw();
  ///
  const Coeur *getCoeur() const throw();

  ///
  void photo() throw();

  //
  ///
  void paint(WindowPipe& windowPipe) throw();


  //

  ///
  Trembleur *getTrembleur() throw();

  //

  ///
  void begin() throw();


 protected:
  ///
  bool nightActive;
  ///
  Sprite * nightSprite;

 public:
  /// 
  virtual void setNight(bool nightActive) throw();
  /// 
  virtual bool isInNight() throw();
};


#endif
