/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "HFileModifier.h"

HFileModifier::HFileModifier(const String& className,
			     List<const String *>& markList) throw(Exception)
  :TextFileModifier(className+*new String(".h"),markList)
{
}

HFileModifier::~HFileModifier() throw()
{
}

bool HFileModifier::addInheritance(const String& protection,const String& motherClass) throw()
{
  place();

  do {
    if(!after("class")) {
      message("Cannot find 'class'");
      return false;
    }
    if(!after(*nextWord())) {
      message("Cannot find class name");
      return false;
    }
  } while(*nextWord()==";");

  if(*nextWord()=="{") {
    insert(*new String(":")+protection+*new String(" ")+motherClass);
    return true;
  }
  if(!after("{")) {
    message("Cannot find '{'");
    return false;
  }
  before("{");
  jumpBlank();
  insert(*new String(",")+protection+*new String(" ")+motherClass);
  return true;
}

bool HFileModifier::placeAtEnd() throw()
{
  place();

  //ONE class per file !!
  if(after("#endif") && before("};")) {
    jumpBlank();
    return true;
  }
  return false;
}

bool HFileModifier::placeAfterSharps() throw() //diese #
{
  place();
  if(!after("#define")) {
    message("Cannot find '#define'");
    return false;
  }
  if(!after(*nextWord())) {
    message("Cannot find __fileName_h__");
    return false;
  }

  for(;;) {
    if(*nextWord()=="#define") {
      after("#define");
      goEndLine();
    }
    else if(*nextWord()=="#include") {
      after("#include");
      goEndLine();
    }
    else break;
  }

  /*%%
  do {
    if(!after("class")) {
      message("Cannot find 'class'");
      return false;
    }
    if(!after(*nextWord())) {
      message("Cannot find class name");
      return false;
    }
  } while(*nextWord()==";");

  bool ret=before("class");
  jumpBlankInvers();
  return ret;
  */
  return true;
}

bool HFileModifier::addParameter(const String& protection,const String& type,const String& name,List<const String *>& comment) throw()
{
  if(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    l.put(new String("/**"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("*/"));
    l.put(&(type+*new String(" ")+name+*new String(";")));
    insert(l);
    return true;
  }
  return false;
}

bool HFileModifier::addFunction(const String& protection,const String& declaration,List<const String *>& comment) throw()
{
  if(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    l.put(new String("/**"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("*/"));
    l.put(&(declaration+*new String(";")));
    insert(l);
    return true;
  }
  return false;
}


bool HFileModifier::addDefine(const String& id,const String& value) throw()
{
  if(placeAfterSharps()) {
    insert(*new String("#define ")+id+*new String(" ")+value);
    return true;
  }
  return false;
}

bool HFileModifier::addInclude(const String& file) throw()
{
  if(placeAfterSharps()) {
    insert(*new String("#include \"")+file+*new String("\""));
    return true;
  }
  return false;
}


bool HFileModifier::findEnum(const String& enumName) throw()
{
  place();
  do {
    if(!after(*new String("enum ")+enumName)) {
      message(*new String("Cannot find 'enum' ")+enumName);
      return false;
    }
    /*
    if(!after("enum")) {
      message("Cannot find 'enum'");
      return false;
    }
    if(!after(*nextWord())) {
      message("Cannot find enum name");
      return false;
      }*/
  } while(*nextWord()==";");

  if(!after("{")) {
    message("Cannot find '{' of enum");
    return false;
  }

  return true;
}

bool HFileModifier::addInEnumAfter(const String& enumName,const String& id,List<const String *>& afterLines,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after(afterLines)) {
    List<const String *> l;
    l.put(new String("/**"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("*/"));
    l.put(&(id+*new String(",")));
    insert(l);
    return true;
  }
  return false;
}

bool HFileModifier::addInEnumBefore(const String& enumName,const String& id,List<const String *>& beforeLines,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after(beforeLines) && before(beforeLines)) {
    List<const String *> l;
    l.put(new String("/**"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("*/"));
    l.put(&(id+*new String(",")));
    insert(l);
    return true;
  }
  return false;
}

bool HFileModifier::addInEnumAtEnd(const String& enumName,const String& id,List<const String *>& comment) throw()
{
  if(findEnum(enumName) && after("};") && before("};")) {
    jumpBlank();
    List<const String *> l;
    l.put(new String("/**"));
    comment.place();
    while(comment.walk()) l.put(comment.get());
    l.put(new String("*/"));
    l.put(&(*new String(",")+id));
    insert(l);
    return true;
  }
  return false;
}

void HFileModifier::message(const String& m) throw()
{
  cerr<<H_FILE_MODIFIER_ERROR<<":"<<(const char*)m<<"\n";cerr.flush();
}


bool HFileModifier::addSubclass(const String& protection,List< const String * >& wallDeclaration) throw()
{
  if(placeAtEnd()) {
    List<const String *> l;
    l.put(&(protection+*new String(":")));
    /*%%
      l.put(new String("/ **"));
      comment.place();
      while(comment.walk()) l.put(comment.get());
      l.put(new String("* /"));
    */
    int cpt=0;
    wallDeclaration.place();
    while(wallDeclaration.walk()) cpt++;
    wallDeclaration.place();
    for(int k=0;k<cpt-1;k++) {
      wallDeclaration.walk();
      l.put(wallDeclaration.get());
    }
    wallDeclaration.walk();
    l.put(&(*wallDeclaration.get()+*new String(";")));
    insert(l);
    return true;
  }
  return false;
}


bool HFileModifier::addRoughlyAfter(List< const String * >& body,List< const String * >& afterLines) throw()
{
  place();
  if(after(afterLines)) {
    insert(body);
    return true;
  }
  return false;
}

bool HFileModifier::addRoughlyBefore(List< const String * >& body,List< const String * >& beforeLines) throw()
{
  place();
  if(after(beforeLines) && before(beforeLines)) {
    insert(body);
    return true;
  }
  return false;
}

bool HFileModifier::addRoughlyAtBeginning(List< const String * >& body) throw()
{
  place();
  insert(body);
  return true;
}

bool HFileModifier::addRoughlyAtEnd(List< const String * >& body) throw()
{
  place();
  while(walkIgnoring());
  insert(body);
  return true;
}
