/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include "Ini.h"
#include "File.h"
#include "error.h"
#include "util.h"

#define DEFAULT_LIST_SEPARATOR ","
#define DEFAULT_LANGUAGE "english"

Ini::Ini(const String& name) throw (Exception)
  :IniFileAnalyser(name),
   currentValueString(0),tmpId(0),currentListValueString(0),continueValue(false)
{
  analyse();
  listSeparator=(*getWithDefault("[Global]",0,
				 "list separator",0,
				 DEFAULT_LIST_SEPARATOR))[0];
  const String *s=getWithDefault("[Global]",0,"language",0,DEFAULT_LANGUAGE);
  if(*s=="francais") {
    language=KeyboardLanguage::FRENCH;
  }
  else if(*s=="english") {
    language=KeyboardLanguage::ENGLISH;
  } else {
    language=KeyboardLanguage::ENGLISH;
    set("[Global]",0,"language",0,DEFAULT_LANGUAGE);
  }
}

Ini::~Ini() throw ()
{
  section.place();
  while(section.walk()) delete section.get();
}


bool Ini::isPresentSection(const String& s,int numero) throw ()
{
  int cpt=0;
  section.place();
  while(section.walk()) {
    if(*(section.get()->getTitle())==s) {
      if(cpt==numero) return true;
      cpt++;
    }
  } 
  return false;
}

const String *Ini::get(const String& s,int numero,
		 const String& id,int idNumero) throw (Exception)
{
  return auxGet(s,numero,id,idNumero,true);
}
const String *Ini::getWithoutException(const String& s,int numero,
				 const String& id,int idNumero) throw (Exception)
{
  return auxGet(s,numero,id,idNumero,false);
}

const String *Ini::auxGet(const String& s,int numero,
		    const String& id,int idNumero,bool ex) throw (Exception)
{
  //chercher la section
  int cpt=0;
  section.place();
  while(section.walk()) {
    if(*(section.get()->getTitle())==s) {
      if(cpt==numero) {
	if(ex) return section.get()->get(id,idNumero);
	else return section.get()->getWithoutException(id,idNumero);
      }
      cpt++;
    }
  }
  if(ex) throw Exception(ERROR_SECTION_NOT_FOUND,s,id);
  else return 0;
}


void Ini::set(const String& s,int numero,const String& id,int idNumero,const String& val)
  throw (Exception)
{
  //chercher la section
  int cptS=0;
  section.place();
  while(section.walk()) {
    if(*(section.get()->getTitle())==s) {
      if(cptS==numero) {
	section.get()->set(id,idNumero,val);
	return;
      }
      cptS++;
    }
  }

  //mettre les sections
  while(cptS<=numero) {
    section.put(new SectionAssoc(s,*new String("")));
    cptS++;
  }

  section.get()->set(id,idNumero,val);
}

void Ini::setWithMaximumNumber(const String& s,int numero,const String& id,const String& val)
  throw (Exception)
{
  //chercher la section
  int cptS=0;
  section.place();
  while(section.walk()) {
    if(*(section.get()->getTitle())==s) {
      if(cptS==numero) {
	section.get()->setWithMaximumNumber(id,val);
	return;
      }
      cptS++;
    }
  }

  //mettre les sections
  while(cptS<=numero) {
    section.put(new SectionAssoc(s,*new String("")));
    cptS++;
  }

  section.get()->setWithMaximumNumber(id,val);
}


void Ini::setInAllSection(const String& s,const String& id,int idNumero,const String& val)
  throw (Exception)
{
  section.place();
  while(section.walk())
    if(*(section.get()->getTitle())==s)
      section.get()->set(id,idNumero,val);
}

const String *Ini::getWithDefault(const String& s,int numero,
				  const String& id,int idNumero,
				  const String& stringDefault) throw(Exception)
{
  const String *ret;
  ret=getWithoutException(s,numero,id,idNumero);
  if(!ret) {
    ret=&stringDefault;
    set(s,numero,id,idNumero,stringDefault);
  }
  return ret;
}

int Ini::getWithDefault(const String& s,int numero,
			const String& id,int idNumero,
			int intDefault) throw(Exception)
{
  return getWithDefault(s,numero,id,idNumero,*new String(intDefault))->toInt();
}

int Ini::getWithDefault(const String& s,int numero,
			const String& id,int idNumero,
			int intDefault,bool& withStar) throw(Exception)
{
  return getWithDefault(s,numero,id,idNumero,*new String(intDefault))
    ->toInt(withStar);
}


Pixel Ini::getWithDefaultColor(const String& s,int numero,
			       const String& id,int idNumero,
			       Pixel pixelDefault) throw(Exception)
{
  char aux[2]={listSeparator,0};
  String *r=new String(getRedFromPixel(pixelDefault));
  String *g=new String(getGreenFromPixel(pixelDefault));
  String *b=new String(getBlueFromPixel(pixelDefault));
  String *v=new String(aux);
  String *l=&(*r+*v+*g+*v+*b);
  delete v;
  delete r;
  delete g;
  delete b;

  const String *st=getWithDefault(s,numero,id,idNumero,*l);
  List<int> listInt;
  Pixel p;
  if(extractInt(*st,listSeparator,listInt,language) && listToPixel(listInt,p)) return p;

  set(s,numero,id,idNumero,*l);
  delete l;

  return pixelDefault;
}


List< int > *Ini::getWithDefaultIntList(const String& s,int numero,
					const String& id,int idNumero,
					const String& intListDefault) throw(Exception)
{
  const String *st=getWithDefault(s,numero,id,idNumero,intListDefault);
  List<int> *listInt=new List<int>;
  if(extractInt(*st,listSeparator,*listInt,language)) return listInt;

  set(s,numero,id,idNumero,intListDefault);

  extractInt(intListDefault,listSeparator,*listInt,language);

  return listInt;
}



List<const String*> *Ini::getList(const String& s,int numero,
			    const String& id,
			    const String& defaultSection,int numeroD)
  throw(Exception)
{
  int cpt=0;
  SectionAssoc *ref=0;
  section.place();
  while(section.walk())
    if(*(section.get()->getTitle())==defaultSection) {
      if(cpt==numeroD) {
	ref=section.get();
	break;
      } else cpt++;
    }
  if(!ref) throw Exception(ERROR_SECTION_NOT_FOUND,
			   defaultSection,*new String(numeroD));

  cpt=0;
  section.place();
  while(section.walk())
    if(*(section.get()->getTitle())==s) {
      if(cpt==numero) 
	return section.get()->getListWithDefaultSection(id,*ref);
      else cpt++;
    }
  throw Exception(ERROR_SECTION_NOT_FOUND,s,*new String(numero));
}


List<const String*> *Ini::getListWithoutException(const String& s,int numero,
						  const String& id)
  throw(Exception)
{
  int cpt=0;
  section.place();
  while(section.walk())
    if(*(section.get()->getTitle())==s) {
      if(cpt==numero) 
	return section.get()->getList(id);
      else cpt++;
    }
  throw Exception(ERROR_SECTION_NOT_FOUND,s,*new String(numero));
}



void Ini::putSection(const String& s) throw(Exception)
{
  section.put(new SectionAssoc(s,*new String("")));
  currentValueString=0;
  tmpId=0;
}

void Ini::putId(const String& s) throw(Exception)
{
  if(section.empty()) throw Exception(ERROR_ID_OUT_OF_SECTION,s);
  tmpId=&s;
}

void Ini::putValue(const String& s) throw(Exception)
{
  if(tmpId==0) throw Exception(ERROR_VALUE_AND_NOT_ID,s);
  currentValueString=new ValueString(*tmpId,s,*new String(""));
  section.get()->add(*currentValueString);
  tmpId=0;
}

void Ini::putComment(const String& s) throw(Exception)
{
  if(section.empty()) cerr<<"Ignoring comments beginning initialisation file\n";
  else {
    if(currentValueString==0) section.get()->setComment(s);
    else currentValueString->setComment(s);
  }
}


void Ini::initForSave() throw(Exception)
{
  section.place();
  currentValueString=0;
  continueValue=false;
}

const String *Ini::getSection() throw(Exception)
{
  return section.get()->getTitle();
}

bool Ini::isSectionUsed() throw()
{
  return section.get()->used();
}

bool Ini::walkSection() throw(Exception)
{
  if(!section.walk()) return false;
  currentListValueString=section.get()->getAll();
  currentListValueString->place();
  continueValue=false;
  return true;
}

const String *Ini::getId() throw(Exception)
{
  return currentListValueString->get()->getId();
}

bool Ini::isIdUsed() throw()
{
  return currentListValueString->get()->used;
}

bool Ini::walkId() throw(Exception)
{
  if(!currentListValueString->walk()) return false;
  continueValue=true;
  return true;
}

const String *Ini::getValue() throw(Exception)
{
  return currentListValueString->get()->getVal();
}

bool Ini::walkValue() throw(Exception)
{
  //une seule value par id
  if(continueValue) {
    continueValue=false;
    return true;
  }
  return false;
}

const String *Ini::getComment() throw(Exception)
{
  if(currentListValueString->atTheQueue())
    return section.get()->getComment();
  else
    return currentListValueString->get()->getComment();
}
