/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "IniFileAnalyser.h"
#include "File.h"
#include "util.h"

IniFileAnalyser::IniFileAnalyser(const String& _nameFile) throw()
  :nameFile(_nameFile)
{
}

void IniFileAnalyser::analyse() throw(Exception)
{
  File f(nameFile);

  while(!f.eof()) {
    String *s;
    f.read(s);
    list.put(s);
  }


  bool inSection=false;
  list.place();
  while(list.walk()) {
    String *s=supprStringLimiter(*list.get());
    if(isCloseSection(*s,inSection)) {
      inSection=false;
    }
    else if(isSection(*s,inSection)) {
      putSection(*extractSection(*s));
      inSection=true;
    }
    else if(isIdValue(*s,inSection)) {
      putId(*extractId(*s));
      putValue(*extractValue(*s));
    }
    delete s;
  }

  list.place();
  while(list.walk()) delete list.get();
}

IniFileAnalyser::~IniFileAnalyser() throw()
{
}

int IniFileAnalyser::getFirstNonBlank(const String& s) throw()
{
  if(s.size()==0) return -1;
  return 0;
  /*
  int k=0;
  for(;;) {
    if(k>=s.size()) return -1;
    if(!isBlank(s[k])) return k;
    k++;
  }
  */
}

bool IniFileAnalyser::isSection(const String& s,bool inSection) throw()
{
  int k=getFirstNonBlank(s);
  if(k<0) return false;
  if(s[k]=='#') return !inSection;
  for(;;) {
    if(k>=s.size()) return false;
    if(s[k]==':') return false;
    if(s[k]=='=') return true;
    k++;
  }
}

bool IniFileAnalyser::isCloseSection(const String& s,bool inSection) throw()
{
  if(!inSection) return false;
  int k=getFirstNonBlank(s);
  if(k<0) return false;
  if(k+1>=s.size()) return false;
  if(s[k]!='[' || s[k+1]!=']') return false;
  return true;
}

bool IniFileAnalyser::isIdValue(const String& s,bool inSection) throw()
{
  int k=getFirstNonBlank(s);
  if(k<0) return false;
  if(s[k]=='#') return inSection;
  for(;;) {
    if(k>=s.size()) return false;
    if(s[k]==':') return true;
    if(s[k]=='=') return false;
    k++;
  }
}

String *IniFileAnalyser::extractSection(const String& s) throw()
{
  List<char> l;
  int k=getFirstNonBlank(s);
  if(s[k]=='#') {
    while(k<s.size()) {
      l.put(s[k]);
      k++;
    }
    return new String(l);
  }
  for(;;) {
    if(s[k]=='=') return new String(l);
    l.put(s[k]);
    k++;
  }
}

String *IniFileAnalyser::extractId(const String& s) throw()
{
  List<char> l;
  int k=getFirstNonBlank(s);
  if(s[k]=='#') {
    while(k<s.size()) {
      l.put(s[k]);
      k++;
    }
    return new String(l);
  }
  for(;;) {
    if(s[k]==':') return new String(l);
    l.put(s[k]);
    k++;
  }
}

String *IniFileAnalyser::extractValue(const String& s) throw()
{
  List<char> l;
  int k=getFirstNonBlank(s);
  if(s[k]=='#') return new String("");
  bool beg=false;
  for(;;) {
    if(k>=s.size()) return new String(l);
    if(beg) l.put(s[k]);
    if(s[k]==':') beg=true;
    k++;
  }
}

String *IniFileAnalyser::supprStringLimiter(const String& s) throw()
{
  List<char> l;
  int k=0;
  while(k<s.size()) {
    if(k+1<s.size() && s[k]=='\\'
       && (s[k+1]=='\'' || s[k+1]=='\"' || s[k+1]=='`')) {
      k++;
    }
    l.put(s[k]);
    k++;
  }
  return new String(l);
}


String *IniFileAnalyser::completeStringLimiter(const String& s) throw()
{
  List<char> l;
  int k=0;
  while(k<s.size()) {
    if(s[k]=='\'' || s[k]=='\"' || s[k]=='`') {
      l.put('\\');
    }
    l.put(s[k]);
    k++;
  }
  return new String(l);
}

void IniFileAnalyser::save() throw(Exception)
{
  File f(nameFile,File::WRITE_ONLY);

  //%%%%%  f.write("# See this file in color : in emacs, type M-x makefile-mode\n");

  initForSave();
  while(walkSection()) {

    const String *s=getSection();
    bool com=false;
    if((*s)[0]=='#') com=true;

    if(com)
      f.write((const char*)(*completeStringLimiter(*s)+*new String("\n")));
    else 
      f.write((const char*)(*completeStringLimiter(*s)+*new String("=\n")));

    while(walkId()) {
      const String *s=getId();
      if((*s)[0]=='#')
	f.write((const char*)(*completeStringLimiter(*s)+*new String("\n")));
      else {
	walkValue();
	f.write((const char*)(*completeStringLimiter(*s)+*new String(":")+
			*completeStringLimiter(*getValue())+*new String("\n")));
      }
    }

    if(!com)
      f.write("[]\n");
  }
}

void IniFileAnalyser::saveCleaned() throw(Exception)
{
  String cleanedFile=nameFile;
  String uncleanedFile=nameFile+*new String(".uncleaned");
  nameFile=uncleanedFile;
  save();

  cout<<"Cleaning "<<(const char*)cleanedFile<<" file\n";cout.flush();

  File f(cleanedFile,File::WRITE_ONLY);
  

  //%%%%%  f.write("# See this file in color : in emacs, type M-x makefile-mode\n");

  int cpt=0;
  initForSave();
  while(walkSection()) {

    const String *s=getSection();
    
    bool com=false;
    if((*s)[0]=='#') com=true;
      
    if(com || isSectionUsed()) {
      
      if(com)
	f.write((const char*)(*s+*new String("\n")));
      else 
	f.write((const char*)(*completeStringLimiter(*s)+*new String("=\n")));
      
      while(walkId()) {
	const String *s=getId();
	bool idCom=false;
	if((*s)[0]=='#') idCom=true;

	if(idCom || isIdUsed()) {
	  if(idCom)
	    f.write((const char*)(*s+*new String("\n")));
	  else {
	    walkValue();
	    f.write((const char*)(*completeStringLimiter(*s)+*new String(":")+
			    *completeStringLimiter(*getValue())+*new String("\n")));
	  }
	}
	else {
	  cout<<"-> "<<(const char*)(*getSection())<<"#"<<cpt<<"\n";cout.flush();
	  cout<<"\t"<<(const char*)(*getId())<<":"<<(const char*)(*getValue())<<"\n";cout.flush();
	}

      }

      if(!com)
	f.write("[]\n");
    }
    else cout<<"-> "<<(const char*)(*getSection())<<"#"<<cpt<<"\n";cout.flush();
    if(!com) cpt++;
  }
}

