/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class IniFileAnalyser;

#ifndef __IniFileAnalyser_h__
#define __IniFileAnalyser_h__

#include "String.h"

//#define ERROR_HTML_FILE_ANALYSER "Error while analysing html file"

/**
   Analyse a text file in Ini form
   @see Ini
   @author David FAUTHOUX
 */
class IniFileAnalyser {
 protected:
  ///
  String nameFile;
  ///
  List< const String * > sectionList;
  ///
  List< const String * > list;
  ///
  int getFirstNonBlank(const String& s) throw();
  ///
  bool isSection(const String& s,bool inSection) throw();
  ///
  bool isCloseSection(const String& s,bool inSection) throw();
  ///
  bool isIdValue(const String& s,bool inSection) throw();
  ///
  String *extractSection(const String& s) throw();
  ///
  String *extractId(const String& s) throw();
  ///
  String *extractValue(const String& s) throw();
  ///
  String *supprStringLimiter(const String& s) throw();
  ///
  String *completeStringLimiter(const String& s) throw();

  ///
  virtual void putSection(const String& s) throw(Exception)=0;
  ///
  virtual void putId(const String& s) throw(Exception)=0;
  ///
  virtual void putValue(const String& s) throw(Exception)=0;
  ///
  virtual void putComment(const String& s) throw(Exception)=0;

  ///
  virtual void initForSave() throw(Exception)=0;
  ///
  virtual const String *getSection() throw(Exception)=0;
  ///
  virtual bool isSectionUsed() throw()=0;
  ///
  virtual bool walkSection() throw(Exception)=0;
  ///
  virtual const String *getId() throw(Exception)=0;
  ///
  virtual bool isIdUsed() throw()=0;
  ///
  virtual bool walkId() throw(Exception)=0;
  ///
  virtual const String *getValue() throw(Exception)=0;
  ///
  virtual bool walkValue() throw(Exception)=0;

  ///
  virtual const String *getComment() throw(Exception)=0;

  ///protected
  void analyse() throw(Exception);

 public:
  ///
  IniFileAnalyser(const String& nameFile) throw();
  ///
  virtual ~IniFileAnalyser() throw();
  ///
  virtual void save() throw(Exception);
  ///
  virtual void saveCleaned() throw(Exception);
};



#endif
