/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "LineFileModifier.h"
#include "File.h"
#include "util.h"

LineFileModifier::LineFileModifier(const String& nameFile,
				   List<const String *>& markList,
				   const String& commentSymbols) throw(Exception)
  :mark(&markList),commentMark(&commentSymbols),name(&nameFile)
{
  File f(nameFile);
  while(!f.eof()) {
    char *s=0;
    f.read(s);
    insertRaw(*new String(s));
  }
  list.place();
}

LineFileModifier::~LineFileModifier() throw()
{
  list.place();
  while(list.walk()) delete list.get();
}

void LineFileModifier::save() throw(Exception)
{
  save(*name);
}

void LineFileModifier::save(const String& nameFile) throw(Exception)
{
  File f(nameFile,File::WRITE_ONLY);
  //  if(list.empty()) return;
  list.place();
  while(list.walk()) {
    f.write((const char*)(*list.get()));
  }
}


void LineFileModifier::place() throw()
{
  list.place();
}

bool LineFileModifier::walk() throw()
{
  if(list.walk()) return true;
  return false;
}

bool LineFileModifier::walkInvers() throw()
{
  if(list.walkHeadToQueue()) return true;
  return false;
}

const String *LineFileModifier::current() const throw(Exception)
{
  return list.get();
}


bool LineFileModifier::after(List< const String * >& afterLines) throw()
{
  afterLines.place();
  if(!afterLines.walk()) return true;
  bool together=false;
  for(;;) {
    if(!walk()) return false;
    String *s0=simplifyBlank(*current());
    String *s1=simplifyBlank(*afterLines.get());
    if(*s0==*s1) together=true;
    else {
      together=false;
      afterLines.place();
      afterLines.walk();
    }
    if(together) if(!afterLines.walk()) return true;
    delete s0;
    delete s1;
  }
}

bool LineFileModifier::after(const String& afterLine) throw()
{
  String *s1=simplifyBlank(afterLine);
  for(;;) {
    if(!walk()) {
      delete s1;
      return false;
    }
    String *s0=simplifyBlank(*current());
    if(*s0==*s1) {
      delete s0;
      delete s1;
      return true;
    }
    delete s0;
  }
}

bool LineFileModifier::before(List< const String * >& beforeLines) throw()
{
  if(beforeLines.empty()) return true;
  beforeLines.placeCurrentOnHead();
  //%%  if(!beforeLines.walk()) return true;

  if(list.atTheQueue()) return false;

  bool together=false;
  for(;;) {
    String *s0=simplifyBlank(*current());
    String *s1=simplifyBlank(*beforeLines.get());
    if(*s0==*s1) together=true;
    else {
      together=false;
      beforeLines.placeCurrentOnHead();
      //%%      beforeLines.walkHeadToQueue();
    }
    if(together) if(!beforeLines.walkHeadToQueue()) {
      walkInvers();
      return true;
    }
    if(!walkInvers()) return false;
    delete s0;
    delete s1;
  }
}

bool LineFileModifier::before(const String& beforeLine) throw()
{
  if(list.atTheQueue()) return false;

  String *s1=simplifyBlank(beforeLine);
  for(;;) {
    String *s0=simplifyBlank(*current());
    if(*s0==*s1) {
      delete s0;
      delete s1;
      return true;
    }
    if(!walkInvers()) {
      delete s1;
      return false;
    }
    delete s0;
  }
}


void LineFileModifier::insert(List<const String*>& lines) throw()
{
  //  insertRaw(*new String("\n"));
  mark->place();
  while(mark->walk()) insertRaw(*(mark->get())+*new String("\n"));
  
  int cpt=0;
  lines.place();
  while(lines.walk()) cpt++;
  if(cpt<=1) insertRaw(*commentMark+*new String(cpt)+*new String(" line added\n"));
  else insertRaw(*commentMark+*new String(cpt)+*new String(" lines added\n"));
  
  lines.place();
  while(lines.walk()) insertRaw(*(lines.get())+*new String("\n"));
  
}

void LineFileModifier::insert(const String& line) throw()
{
  //  insertRaw(*new String("\n"));
  mark->place();
  while(mark->walk()) insertRaw(*(mark->get())+*new String("\n"));
  
  insertRaw(*commentMark+*new String("1 line added\n"));
  
  insertRaw(line+*new String("\n"));
}

void LineFileModifier::insertRaw(const String& line) throw()
{
  list.put(&line);
}


void LineFileModifier::suppress() throw()
{
  if(list.empty()) return;

  place();
  while(walk());

  while(before(*mark)) {
    
    //enlever le mark    
    after(*mark);

    mark->place();
    while(mark->walk()) {
      list.suppr();
    }
    list.walk();

    //enlever la ligne '//x line[s] added' et x lignes
    int cpt=0;
    const String *intLine=current();
    if(intLine->size()<commentMark->size() || (*intLine)[0]!=(*commentMark)[0]) {
      cerr<<ERROR_LINE_FILE_MODIFIER<<":Cannot know how many lines to suppress, continuing...\n";
      cerr.flush();
    }
    else {
      List<char> lint;
      for(int k=commentMark->size();k<intLine->size();k++) {
	if(isBlank((*intLine)[k])) break;
	lint.put((*intLine)[k]);
      }
      try {
	cpt=(new String(lint))->toInt();
	list.suppr();
	for(int k=0;k<cpt;k++) {
	  if(list.walk()) list.suppr();
	  else {
	    cerr<<ERROR_LINE_FILE_MODIFIER<<":when suppressing lines, continuing...\n";
	    cerr.flush();
	    break;
	  }
	}
	//	list.walk();
      } catch(Exception) {
	cerr<<"Continuing...\n";
	cerr.flush();
      }

      if(!list.atTheQueue()) {
      /*NON
	//enlever le \n a la fin de la ligne courante
	
	List<char> copyCurrent;
	for(int k=0;k<current()->size();k++) {
	  copyCurrent.put((*(current()))[k]);
	}
	
	copyCurrent.placeCurrentOnHead();
	if(copyCurrent.get()=='\n') {
	  copyCurrent.suppr();
	  list.suppr();
	  if(walk()) {
	    for(int k=0;k<current()->size();k++) {
	      copyCurrent.put((*(current()))[k]);
	    }
	    list.suppr();
	    list.put(new String(copyCurrent));
	  } else {
	    if(!copyCurrent.empty()) list.put(new String(copyCurrent));
	  }
	}
      */
	
	list.walk();
      }

    }
  }
}

void LineFileModifier::suppressLine() throw()
{
  list.suppr();
}
