/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */

#include <stdlib.h>

#include "error.h"
#include "String.h"

template <class T_item>
List<T_item>::
List() throw ()
  :queue(0),cur(0),l(0)
{
}

template <class T_item>
List<T_item>::
~List() throw ()
{
  cur=queue;
  Cell *save;
  while(cur!=0) {
    save=cur->last;
    delete cur;
    cur=save;
  }
}

template <class T_item>
List<T_item>::
List(List<T_item>& from) throw ()
  :queue(0),cur(0),l(from.l)
{
  from.place();
  while(from.walk()) put(from.get());
}

template <class T_item>
void List<T_item>::put(T_item item) throw ()
{
  struct Cell *n=new Cell;
  n->item=item;

  if(cur==0) {
    if(queue==0) {
      head=queue=cur=n;
      n->next=n->last=0;
    } else {
      queue->next=n;
      n->last=queue;
      n->next=0;
      queue=cur=n;
    }
  }
  else {
    if(cur==head) {
      n->last=0;
      head=n;
    } else {
      cur->last->next=n;
      n->last=cur->last;
    }
    n->next=cur;
    cur->last=n;
    cur=n;
  }

  l++;
}

template <class T_item>
T_item
List<T_item>::
get() const throw (Exception)
{
  if(cur==0) throw Exception(ERROR_LIST);
  return cur->item;
}

template <class T_item>
void List<T_item>::suppr() throw (Exception)
{
  if(cur==0) throw Exception(ERROR_LIST);
  if(cur==head && head==queue) {
    delete cur;
    queue=cur=0;
    l--;
    return;
  }
  if(cur==head) {
    head=cur->next;
    head->last=0;
    delete cur;
    cur=head;
    l--;
    return;
  }
  if(cur==queue) {
    queue=cur->last;
    queue->next=0;
    delete cur;
    cur=0;
    l--;
    return;
  }
  Cell *save=cur->next;
  cur->last->next=cur->next;
  cur->next->last=cur->last;
  delete cur;
  cur=save;
  l--;
}



template <class T_item>
void List<T_item>::place() throw ()
{
  cur=0;
}

template <class T_item>
void List<T_item>::place(int k) throw (Exception)
{
  if(k<0 || k>=l) throw Exception(ERROR_LIST);
  cur=0;
  place();
  walk();
  int cpt=0;
  while(cpt<k) {
    walk();
    cpt++;
  }
}

template <class T_item>
void List<T_item>::placeCurrentOnHead() throw ()
{
  if(queue==0) return;
  cur=head;
}


template <class T_item>
int
List<T_item>::
walk() throw ()
{
  if(queue==0) return 0;
  if(cur==head) return 0;
  if(cur==0) cur=queue;
  else cur=cur->last;
  return 1;
}

template <class T_item>
int List<T_item>::walkHeadToQueue() throw ()
{
  if(cur==0) return 0;
  cur=cur->next;
  if(cur==0) return 0;
  return 1;
}

template <class T_item>
List<T_item>&
List<T_item>::
operator=(List<T_item>& from) throw ()
{
  //detruire
  Cell *save;
  cur=queue;
  while(cur!=0) {
    save=cur->last;
    delete cur;
    cur=save;
  }
  queue=cur=0;

  //copier
  from.place();
  while(from.walk()) put(from.get());
  l=from.l;
  return *this;
}

template <class T_item>
bool List<T_item>::equal(List<T_item>& from) throw ()
{
  if(from.l!=l) return 0;
  place();
  from.place();
  while(from.walk()) {
    if(!walk()) return 0;
    if(get()!=from.get()) return 0;
  }
  if(walk()) return 0;
  return 1;
}


template <class T_item>
int List<T_item>::empty() const throw ()
{
  return queue==0;
}

template <class T_item>
bool List<T_item>::atTheQueue() const throw ()
{
  return cur==0;
}

template <class T_item>
bool List<T_item>::atTheHead() const throw ()
{
  return cur==head;
}


template <class T_item>
int List<T_item>::length() const throw()
{
  return l;
}


template <class T_item>
T_item List<T_item>::getRandomly() throw (Exception)
{
  if(l==0) throw Exception(ERROR_LIST);
  return (*this)[rand()%l];
}


template <class T_item>
T_item List<T_item>::operator[](int k) throw(Exception)
{
  place(k);
  return get();
}
