/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Mot.h"
#include "CharacterMot.h"

#define DEFAULT_SPACING_OUT 30

Mot::Mot(Ini& ini,const String& m,
	 int jInitPos,int iInitPos,
	 Pixel begColor,Pixel endColor,
	 int keyBlowUp,Sounder& sounder,int soundBlowUp) throw(Exception)
  :PageableContainer(0),wordMot(m)
{
  construct(ini,m,jInitPos,iInitPos,begColor,endColor,keyBlowUp,
	    &sounder,soundBlowUp);
}

Mot::Mot(Ini& ini,const String& m,
	 int jInitPos,int iInitPos,
	 Pixel begColor,Pixel endColor) throw(Exception)
  :PageableContainer(0),wordMot(m)
{
  construct(ini,m,jInitPos,iInitPos,begColor,endColor,0,0,0);
}

Mot::Mot(Ini& ini,const String& m,
	 int jInitPos,int iInitPos,
	 Pixel color,
	 int keyBlowUp,Sounder& sounder,int soundBlowUp) throw(Exception)
  :PageableContainer(0),wordMot(m)
{
  construct(ini,m,jInitPos,iInitPos,color,keyBlowUp,
	    &sounder,soundBlowUp);
}

Mot::Mot(Ini& ini,const String& m,
	 int jInitPos,int iInitPos,
	 Pixel color) throw(Exception)
  :PageableContainer(0),wordMot(m)
{
  construct(ini,m,jInitPos,iInitPos,color,0,0,0);
}

void Mot::construct(Ini& ini,const String& m,
		    int jInitPos,int iInitPos,
		    Pixel begColor,Pixel endColor,
		    int keyBlowUp,Sounder *sounder,int soundBlowUp) throw(Exception)
{
  sounderRef=sounder;
  soundBlowUpRef=soundBlowUp;
  keyCodeBlowUp=keyBlowUp;

  spacingOut=ini.getWithDefault("[Character]",0,
				"spacing out",0,
				DEFAULT_SPACING_OUT);

  for(int i=0;i<m.size();i++)
    add(*new CharacterMot(ini,m[i],toPixel(0,0,0,0)));

  setColor(begColor,endColor);

  move(jInitPos,iInitPos);
  blowedUp=false;
}

void Mot::construct(Ini& ini,const String& m,
		    int jInitPos,int iInitPos,
		    Pixel color,
		    int keyBlowUp,Sounder *sounder,int soundBlowUp) throw(Exception)
{
  sounderRef=sounder;
  soundBlowUpRef=soundBlowUp;
  keyCodeBlowUp=keyBlowUp;

  spacingOut=ini.getWithDefault("[Character]",0,
				"spacing out",0,
				DEFAULT_SPACING_OUT);

  for(int i=0;i<m.size();i++)
    add(*new CharacterMot(ini,m[i],toPixel(0,0,0,0)));

  setColor(color);

  move(jInitPos,iInitPos);
  blowedUp=false;
}

Mot::~Mot() throw()
{}

void Mot::setColor(Pixel begColor,Pixel endColor) throw()
{
  int cpt=0;
  list.place();
  while(list.walk()) cpt++;

  if(cpt==0) return;

  int rBeg=getRedFromPixel(begColor);
  int gBeg=getGreenFromPixel(begColor);
  int bBeg=getBlueFromPixel(begColor);

  int rEnd=getRedFromPixel(endColor);
  int gEnd=getGreenFromPixel(endColor);
  int bEnd=getBlueFromPixel(endColor);

  int rInc=(rEnd-rBeg)/cpt;
  int gInc=(gEnd-gBeg)/cpt;
  int bInc=(bEnd-bBeg)/cpt;
  int r=rBeg;
  int g=gBeg;
  int b=bBeg;
  list.place();
  while(list.walk()) {
    if(r<minRedValue) r=minRedValue;
    if(g<minRedValue) g=minRedValue;
    if(b<minRedValue) b=minRedValue;
    if(r>maxRedValue) r=maxRedValue;
    if(g>maxRedValue) g=maxRedValue;
    if(b>maxRedValue) b=maxRedValue;
    ((CharacterMot*)(list.get()))->setColor(toPixel(r,g,b,0));
    r+=rInc;
    g+=gInc;
    b+=bInc;
  }
}

void Mot::setColor(Pixel color) throw()
{
  int r=getRedFromPixel(color)/2;
  int g=getGreenFromPixel(color)/2;
  int b=getBlueFromPixel(color)/2;
  setColor(color,toPixel(r,g,b,0));
}

void Mot::move(int j,int i) throw()
{
  pos.j=j;
  pos.i=i;
  list.place();
  while(list.walk()) {
    ((CharacterMot*)list.get())->move(j,i);
    i+=spacingOut;
  }
}

void Mot::getPos(int& j,int& i) const throw()
{
  j=pos.j;
  i=pos.i;
}

void Mot::keyAction(const Key& key) throw()
{
  if(key.isDown() && key.getCode()==keyCodeBlowUp) blowUp();
}

void Mot::blowUp() throw()
{
  if(blowedUp) return;
  blowedUp=true;
  list.place();
  while(list.walk()) ((CharacterMot *)(list.get()))->blowUp();
  if(sounderRef) sounderRef->play(soundBlowUpRef);
}

void Mot::reinit() throw()
{
  PageableContainer::reinit();
  blowedUp=false;
}

Mot::operator const String*() const throw()
{
  return &wordMot;
}
