/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class Mot;

#ifndef __Mot_h__
#define __Mot_h__

#include "Ini.h"
#include "PageableContainer.h"
#include "Movable.h"
#include "Key.h"
#include "Sounder.h"
#include "Tji.h"

/**
   A "Mot" (word) contains CharacterMot and manages them
   @see CharacterMot
   @author David FAUTHOUX
 */
class Mot:public PageableContainer,public Movable {
 protected:
  ///Word saved in String form
  String wordMot;
  ///
  Tji pos;
  ///
  Sounder *sounderRef;
  ///
  int soundBlowUpRef;
  ///
  int spacingOut;
  ///
  int keyCodeBlowUp;
  ///
  bool blowedUp;

  ///
  void construct(Ini& ini,const String& m,
		 int jInitPos,int iInitPos,
		 Pixel begColor,Pixel endColor,
		 int keyBlowUp,Sounder *sounder,int soundBlowUp) throw(Exception);
  ///
  void construct(Ini& ini,const String& m,
		 int jInitPos,int iInitPos,
		 Pixel color,
		 int keyBlowUp,Sounder *sounder,int soundBlowUp) throw(Exception);
 public:
  ///
  Mot(Ini& ini,const String& m,
      int jInitPos,int iInitPos,
      Pixel begColor,Pixel endColor,
      int keyBlowUp,Sounder& sounder,int soundBlowUp) throw(Exception);
  ///
  Mot(Ini& ini,const String& m,
      int jInitPos,int iInitPos,
      Pixel begColor,Pixel endColor) throw(Exception); //do not blow up automatically
  ///
  Mot(Ini& ini,const String& m,
      int jInitPos,int iInitPos,
      Pixel color,
      int keyBlowUp,Sounder& sounder,int soundBlowUp) throw(Exception);
  ///
  Mot(Ini& ini,const String& m,
      int jInitPos,int iInitPos,
      Pixel color) throw(Exception); //do not blow up automatically
  ///
  virtual ~Mot() throw ();
  ///
  virtual void blowUp() throw ();
  ///
  virtual void move(int j,int i) throw();
  ///
  virtual void getPos(int& j,int& i) const throw();
  ///
  virtual void keyAction(const Key& key) throw();
  ///
  virtual void reinit() throw();
  ///
  virtual void setColor(Pixel begColor,Pixel endColor) throw();
  ///Use this color (begin) and the middle of this color (end)
  virtual void setColor(Pixel color) throw();

  ///
  virtual operator const String*() const throw();
};

#endif
