/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class MultiAnim;

#ifndef __MultiAnim_h__
#define __MultiAnim_h__

#include "Pageable.h"
#include "Movable.h"
#include "Anim.h"
#include "Tji.h"

// 'MultiAnim' gere plusieurs 'Anim' en les excluant :
// une seule animation active en meme temps.

/**
   Manages many animations excluding themselves : one animation is painted and managed at a time.
   For more powerful, see MoreMultiAnim
   @see Anim
   @author David FAUTHOUX   
 */
class MultiAnim:public Pageable,public Movable {
 protected:
  ///
  Anim **tab;
  ///
  int length;
  ///
  Tji pos;
  ///
  Anim *activeAnim;
  ///
  Pixel color;
  ///
  bool colored;
  ///
  bool _on;
  ///
  bool to;
  ///
  bool under;
  ///
  const PixelPalette *pal;
  ///
  int currentAnim;
  ///
  bool uppest;

  ///
  int virtual virtualWalk() throw();

 public:
  ///
  MultiAnim() throw();
  ///
  virtual ~MultiAnim() throw();

  ///
  void virtual setOn() throw();
  ///
  void virtual setUppest() throw();
  ///
  void virtual setTo() throw();
  ///
  void virtual setUnder() throw();
  ///
  void virtual setPalette(const PixelPalette& pal) throw();

  ///
  void virtual paint(WindowPipe& windowPipe) throw();
  ///
  void virtual paintNight(WindowPipe& windowPipe) throw();
  ///
  void virtual reinit() throw();
  ///
  void virtual keyAction(const Key& key) throw();
  ///
  void virtual move(int j,int i) throw();
  ///
  void virtual getPos(int& j,int& i) const throw();


  ///Returns the id of the animation
  int virtual add(Anim& anim) throw(); //renvoie l'id
  // ATTENTION ! delete des Anim fait par le destructor de MultiAnim

  ///Activate the animations "id" (id returns by add())
  void virtual on(int id) throw (Exception);

  ///
  Anim *getAnim(int id) throw (Exception);

  ///
  void virtual setColor(Pixel col) throw();
  ///
  void virtual restoreColor() throw();

  /*
  ///
  void virtual changeColor(List< Pixel >& colorFrom,List< Pixel >& colorTo) throw ();
  */

  ///
  int getCurrent() throw();

  ///
  void getSize(int& j,int& i) const throw();
};

#endif
