/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <iostream.h>
#include "Page.h"
#include "Mot.h"
#include "LoopAnim.h"
#include "ShakeAnim.h"

#define DEFAULT_MOT_POS_J 0
#define DEFAULT_MOT_POS_I 0
#define DEFAULT_MOT_COLOR toPixel(0,0,0,0)

#define DEFAULT_LOOPANIM_SLOW 0
#define DEFAULT_LOOPANIM_POS_J 0
#define DEFAULT_LOOPANIM_POS_I 0

#define DEFAULT_SHAKEANIM_SLOW 0
#define DEFAULT_SHAKEANIM_POS_J 0
#define DEFAULT_SHAKEANIM_POS_I 0
#define DEFAULT_SHAKE_J 3
#define DEFAULT_SHAKE_I 3

#define DEFAULT_DECAL_SHADOW_J 15
#define DEFAULT_DECAL_SHADOW_I 15

Page::Page(Ini& ini,int numeroSection,GraphicMode& graphicMode,
	   Sounder& sounder,int soundBlowUpMotId,
	   StripContainer& stripContainer) throw(Exception)
  :Painter(*new Background(ini.getWithoutException("[Page]",numeroSection,
						   "background",0),
			   graphicMode,
			   ini.getWithDefault("[Page]",numeroSection,
					      "decal shadow j",0,DEFAULT_DECAL_SHADOW_J),
			   ini.getWithDefault("[Page]",numeroSection,
					      "decal shadow i",0,DEFAULT_DECAL_SHADOW_I))
	   ,0),
   sounderRef(&sounder),
   stripContainerRef(&stripContainer),
   soundBlowUpId(soundBlowUpMotId),
   totalyGone(true)
{
  int cpt;

  cpt=0;
  const String *m;
  while((m=ini.getWithoutException("[Page]",numeroSection,"mot",cpt))) {
    add(*new Mot(ini,*m,
		 ini.getWithDefault("[Page]",numeroSection,
				    "pos mot j",cpt,
				    DEFAULT_MOT_POS_J),
		 ini.getWithDefault("[Page]",numeroSection,
				    "pos mot i",cpt,
				    DEFAULT_MOT_POS_I),
		 ini.getWithDefaultColor("[Page]",numeroSection,
					 "color begin",cpt,
					 DEFAULT_MOT_COLOR),
		 ini.getWithDefaultColor("[Page]",numeroSection,
					 "color end",cpt,
					 DEFAULT_MOT_COLOR),
		 ini.getWithDefault("[Page]",numeroSection,
				    "blow up on key",cpt,
				    0),
		 *sounderRef,
		 soundBlowUpId));
    cpt++;
  }

  cpt=0;
  while((m=ini.getWithoutException("[Page]",numeroSection,"loop anim",cpt))) {
    Anim *a=new LoopAnim(*m,ini.getWithDefault("[Page]",numeroSection,
					       "loop anim slow",cpt,
					       DEFAULT_LOOPANIM_SLOW),
			 ini.getListSeparator());
    a->move(ini.getWithDefault("[Page]",numeroSection,
			       "pos loop anim j",cpt,
			       DEFAULT_LOOPANIM_POS_J),
	    ini.getWithDefault("[Page]",numeroSection,
			       "pos loop anim i",cpt,
			       DEFAULT_LOOPANIM_POS_I));
    a->setOn();
    add(*a);
    cpt++;
  }

  cpt=0;
  while((m=ini.getWithoutException("[Page]",numeroSection,"shake anim",cpt))) {
    Anim *a=new ShakeAnim(*m,
			  ini.getWithDefault("[Page]",numeroSection,
					     "shake anim slow",cpt,
					     DEFAULT_SHAKEANIM_SLOW),
			  ini.getListSeparator(),
			  ini.getWithDefault("[Page]",numeroSection,
					     "shake j",cpt,
					     DEFAULT_SHAKE_J),
			  ini.getWithDefault("[Page]",numeroSection,
					     "shake i",cpt,
					     DEFAULT_SHAKE_I));
    a->move(ini.getWithDefault("[Page]",numeroSection,
			       "pos shake anim j",cpt,
			       DEFAULT_SHAKEANIM_POS_J),
	    ini.getWithDefault("[Page]",numeroSection,
			       "pos shake anim i",cpt,
			       DEFAULT_SHAKEANIM_POS_I));
    a->setOn();
    add(*a);
    cpt++;
  }
  
  stripList=ini.getListWithoutException("[Page]",numeroSection,"strip");

}
  
Page::~Page() throw()
{
}

void Page::keyAction(const Key& key) throw()
{
  if(isTotalyGone()) {
    Painter::keyAction(key);
    work(key);
  }
}

void Page::reinit() throw()
{
  totalyGone=false;
  Painter::reinit();

  //strips
  stripList->place();
  while(stripList->walk())
    stripContainerRef->add(*stripList->get());
}

void Page::setTotalyGone() throw()
{
  totalyGone=true;
}

bool Page::isTotalyGone() const throw()
{
  return totalyGone;
}

void Page::goOut() throw()
{
  totalyGone=false;
}

int Page::virtualWalk() throw()
{
  return Painter::virtualWalk();
}


void Page::action(int idAction) throw()
{
  switch(idAction) {
  default:
    break;
  }
}
