/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <limits.h>
#include "PageBegin.h"

#define DEFAULT_ANIM_SLOW 5

#define DEFAULT_POS_ANIM_J 0
#define DEFAULT_POS_ANIM_I 0
#define DEFAULT_VIT_ANIM_J 0
#define DEFAULT_VIT_ANIM_I 1
#define DEFAULT_REBOND_J 200
#define DEFAULT_REBOND_I INT_MAX
#define DEFAULT_AMORT_J 10
#define DEFAULT_AMORT_I 10

#define DEFAULT_TIME_WAIT 2000

PageBegin::PageBegin(Ini& ini,int numeroSection,
		     GraphicMode& graphicMode,
		     Grille& grille,
		     Sounder& sounder,int soundBlowMotId,
		     StripContainer& stripContainer) throw(Exception)
  :Page(ini,numeroSection,graphicMode,sounder,soundBlowMotId,stripContainer),
   grilleRef(&grille),go(false),
   a(new Anim(*ini.get("[Page]",numeroSection,"anim begin",0),
	      ini.getWithDefault("[Page]",numeroSection,
				 "anim begin slow",0,DEFAULT_ANIM_SLOW),
	      ini.getListSeparator()))
{
  a->move(ini.getWithDefault("[Page]",numeroSection,"pos anim begin j",
			     0,DEFAULT_POS_ANIM_J),
	  ini.getWithDefault("[Page]",numeroSection,"pos anim begin i",
			     0,DEFAULT_POS_ANIM_I));
  vit.j=ini.getWithDefault("[Page]",numeroSection,"vitesse anim begin j",
			   0,DEFAULT_VIT_ANIM_J);
  vit.i=ini.getWithDefault("[Page]",numeroSection,"vitesse anim begin i",
			   0,DEFAULT_VIT_ANIM_I);
  rebond.j=ini.getWithDefault("[Page]",numeroSection,"rebond anim begin j",
			      0,DEFAULT_REBOND_J);
  rebond.i=ini.getWithDefault("[Page]",numeroSection,"rebond anim begin i",
			      0,DEFAULT_REBOND_I);
  amort.j=ini.getWithDefault("[Page]",numeroSection,"amort anim begin j",
			     0,DEFAULT_AMORT_J);
  amort.i=ini.getWithDefault("[Page]",numeroSection,"amort anim begin i",
			     0,DEFAULT_AMORT_I);
  CounterContainer::add(ini.getWithDefault("[Page]",numeroSection,
					   "time wait",0,DEFAULT_TIME_WAIT),0);
}

PageBegin::~PageBegin() throw()
{
  delete a;
}

int PageBegin::virtualWalk() throw()
{
  inc();
  int ret=1;
  if(go && a->isActive()) {
    int j,i,sj,si;
    a->getPos(j,i);
    a->getSize(sj,si);
    vit.j++;
    j+=vit.j;
    i+=vit.i;
    if(j+sj>=rebond.j) {
      j=rebond.j-sj;
      if(vit.j<=amort.j) vit.j=0;
      else vit.j-=amort.j;
      vit.j=-vit.j;

      if(vit.i>0) 
	if(vit.i<=amort.i) vit.i=0;
	else vit.i-=amort.i;
      else 
	if(-vit.i<=amort.i) vit.i=0;
	else vit.i+=amort.i;
    }
    if(i+si>=rebond.i) {
      i=rebond.i-si;
      if(vit.i<=amort.i) vit.i=0;
      else vit.i-=amort.i;
      vit.i=-vit.i;
    }
    a->move(j,i);
    
    if(!a->walk()) {
      grilleRef->begin();
      ret=0;
    }
  }
  Page::virtualWalk();

  return ret;
}


void PageBegin::paint() throw()
{
  Page::paint();
  if(go && a->isActive()) a->paint(*(background->getWindowPipe()));
}

void PageBegin::endOfCount(int) throw()
{
  go=true;
}
