/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <stdlib.h>
#include "PageChanger.h"
#include "error.h"

#include "PageDefault.h"
#include "PageBombOBomb.h"
#include "PageOption.h"
#include "PageConfigKey.h"
#include "PageBonhomme.h"
#include "PageChoiceFond.h"
#include "PageGame.h"
#include "PagePause.h"
#include "PageScore.h"
#include "PageBegin.h"
#include "PageWin95.h"

#define KEY_AUTOCHANGE 0


//%%PageChanger::AuxStruct::AuxStruct(Couple *c,int i) throw():couple(c),cpt(i) {}

/*
1) creer toutes les pages en fonction de id (bomb o bomb => PageBombOBomb)
2) pour toutes les sections [Page], rechercher la page dans Couple grace a id
3) creer la corresp avec Couple::add
verifier que l'id existe dans les pages
*/

void PageChanger::constructCouple(Walkable& allTheGame,Keyer& keyer,bool clean)
  throw(Exception)
{
  cout<<"Loading Pages...";cout.flush();
      
  PageGame *pageGame=0;
  PageChoiceFond *pageFond=0;
  List<AuxStruct*> aux;
  int cpt=0;
  const String *id;
  while(iniRef->isPresentSection("[Page]",cpt)) {
    id=iniRef->getWithoutException("[Page]",cpt,"id",0);
    if(id) {
      Page *page;

      cout<<"*";cout.flush();

      if(*id=="begin")
	page=new PageBegin(*iniRef,cpt,*graphicModeRef,*grille,
			   *sounderRef,soundBlowUpMotId,
			   *stripContainerRef);
      else if(*id=="bomb o bomb")
	page=new PageBombOBomb(*iniRef,cpt,*graphicModeRef,*bonhommeContainer,
			       *sounderRef,soundBlowUpMotId,
			       *stripContainerRef);
      else if(*id=="option")
	page=new PageOption(*iniRef,cpt,*graphicModeRef,*bonhommeContainer,
			    allTheGame,*sounderRef,soundBlowUpMotId,
			    *stripContainerRef);
      else if(*id=="bonhomme")
	page=new PageBonhomme(*iniRef,cpt,*graphicModeRef,*bonhommeContainer,
			      *sounderRef,soundBlowUpMotId,
			      *stripContainerRef);
      else if(*id=="fond") {
	pageFond=new PageChoiceFond(*iniRef,cpt,*graphicModeRef,
				    *sounderRef,soundBlowUpMotId,
				    *stripContainerRef);
	page=pageFond;
      }
      else if(*id=="pause")
	page=new PagePause(*iniRef,cpt,*graphicModeRef,
			   *sounderRef,soundBlowUpMotId,
			   *stripContainerRef);
      else if(*id=="score")
	page=new PageScore(*iniRef,cpt,*graphicModeRef,*bonhommeContainer,
			   *sounderRef,soundBlowUpMotId,
			   *stripContainerRef);
      else if(*id=="config key")
	page=new PageConfigKey(*iniRef,cpt,*graphicModeRef,*bonhommeContainer,
			       *sounderRef,soundBlowUpMotId,
			       *stripContainerRef);
      else if(*id=="game") {
	pageGame=new PageGame(*iniRef,cpt,*graphicModeRef,
			      *bonhommeContainer,*grille,
			      *sounderRef,soundBlowUpMotId,
			      *stripContainerRef,
			      *redirectorRef);
	page=pageGame;
      }
      else if(*id=="crash win95")
	page=new PageWin95(*iniRef,cpt,*graphicModeRef,
			   *sounderRef,soundBlowUpMotId,keyer,
			   *stripContainerRef);
      else 
	page=new PageDefault(*iniRef,cpt,*graphicModeRef,
			     *sounderRef,soundBlowUpMotId,
			     *stripContainerRef);
      
      PageChangerCouple *couple=new PageChangerCouple(page,*id);
      listCouple.put(couple);
      AuxStruct *auxStruct=new AuxStruct;
      auxStruct->couple=couple;
      auxStruct->cpt=cpt;
      aux.put(auxStruct);
    }
    cpt++;
  }

  if(pageGame) {
    if(pageFond) pageFond->setPageGame(*pageGame,clean);
    else {
      int cpt=0;
      bool ok=false;
      while(!ok && iniRef->isPresentSection("[Fond]",cpt)) {
	try {
	  pageGame->setNumeroFond(cpt);
	  ok=true;
	} catch(Exception) {
	}
	cpt++;
      }
      if(!ok) {
	PageChangerCouple *couple=0;
	listCouple.place();
	while(listCouple.walk()) {
	  if(listCouple.get()->getPage()==pageGame) {
	    couple=listCouple.get();
	    delete listCouple.get();
	    listCouple.suppr();
	    break;
	  }
	}
	aux.place();
	while(aux.walk()) {
	  if(aux.get()->couple==couple) {
	    delete aux.get();
	    aux.suppr();
	    break;
	  }
	}
	delete pageGame;
      }
    }
    grille->setTrembleur(*pageGame);
  }

  aux.place();
  while(aux.walk()) {
    cpt=aux.get()->cpt;
    int cptChange=0;
    const String *idTo;
    while((idTo=iniRef->getWithoutException("[Page]",cpt,
					 "change to",cptChange))) {
      if(*(aux.get()->couple->getId())==*idTo)
	throw Exception(ERROR_TOPAGE_MUST_BE_DIFFERENT_FROM_ITSELF,*idTo);

      int keyTo=iniRef->getWithDefault("[Page]",cpt,"when key",cptChange,0);
      Page *pageTo=0;

      bool found=false;

      if(!(*idTo=="exit")) {
	listCouple.place();
	while(listCouple.walk()) {
	  if(*(listCouple.get()->getId())==*idTo) {
	    pageTo=listCouple.get()->getPage();
	    found=true;
	  }
	}
	//	if(!found) throw Exception(ERROR_TOPAGE_NOT_FOUND,*idTo);
      }
      else found=true;

      if(found) aux.get()->couple->add(keyTo,pageTo);

      cptChange++;
    }
    delete aux.get();
  }

  if(clean) {
    //I load it to mark 'used' in ini file
    char aux[2]={0,0};
    for(int k=0;k<2;k++) {
      for(int c=0;c<=255;c++) {
	aux[0]=(char)c;
	iniRef->get("[Character]",0,aux,k);
      }
    }
  }

  cout<<"ok\n";cout.flush();
}


void PageChanger::getChangingValues() throw()
{
  int cpt=0;
  nbChanging=0;
  while(cpt==0 || iniRef->isPresentSection("[Changing Page]",cpt)) {
    listChanging.
      put(new
	  PageChangerDescChanging(
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "new pos j from",0,0),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "new pos i from",0,0),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "old pos j to",0,0),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "old pos i to",0,0),
				  
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "new vit j from",0,0),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "new vit i from",0,0),
				  
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "old vit j to",0,0),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "old vit i to",0,0),

				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "amort j",0,
							 DEFAULT_AMORT),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "amort i",0,
							 DEFAULT_AMORT),
				  
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "clear back",0,1),
				  iniRef->getWithDefault("[Changing Page]",cpt,
							 "ground",0,1)));
    cpt++;
    nbChanging++;
  }
}


PageChanger::PageChanger(Walkable& allTheGame,Ini& _iniRef,
			 GraphicMode& graphicModeRef,FlashPalette& palette,
			 Sounder& sounder,Keyer& keyer,
			 Blower& redirector,
			 TextPainter& textPainter,
			 StripContainer& stripContainer,
			 bool clean) throw(Exception)
  :Walkable(0),
   newPage(0),oldPage(0),changing(false),currentChanging(0),
   finished(false),graphicModeRef(&graphicModeRef),paletteRef(&palette),
   iniRef(&_iniRef),sounderRef(&sounder),
   bonhommeContainer(new BonhommeContainer(_iniRef,sounder,
					   textPainter,
					   stripContainer)),
   grille(new Grille(_iniRef,*bonhommeContainer,palette,sounder,redirector,
		     stripContainer)),
   soundBlowUpMotId(sounder.add(*_iniRef.getList("[Sound]",0,
						 "sound blow up mot",
						 "[Sound]"))),
   redirectorRef(&redirector),
   stripContainerRef(&stripContainer)
{
  constructCouple(allTheGame,keyer,clean);
  listCouple.place();
  if(!listCouple.walk()) throw Exception(ERROR_NO_VALID_PAGE);
  newPage=listCouple.get()->getPage();
  currentCouple=listCouple.get();

  newPage->moveOnScreen(0,0);

  newVit.j=newVit.i=0;
  getChangingValues();
}

PageChanger::~PageChanger() throw()
{
  listCouple.place();
  while(listCouple.walk()) {
    delete listCouple.get()->getPage();
    delete listCouple.get();
  }
  listChanging.place();
  while(listChanging.walk()) delete listChanging.get();
  delete grille;
  delete bonhommeContainer;
}

void PageChanger::keyAction(const Key& key) throw(Exception)
{
  if(changing && oldPage) oldPage->keyAction(key);
  newPage->keyAction(key);
  if(key.isDown()) changeOn(key.getCode());
}

void PageChanger::paint() throw()
{
  if(changing) {
    if(currentChanging->clearBack) graphicModeRef->clear();
    oldPage->paint();
    oldPage->refresh();
  }
  newPage->paint();
  newPage->refresh();
}

void PageChanger::changeOn(int key) throw()
{
  if(changing) return;

  Page *aux;
  bool found=currentCouple->getCorresp(key,aux);
  if(found) {
    if(!aux) {
      finished=true;
      return;
    }
    changing=true;
    listChanging.place();
    listChanging.walk();
    for(int cpt=rand()%nbChanging;cpt>0;cpt--) listChanging.walk();
    currentChanging=listChanging.get();
    
    aux->moveOnScreen(currentChanging->from.j,currentChanging->from.i);
    oldPage=newPage;
    newPage=aux;
    
    oldPage->goOut();
    newPage->reinit();
    
    if(paletteRef->isGris()) paletteRef->flashToColor();
    
    listCouple.place();
    while(listCouple.walk())
      if(listCouple.get()->getPage()==newPage) {
	currentCouple=listCouple.get();
	break;
      }

    oldVit.j=currentChanging->vit.to.j;
    oldVit.i=currentChanging->vit.to.i;
    newVit.j=currentChanging->vit.from.j;
    newVit.i=currentChanging->vit.from.i;
  }
}

int PageChanger::virtualWalk() throw()
{
  paletteRef->walk();

  redirectorRef->walk();

  if(changing) oldPage->walk();
  int walked=newPage->walk();

  if(changing) {
    elastiqueEtSol(*oldPage,
		   currentChanging->to.j,currentChanging->to.i,
		   oldVit.j,oldVit.i,false);

    changing=elastiqueEtSol(*newPage,0,0,newVit.j,newVit.i,true);
  }

  if(!walked) changeOn(KEY_AUTOCHANGE);
  if(finished) return 0;

  return 1;
}

bool PageChanger::elastiqueEtSol(Page& page,int jTo,int iTo,
				 int& jVit,int& iVit,bool haveToSetTotalyGone) throw()
{
  int ground;
  struct { int j,i; } amort;
  if(currentChanging) {
    amort.j=currentChanging->amort.j;
    amort.i=currentChanging->amort.i;
    ground=currentChanging->ground;
  }
  else {
    amort.j=amort.i=0;
    ground=1;
  }

  int j,i;
  page.getPosOnScreen(j,i);

  if(i<iTo) iVit++;
  if(i>iTo) iVit--;
  if((i+iVit<iTo && i>=iTo) || (i+iVit>iTo && i<=iTo)) {
    if(iVit>0) {
      if(iVit<amort.i) iVit=0;
      else iVit-=amort.i;
    } else {
      if(-iVit<amort.i) iVit=0;
      else iVit+=amort.i;
    }
    if(iVit==0) i=iTo;
  }

  if(j<jTo) jVit++;
  if(j>jTo) jVit--;

  if(ground && j+jVit>0) {
    jVit=-jVit;
    if(jVit>0) {
      if(jVit<amort.j) jVit=0;
      else jVit-=amort.j;
    } else {
      if(-jVit<amort.j) jVit=0;
      else jVit+=amort.j;
    }
    if(jVit==0) j=jTo;
  } else 
    if((j+jVit<jTo && j>=jTo) || (j+jVit>jTo && j<=jTo)) {
      if(jVit>0) {
	if(jVit<amort.j) jVit=0;
	else jVit-=amort.j;
      } else {
	if(-jVit<amort.j) jVit=0;
	else jVit+=amort.j;
      }
      if(jVit==0) j=jTo;
    }

  j+=jVit;
  i+=iVit;

  page.moveOnScreen(j,i);
  if(j==jTo && i==iTo && jVit==0 && iVit==0) {
    if(haveToSetTotalyGone) page.setTotalyGone();
    return false;
  }
  return true;
}

