/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "PageConfigKey.h"

#define DEFAULT_MOT_UP "up"
#define DEFAULT_MOT_DOWN "down"
#define DEFAULT_MOT_LEFT "left"
#define DEFAULT_MOT_RIGHT "right"
#define DEFAULT_MOT_FIRE0 "fire"
#define DEFAULT_MOT_FIRE1 "special fire"

#define DEFAULT_POS_MOT_J 0
#define DEFAULT_POS_MOT_I 0

#define DEFAULT_KEY_CANCEL 1

#define BLACK toPixel(minGreyValue,minGreyValue,minGreyValue,0)


PageConfigKey::PageConfigKey(Ini& ini,int numeroSection,
			     GraphicMode& graphicMode,
			     BonhommeContainer& bonhommeContainer,
			     Sounder& sounder,int soundBlowMotId,
			     StripContainer& stripContainer)
  throw(Exception)
  :Page(ini,numeroSection,graphicMode,sounder,soundBlowMotId,stripContainer),
   bonhommeContainerRef(&bonhommeContainer),
   current(0),finished(false),
   keyCancel(ini.getWithDefault("[Page]",numeroSection,"key cancel",0,DEFAULT_KEY_CANCEL))
{
  int jPosMot=ini.getWithDefault("[Page]",numeroSection,"pos mot config j",0,
				 DEFAULT_POS_MOT_J);
  int iPosMot=ini.getWithDefault("[Page]",numeroSection,"pos mot config i",0,
				 DEFAULT_POS_MOT_J);

  idMot[0]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,"mot up",0,
				       DEFAULT_MOT_UP),
		   jPosMot,iPosMot,BLACK,BLACK);
  idMot[1]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,
				       "mot down",0,
				       DEFAULT_MOT_DOWN),
		   jPosMot,iPosMot,BLACK,BLACK);
  idMot[2]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,
				       "mot left",0,
				       DEFAULT_MOT_LEFT),
		   jPosMot,iPosMot,BLACK,BLACK);
  idMot[3]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,
				       "mot right",0,
				       DEFAULT_MOT_RIGHT),
		   jPosMot,iPosMot,BLACK,BLACK);
  idMot[4]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,
				       "mot fire 0",0,
				       DEFAULT_MOT_FIRE0),
		   jPosMot,iPosMot,BLACK,BLACK);
  idMot[5]=new Mot(ini,
		   *ini.getWithDefault("[Page]",numeroSection,
				       "mot fire 1",0,
				       DEFAULT_MOT_FIRE1),
		   jPosMot,iPosMot,BLACK,BLACK);

}

PageConfigKey::~PageConfigKey() throw()
{
}

void PageConfigKey::reinit() throw()
{
  finished=false;
  listPlaying=*(bonhommeContainerRef->getPlayerPlaying());
  listPlaying.place();
  listPlaying.walk();
  current=0;
  Pixel c=listPlaying.get()->getBonhomme()->getColor();
  for(int i=0;i<6;i++) idMot[i]->setColor(c);
  Page::reinit();
}

void PageConfigKey::keyAction(const Key& key) throw()
{
  Page::keyAction(key);
  if(finished) return;
  if(key.getCode()==keyCancel) {
    finished=true;
    return;
  }
  if(key.isDown()) {
    listPlaying.get()->setKey(current,key.getCode());
    current++;
    if(current==6) {
      current=0;
      if(!listPlaying.walk()) finished=true;
    }
    if(!finished) {
      Pixel c=listPlaying.get()->getBonhomme()->getColor();
      for(int i=0;i<6;i++) idMot[i]->setColor(c);
    }
  }
}


int PageConfigKey::virtualWalk() throw()
{
  if(current>=0) idMot[current]->walk();
  Page::virtualWalk();
  if(finished) return 0;
  else return 1;
}

void PageConfigKey::paint() throw()
{
  Page::paint();
  if(!finished) idMot[current]->paint(*(background->getWindowPipe()));
}



