/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "PageableContainer.h"
#include "error.h"

PageableContainer::PageableContainer(int slow) throw()
  :Pageable(slow)
{
}

PageableContainer::~PageableContainer() throw ()
{
  list.place();
  while(list.walk()) delete list.get();
}

void PageableContainer::add(Pageable& pageable) throw ()
{
  list.placeCurrentOnHead(); 
  list.put(&pageable); 
}

void PageableContainer::addWithoutDelete(Pageable& pageable) throw ()
{
  listWithoutDelete.put(&pageable); 
}

void PageableContainer::suppr(Pageable *pageable) throw (Exception)
{
  list.place();
  while(list.walk())
    if(list.get()==pageable) {
      delete pageable;
      list.suppr();
      return;
    }
  listWithoutDelete.place();
  while(listWithoutDelete.walk())
    if(listWithoutDelete.get()==pageable) {
      listWithoutDelete.suppr();
      return;
    }
  throw Exception(ERROR_PAGEABLECONTAINER);
}

void PageableContainer::supprAll() throw ()
{
  list.place();
  while(list.walk()) {
    delete list.get();
    list.suppr();
  }
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) listWithoutDelete.suppr();
}

int PageableContainer::virtualWalk() throw()
{
  int ret=0;
  list.place();
  while(list.walk()) {
    if(list.get()->isActive())
      ret+=list.get()->walk();
  }
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) {
    if(listWithoutDelete.get()->isActive())
      ret+=listWithoutDelete.get()->walk();
  }
  if(ret!=0) return 1;
  return 0;
}

void PageableContainer::paint(WindowPipe& windowPipe) throw()
{
  list.place();
  while(list.walk()) {
    if(list.get()->isActive())
      list.get()->paint(windowPipe);
  }
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) {
    if(listWithoutDelete.get()->isActive())
      listWithoutDelete.get()->paint(windowPipe);
  }
}

void PageableContainer::reinit() throw()
{
  list.place();
  while(list.walk()) list.get()->reinit();
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) listWithoutDelete.get()->reinit();
}

void PageableContainer::keyAction(const Key& key) throw()
{
  list.place();
  while(list.walk()) {
    if(list.get()->isActive())
      list.get()->keyAction(key);
  }
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) {
    if(listWithoutDelete.get()->isActive())
      listWithoutDelete.get()->keyAction(key);
  }
}
