/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */

#include "PixelMatrix.h"

PixelMatrix::
PixelMatrix(int nbLine_j,int nbColumn_i) throw ()
  :a(nbLine_j)
{
  p=new Pixel[nbLine_j*nbColumn_i];
  Pixel *ref=p;
  for(int k=0;k<nbLine();k++) {
    a[k]=new PixelArray(ref,nbColumn_i);
    ref+=nbColumn_i;
  }
}

PixelMatrix::
~PixelMatrix() throw ()
{
  for(int k=0;k<nbLine();k++) delete a[k];
  delete p;
}

PixelArray& 
PixelMatrix::
operator[](int k) throw ()
{
  return *a[k];
}

const PixelArray& 
PixelMatrix::
operator[](int k) const throw ()
{
  return *a[k];
}


int 
PixelMatrix::
nbLine() const throw()
{
  return a.size();
}

int 
PixelMatrix::
nbColumn() const throw()
{
  return a[0]->size();
}


void 
PixelMatrix::
putSprite(const PixelMatrix& data,
	  const PixelMatrix& mask,
	  int jPosFrom,int iPosFrom,
	  int jPosTo,int iPosTo,
	  int jSizeFrom,int iSizeFrom) throw ()
{
  for(int j=jPosFrom,k=jPosTo;j<jPosFrom+jSizeFrom;j++,k++)
    a[k]->putSprite(data[j],mask[j],iPosFrom,iPosTo,iSizeFrom);
}

void 
PixelMatrix::
putSpriteColor(Pixel color,
	       const PixelMatrix& data,
	       int jPosFrom,int iPosFrom,
	       int jPosTo,int iPosTo,
	       int jSizeFrom,int iSizeFrom) throw ()
{
  for(int j=jPosFrom,k=jPosTo;j<jPosFrom+jSizeFrom;j++,k++)
    a[k]->putSpriteColor(color,data[j],iPosFrom,iPosTo,iSizeFrom);
}

void 
PixelMatrix::
putSpriteUnderMask(const PixelMatrix& data,
		   const PixelMatrix& mask,
		   int jPosFrom,int iPosFrom,
		   int jPosTo,int iPosTo,
		   int jSizeFrom,int iSizeFrom,
		   const UnderMaskMatrix& underMask,
		   int jPosUnderMask,int iPosUnderMask) throw ()
{
  for(int j=jPosFrom,k=jPosTo,u=jPosUnderMask;j<jPosFrom+jSizeFrom;j++,k++,u++)
    a[k]->putSpriteUnderMask(data[j],mask[j],iPosFrom,iPosTo,iSizeFrom,
			     underMask[u],iPosUnderMask);
}

void 
PixelMatrix::
putSpriteUnderMaskPalette(const PixelMatrix& data,
			  const PixelMatrix& mask,
			  int jPosFrom,int iPosFrom,
			  int jPosTo,int iPosTo,
			  int jSizeFrom,int iSizeFrom,
			  const UnderMaskMatrix& underMask,
			  int jPosUnderMask,int iPosUnderMask,
			  const PixelPalette& pal) throw ()
{
  for(int j=jPosFrom,k=jPosTo,u=jPosUnderMask;j<jPosFrom+jSizeFrom;j++,k++,u++)
    a[k]->putSpriteUnderMaskPalette(data[j],mask[j],iPosFrom,iPosTo,iSizeFrom,
				    underMask[u],iPosUnderMask,pal);
}


void 
PixelMatrix::
copy(const PixelMatrix& data,
     int jPosFrom,int iPosFrom,
     int jPosTo,int iPosTo,
     int jSizeFrom,int iSizeFrom) throw ()
{
  for(int j=jPosFrom,k=jPosTo;j<jPosFrom+jSizeFrom;j++,k++)
    a[k]->copy(data[j],iPosFrom,iPosTo,iSizeFrom);
}


PixelMatrix&
PixelMatrix::
operator=(Pixel pixel) throw()
{
  for(int k=0;k<nbLine();k++) (*a[k])=pixel;
  return *this;
}

