/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Player.h"

#define DEFAULT_POS_SCORE_J 0
#define DEFAULT_POS_SCORE_I 0

#define DEFAULT_COLOR toPixel(maxRedValue,maxGreenValue,maxBlueValue,0)

#define DEFAULT_DEC_COLOR_TEAM 5

#define BLACK toPixel(minGreyValue,minGreyValue,minGreyValue,0)


Player::Player(Ini& ini,int numeroSection) throw (Exception)
  :score(0),bonhomme(0),idBonhomme(0),
   iniRef(&ini),numeroSectionRef(numeroSection),
   decColorTeam(ini.getWithDefault("[Global]",0,
				   "dec color team",0,
				   DEFAULT_DEC_COLOR_TEAM))
{
  posScore.j=ini.getWithDefault("[Player]",numeroSection,
				"pos Score j",0,DEFAULT_POS_SCORE_J);
  posScore.i=ini.getWithDefault("[Player]",numeroSection,
				"pos Score i",0,DEFAULT_POS_SCORE_I);
  key[0]=ini.getWithDefault("[Player]",numeroSection,"key up",0,0);
  key[1]=ini.getWithDefault("[Player]",numeroSection,"key down",0,0);
  key[2]=ini.getWithDefault("[Player]",numeroSection,"key left",0,0);
  key[3]=ini.getWithDefault("[Player]",numeroSection,"key right",0,0);
  key[4]=ini.getWithDefault("[Player]",numeroSection,"key fire 0",0,0);
  key[5]=ini.getWithDefault("[Player]",numeroSection,"key fire 1",0,0);

  color=ini.getWithDefaultColor("[Player]",numeroSection,
				     "color",0,
				     DEFAULT_COLOR);

  char testMotChar[2]={0,0};
  for(int k=0;k<10;k++) {
    testMotChar[0]=k+'0';
    Mot *testMot=new Mot(ini,*new String(testMotChar),0,0,BLACK,BLACK);
    delete testMot;
    //ceci permet d'etre certain que la creation des scores n'echouera pas
  }
}

Player::~Player() throw()
{
  iniRef->set("[Player]",numeroSectionRef,
	      "key up",0,key[0]);
  iniRef->set("[Player]",numeroSectionRef,
	      "key down",0,key[1]);
  iniRef->set("[Player]",numeroSectionRef,
	      "key left",0,key[2]);
  iniRef->set("[Player]",numeroSectionRef,
	      "key right",0,key[3]);
  iniRef->set("[Player]",numeroSectionRef,
	      "key fire 0",0,key[4]);
  iniRef->set("[Player]",numeroSectionRef,
	      "key fire 1",0,key[5]);

  if(idBonhomme) iniRef->set("[Player]",numeroSectionRef,
			     "Bonhomme",0,*idBonhomme);
}


void Player::keyAction(const Key& k) throw()
{
  if(k.isDown()) {
    if(k.getCode()==key[0]) bonhomme->getBonhomme20()->moveUp();
    if(k.getCode()==key[1]) bonhomme->getBonhomme20()->moveDown();
    if(k.getCode()==key[2]) bonhomme->getBonhomme20()->moveLeft();
    if(k.getCode()==key[3]) bonhomme->getBonhomme20()->moveRight();
    if(k.getCode()==key[4]) bonhomme->getBonhomme20()->fire0();
    if(k.getCode()==key[5]) bonhomme->getBonhomme20()->fire1();
  }
  if(k.isUp()) {
    if(k.getCode()==key[0] || k.getCode()==key[1] ||
       k.getCode()==key[2] || k.getCode()==key[3])
      bonhomme->getBonhomme20()->doNotMove();
    if(k.getCode()==key[4]) bonhomme->getBonhomme20()->doNotFire0();
    if(k.getCode()==key[5]) bonhomme->getBonhomme20()->doNotFire1();
  }
}

void Player::setBonhomme(Bonhomme& b) throw()
{
  //%%  if(bonhomme) bonhomme->getBonhomme60()->unactivateMore();

  //  int auxScore=0;
  //  if(score) {
  //    auxScore=score->get();
  //    delete score;
  //  }

  bonhomme=&b;
  //cree un score
  Pixel c=bonhomme->getColor();

  if(!score) score=new Score(*iniRef,0,posScore.j,posScore.i,c);
  score->setColor(c);

  idBonhomme=bonhomme->getId();

  //%%  bonhomme->getBonhomme60()->activateMore();
}
  
Bonhomme *Player::getBonhomme() throw()
{
  return bonhomme;
}

Score *Player::getScore() throw()
{
  return score;
}

void Player::resetScore() throw()
{
  if(!bonhomme) return;
  delete score;
  Pixel c=bonhomme->getColor();
  score=new Score(*iniRef,0,posScore.j,posScore.i,c);
}

void Player::incScore() throw()
{
  int auxScore=0;
  if(score) {
    auxScore=score->get();
    delete score;
  }
  //cree un score
  Pixel c=bonhomme->getColor();
  score=new Score(*iniRef,auxScore+1,posScore.j,posScore.i,c);
}

void Player::setKey(int k,int code) throw()
{
  key[k]=code;
}

void Player::setColorToBonhomme() throw()
{
  if(!bonhomme) return;

  List<Bonhomme*> *l=bonhomme->getPlayingTeam();
  
  l->place();
  l->walk();
  if(l->get()==bonhomme)
    bonhomme->setColor(color);
  else {//pas le premier de l'equipe, donc : prend la couleur (dec) du premier
    Pixel col=l->get()->getColor();
    int r=getRedFromPixel(col);
    int g=getGreenFromPixel(col);
    int b=getBlueFromPixel(col);
    while(l->walk()) {
      r-=decColorTeam;
      g-=decColorTeam;
      b-=decColorTeam;
      if(r<minRedValue) r=minRedValue;
      if(g<minGreenValue) g=minGreenValue;
      if(b<minBlueValue) b=minBlueValue;
      if(l->get()==bonhomme) {
	bonhomme->setColor(toPixel(r,g,b,0));
	break;
      }
    }
  }

  //  int auxScore=0;
  //  if(score) {
  //    auxScore=score->get();
  //    delete score;
  //  }

  //cree un score
  Pixel c=bonhomme->getColor();
  if(!score) score=new Score(*iniRef,0,posScore.j,posScore.i,c);
  else score->setColor(c);
}

int Player::getKeyLeft() const throw()
{
  return key[2];
}
int Player::getKeyRight() const throw()
{
  return key[3];
}
int Player::getKeyUp() const throw()
{
  return key[0];
}
int Player::getKeyDown() const throw()
{
  return key[1];
}
int Player::getKeyFire0() const throw()
{
  return key[4];
}
int Player::getKeyFire1() const throw()
{
  return key[5];
}
