/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class SectionFileModifier;

#ifndef __SectionFileModifier_h__
#define __SectionFileModifier_h__

#include "List.h"
#include "String.h"

/**
   Used by AddOnScriptAnalyser, to modify .ini files without suppress possibility.
   @see AddOnScriptAnalyser
   @author David FAUTHOUX
*/
class SectionFileModifier {
 protected:
  ///list of the lines added when a line is added
  List< const String * > *mark;
  ///
  const String *commentMark;

  ///list of the lines contained by the file
  List< const String * > list;

  ///name of the file, used for save()
  const String *name;

  ///line added at end ("# end:"+...)
  String *endMark;

 public:
  ///
  SectionFileModifier(const String& nameFile,
		   List<const String *>& markList,
		   const String& commentSymbols) throw(Exception);
  ///
  virtual ~SectionFileModifier() throw();
  ///
  virtual void save() throw(Exception);
  ///
  virtual void save(const String& nameFile) throw(Exception);

  ///
  virtual void place() throw();
  ///
  virtual bool walk() throw();
  ///
  virtual bool walkInvers() throw();
  ///
  virtual const String *current() const throw(Exception);

  ///go begin to end (to find s and place after)
  virtual bool after(List< const String * >& afterLines) throw();
  ///go begin to end (to find s and place after)
  virtual bool after(const String& afterLines) throw();
  ///go end to begin (to find s and place before)
  virtual bool before(List< const String * >& beforeLines) throw();
  ///go end to begin (to find s and place before)
  virtual bool before(const String& beforeLines) throw();

  ///
  virtual void insert(List<const String*>& lines) throw();
  ///
  virtual void insertRaw(const String& line) throw();

  ///suppress all mark and body inserted with mark
  virtual void suppress() throw();
  ///raw suppress
  virtual void suppressLine() throw();

};

#endif
