/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



class SounderAssoc;

#ifndef __SounderAssoc_h__
#define __SounderAssoc_h__

#include "List.h"

/**
   Used to associate a logical sound id (pointing to a list of physical sound ids) with a channel (if reserved) and with the time of the sounds.
   @see Sounder
   @author David FAUTHOUX
*/
class SounderAssoc {
 public:
  ///This is the id to identify a list of sound files (list of physical ids)
  int logicalId;
  ///This list contains the id returns by the SoundDisplay (the physical ids)
  List< int > physicalIds;
  ///Length of physicalIds
  int nbPhysicalIds;

  ///The durations of the sound files
  List< int > timeFile;
  ///If the sound cannot be removed by other sounds when there is not enough channel, the boolean == true, else, if it can be removed, this boolean ==false
  bool channelReserving;
  ///When channelReserving==true, this value is used : this is the number of the reserved channel
  int reservedChannel;

  ///Give here the logical id to identify the list of sound files
  SounderAssoc(int logicalId) throw();
  ///Does nothing
  ~SounderAssoc() throw();

  ///Adds "physicalId" to the list physicalIds and adds "time" to the list timeFile (and inc nbPhysicalIds)
  void add(int physicalId,int time) throw();

  ///Set channelReserving to true and reservedChannel to channel
  void reserveChannel(int channel) throw();
};



#endif
