/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <stdlib.h>
#include "StripContainer.h"

#define DEFAULT_FONT_COLOR toPixel(maxGreyValue,maxGreyValue,maxGreyValue,0)
#define DEFAULT_BACKGROUND_COLOR toPixel(0,0,0,0)
#define DEFAULT_MOVE_STEP 1

StripContainer::StripContainer(TextPainter& textPainter,Ini& ini,GraphicMode& graphicMode) throw(Exception)
  :PageableContainer(ini.getWithDefault("[Strip]",0,
					"global slow",0,0)),
   textPainterRef(&textPainter),
   underMask(graphicMode.nbLine(),graphicMode.nbColumn()),
   window(new Window(graphicMode)),
   windowPipe(new WindowPipe(*window,*window,underMask)),
   size(graphicMode.nbLine(),graphicMode.nbColumn()),
   randomSlow(ini.getWithDefault("[Strip]",0,
				 "random slow",0,0)),
   /*%%
     font(new Font(ini)),
     defaultFontColor(ini.getWithDefaultColor("[Strip]",0,
     "default font color",0,
     DEFAULT_FONT_COLOR)),
     defaultBackgroundColor(ini.getWithDefaultColor("[Strip]",0,
     "default background color",0,
     DEFAULT_BACKGROUND_COLOR)),
   */
   moveStep(ini.getWithDefault("[Strip]",0,
			       "move step",0,
			       DEFAULT_MOVE_STEP)),
   preStrip(ini)
{
}

StripContainer::~StripContainer() throw()
{
  //%%  delete font;
}

void StripContainer::add(const String& mot,
			 Pixel fontColor,Pixel backgroundColor) throw()
{
  PageableContainer::add(*new Strip(*textPainterRef,
				    //%%				    *font,
				    mot,
				    fontColor,backgroundColor,
				    rand()%(randomSlow+1),
				    size.j,size.i,
				    moveStep));
}

void StripContainer::add(const String& mot,Pixel fontColor) throw()
{
  add(mot,fontColor,textPainterRef->getDefaultBackgroundColor());
}

void StripContainer::add(const String& mot) throw()
{
  add(mot,textPainterRef->getDefaultFontColor(),textPainterRef->getDefaultBackgroundColor());
}

void StripContainer::add(PreStrip::Id mot,
			 Pixel fontColor,Pixel backgroundColor) throw()
{
  add(*preStrip.get(mot),fontColor,backgroundColor);
}

void StripContainer::add(PreStrip::Id mot,
			 Pixel fontColor) throw()
{
  add(*preStrip.get(mot),fontColor,textPainterRef->getDefaultBackgroundColor());
}

void StripContainer::add(PreStrip::Id mot) throw()
{
  add(*preStrip.get(mot),textPainterRef->getDefaultFontColor(),textPainterRef->getDefaultBackgroundColor());
}

int StripContainer::virtualWalk() throw()
{
  //copier coller de PageableContainer avec delete si return=0
  int ret=0;
  int localRet;
  list.place();
  while(list.walk()) {
    if(list.get()->isActive()) {
      localRet=list.get()->walk();
      if(!localRet) {
	delete list.get();
	list.suppr();
      }
      ret+=localRet;
    }
  }
  listWithoutDelete.place();
  while(listWithoutDelete.walk()) {
    if(listWithoutDelete.get()->isActive())
      ret+=listWithoutDelete.get()->walk();
  }
  if(ret!=0) return 1;
  return 0;
}


void StripContainer::paint() throw()
{
  PageableContainer::paint(*windowPipe);
  windowPipe->refreshAll();
}

void StripContainer::activate() throw()
{
  PageableContainer::activate();
  supprAll();
}
