/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include <limits.h>
#include "TextFileModifier.h"
#include "File.h"
#include "util.h"

TextFileModifier::TextFileModifier(const String& nameFile,
				   List<const String *>& markList,
				   bool createFile) throw(Exception)
  :mark(&markList),
   name(&nameFile),
   /*%%
     slashFound(false),
     starFound(false),
     inString(false),
     blanking(false),
     nbSeparator(1),
     comment(0),
   */
   cptWalk(0),
   minLimit(INT_MIN),
   maxLimit(INT_MAX),
   tmpLimit(-1)
{
  if(!createFile) {
    File f(nameFile);
    while(!f.eof()) {
      char c;
      f.read(c);
      SensChar *s=new SensChar;
      s->c=c;
      s->added=false;
      list.put(s);
      //%%    putChar(c);
    }
  }
  else {
    if(File::exists(nameFile)) {
      cerr<<"File already exists:"<<(const char*)nameFile<<"\n";
      String *s=File::renameCheck(nameFile,*new String(".bak"));
      cerr<<"Renaming this file in "<<(const char*)(*s)<<"\n";
    }
  }

  reparse();

  list.place();
}

/*%%
void TextFileModifier::putChar(char c) throw()
{
  SensChar *s=new SensChar;
  s->c=c;
  s->added=false;
  switch(comment) {
  case 0:
    s->s=REAL;
    
    if(c=='\"') {
      if(inString) inString=false;
      else inString=true;
    }
    
    if(!inString) {
      switch(c) {
      case '*':
	if(slashFound) {
	  list.get()->s=IGNORE;
	  s->s=IGNORE;
	  slashFound=false;
	  comment=2;
	  nbSeparator--;//car '/' est un separateur
	}
	break;
      case '/':
	if(slashFound) {
	  list.get()->s=IGNORE;
	  s->s=IGNORE;
	  slashFound=false;
	  comment=1;
	  nbSeparator--;//car '/' est un separateur
	} else slashFound=true;
	break;
      default:
	slashFound=false;
	break;
      }
    }
    
    if(comment==0) {
      if(isSeparator(c)) nbSeparator++;
      
      if(blanking)
	if(isBlank(c)) s->s=IGNORE;
	else {
	  if(nbSeparator==0) {
	    blanking=false;
	    SensChar *s=new SensChar;
	    s->c=BLANK_SPACE;
	    s->s=REAL;
	    s->added=true;
	    list.put(s);
	  } else if(!isSeparator(c)) {
	    blanking=false;
	  }
	}
      else if(isBlank(c)) {
	s->s=IGNORE;
	blanking=true;
      }
      else if(!isSeparator(c)) nbSeparator=0;
    }
    break;
  case 1:
    s->s=IGNORE;
    if(c=='\n') comment=0;
    break;
  case 2:
    s->s=IGNORE;
    if(c=='/' && starFound) comment=0;
    if(c=='*') starFound=true;
    else starFound=false;
    break;
  }
  list.put(s);
}
*/

void TextFileModifier::reparse() throw()
{
  listPosition.place();
  while(listPosition.walk()) listPosition.suppr();


  bool slashFound=false;
  bool starFound=false;
  bool inString=false;
  bool blanking=false;
  int nbSeparator=1;
  int comment=0;

  List<SensChar*> tmp;

  list.place();
  while(list.walk()) {
    if(!list.get()->added) tmp.put(list.get());
    else {
      delete list.get();
    }
    list.suppr();
  }

  

  tmp.place();

  while(tmp.walk()) {
  
    SensChar *s=tmp.get();
    char c=s->c;
    //  s->c=c;
    //  s->added=false;
    switch(comment) {
    case 0:
      s->s=REAL;
      
      if(c=='\"') {
	if(inString) inString=false;
	else inString=true;
      }
      
      if(!inString) {
	switch(c) {
	case '*':
	  if(slashFound) {
	    list.get()->s=IGNORE;
	    s->s=IGNORE;
	    slashFound=false;
	    comment=2;
	    nbSeparator--;//car '/' est un separateur
	  }
	  break;
	case '/':
	  if(slashFound) {
	    list.get()->s=IGNORE;
	    s->s=IGNORE;
	    slashFound=false;
	    comment=1;
	    nbSeparator--;//car '/' est un separateur
	  } else slashFound=true;
	  break;
	default:
	  slashFound=false;
	  break;
	}
      }
      
      if(comment==0) {
	if(isSeparator(c)) nbSeparator++;
	
	if(blanking)
	  if(isBlank(c)) s->s=IGNORE;
	  else {
	    if(nbSeparator==0) {
	      blanking=false;
	      SensChar *s=new SensChar;
	      s->c=BLANK_SPACE;
	      s->s=REAL;
	      s->added=true;
	      list.put(s);
	    } else if(!isSeparator(c)) {
	      blanking=false;
	    }
	  }
	else if(isBlank(c)) {
	  s->s=IGNORE;
	  blanking=true;
	}
	else if(!isSeparator(c)) nbSeparator=0;
      }
      break;
    case 1:
      s->s=IGNORE;
      if(c=='\n') comment=0;
      break;
    case 2:
      s->s=IGNORE;
      if(c=='/' && starFound) comment=0;
      if(c=='*') starFound=true;
      else starFound=false;
      break;
    }
    list.put(s);
    
  }


  /*%%

  TextMode textMode;

  list.place();
  while(list.walk()) {
    switch(list.get()->s) {
    case IGNORE:
      textMode.setForegroundColor(TextMode::RED);
      break;
    case REAL:
      if(list.get()->added) textMode.setForegroundColor(TextMode::BLUE);
      else textMode.setForegroundColor(TextMode::BLACK);
      break;
    default:
      textMode.setForegroundColor(TextMode::CYAN);
      break;
    }
    switch(list.get()->c) {
    case '\n':
      cout<<"\\\n";
      break;
    case ' ':
      cout<<'$';
      break;
    default:
      cout<<list.get()->c;
      break;
    }
    cout.flush();
  }

  textMode.setForegroundColor(TextMode::BLACK);
  getchar();

  list.place();
  while(list.walk()) {
    if(list.get()->s==REAL) cout<<list.get()->c;
    cout.flush();
  }

  */

}
TextFileModifier::~TextFileModifier() throw()
{
  list.place();
  while(list.walk()) delete list.get();
}

void TextFileModifier::save() throw(Exception)
{
  save(*name);
}

void TextFileModifier::save(const String& nameFile) throw(Exception)
{
  File f(nameFile,File::WRITE_ONLY);
  list.place();
  while(list.walk()) if(!list.get()->added) f.write(list.get()->c);

}

void TextFileModifier::place() throw()
{
  list.place();
  cptWalk=0;
  unlimit();
}

bool TextFileModifier::walk() throw()
{
  if(cptWalk<maxLimit) {
    if(cptWalk==tmpLimit) {
      minLimit=tmpLimit;
      tmpLimit=-1;
    }
    if(list.walk()) {
      cptWalk++;
      return true;
    }
  }
  return false;
}

bool TextFileModifier::walkIgnoring() throw()
{
  while(walk()) {
    if(list.get()->s==REAL) return true;
  }
  return false;
}

bool TextFileModifier::walkInvers() throw()
{
  if(cptWalk<=1) return false;
  if(cptWalk>minLimit) {
    if(cptWalk==tmpLimit) {
      maxLimit=tmpLimit;
      tmpLimit=-1;
    }
    list.walkHeadToQueue();
    cptWalk--;
    return true;
  }
  return false;
}

bool TextFileModifier::walkInversIgnoring() throw()
{
  while(walkInvers()) {
    if(list.get()->s==REAL) return true;
  }
  return false;
}

char TextFileModifier::current() const throw(Exception)
{
  return list.get()->c;
}

void TextFileModifier::limit() throw()
{
  tmpLimit=cptWalk;
}

void TextFileModifier::unlimit() throw()
{
  tmpLimit=-1;
  minLimit=INT_MIN;
  maxLimit=INT_MAX;
}

bool TextFileModifier::after(const String& s) throw()
{
  String *r=simplifyBlank(s);

  if(r->size()==0) return true;

  int pos=0;
  while(walkIgnoring()) {
    if(current()==(*r)[pos]) {
      pos++;
      if(pos==r->size()) {
	delete r;
	return true;
      }
    }
    else {
      pos=0;
      if(current()==(*r)[pos]) {
	pos++;
	if(pos==r->size()) {
	  delete r;
	  return true;
	}
      }
    }
  }
  delete r;
  return false;
}

bool TextFileModifier::before(const String& s) throw()
{
  String *r=simplifyBlank(s);
  
  if(r->size()==0) return true;

  int pos=r->size()-1;
  do {
    if(current()==(*r)[pos]) {
      if(pos==0) {
	walkInversIgnoring();
	delete r;
	return true;
      }
      pos--;
    }
    else {
      pos=r->size()-1;
      if(current()==(*r)[pos]) {
	if(pos==0) {
	  walkInversIgnoring();
	  delete r;
	  return true;
	}
	pos--;
      }
    }
  }
  while(walkInversIgnoring());
  delete r;
  return false;
}

bool TextFileModifier::after(List< const String * >& l) throw()
{
  List<char> lc;
  l.place();
  while(l.walk()) {
    for(int i=0;i<l.get()->size();i++) lc.put((*l.get())[i]);
    lc.put('\n');
  }
  String *s=new String(lc);
  bool ret=after(*s);
  delete s;
  return ret;
}

bool TextFileModifier::before(List< const String * >& l) throw()
{
  List<char> lc;
  l.place();
  while(l.walk()) {
    for(int i=0;i<l.get()->size();i++) lc.put((*l.get())[i]);
    lc.put('\n');
  }
  String *s=new String(lc);
  bool ret=before(*s);
  delete s;
  return ret;
}

void TextFileModifier::insert(List<const String*>& lines) throw()
{
  /*%%
  slashFound=false;
  starFound=false;
  inString=false;
  blanking=false;
  nbSeparator=1;
  comment=0;
  */

  if(!list.atTheQueue()) {
    if(list.get()->c!='\n' && list.walk() && list.get()->c!='\n' && list.walkHeadToQueue())
      insertRaw("\n");//securite
  }

  mark->place();
  while(mark->walk()) insertRaw(*mark->get()+*new String("\n"));
  //%%%    list.put(&(*new String(*mark->get())+*new String("\n")));

  int cpt=0;
  lines.place();
  while(lines.walk()) cpt++;
  if(cpt<=1) insertRaw(*new String("//")+*new String(cpt)+*new String(" line added\n"));
  else insertRaw(*new String("//")+*new String(cpt)+*new String(" lines added\n"));
  
  lines.place();
  while(lines.walk()) insertRaw(*lines.get()+*new String("\n"));
  //%%list.put(&(*lines.get()+*new String("\n")));
  
  /*%%
  markEnd->place();
  while(markEnd->walk())
    list.put(&(*new String(*markEnd->get())+*new String("\n")));
  */
  reparse();
}

void TextFileModifier::insert(const String& line) throw()
{
  /*%%
  slashFound=false;
  starFound=false;
  inString=false;
  blanking=false;
  nbSeparator=1;
  comment=0;
  */

  if(!list.atTheQueue()) {
    if(list.get()->c!='\n' && list.walk() && list.get()->c!='\n' && list.walkHeadToQueue())
      insertRaw("\n");//securite
  }

  mark->place();
  while(mark->walk()) insertRaw(*mark->get()+*new String("\n"));

  insertRaw("//1 line added\n");
  insertRaw(line+*new String("\n"));

  reparse();
}

void TextFileModifier::insertRaw(const String& line) throw()
{
  for(int i=0;i<line.size();i++) {
    SensChar *s=new SensChar;
    s->c=line[i];
    s->added=false;
    list.put(s);
  }
  //%%putChar(line[i]);
}

void TextFileModifier::jumpBlank() throw()
{
  while(walk()) {
    if(!isBlank(list.get()->c) && list.get()->s==REAL) {
      walkInvers();
      return;
    }
  }
}

void TextFileModifier::jumpBlankInvers() throw()
{
  do {
    if(!isBlank(list.get()->c) && list.get()->s==REAL) return;
  } while(walkInvers());
}

/*
  supprimer addon:
  trouver '\n'+mark (en trouvant cpt++)
  => supprimer walkingInvers (while cpt>0)
  => trouver "//" et supprimer
  => trouver un string et le mettre dans un int, et supprimer
  => trouver " lines added\n" ou " line added\n" et supprimer
  => for le int : supprLine = ^K - emacs
*/

String *TextFileModifier::nextWord() throw()
{
  //%%  cout<<">"<<list.get()->c;cout.flush();

  keepPosition();

  List<char> l;
  while(walkIgnoring()) {
    char c=current();
    if(isSeparator(c)) {
      if(l.empty()) l.put(c);
      recallPosition();
      return new String(l);
    }
    if(isBlank(c)) {
      if(!l.empty()) {
	recallPosition();
	return new String(l);
      }
    }
    else {
      l.put(c);
    }
  }
  recallPosition();
  return new String();
}

void TextFileModifier::keepPosition() throw()
{
  listPosition.put(cptWalk);
}

void TextFileModifier::recallPosition() throw(Exception)
{
  if(listPosition.empty()) throw Exception("Cannot recall position after insert");
  int cpt=listPosition.get();
  listPosition.suppr();

  if(cptWalk==tmpLimit) {
    if(cpt<cptWalk) {
      maxLimit=tmpLimit;
      tmpLimit=-1;
    }
    else if(cpt>cptWalk) {
      minLimit=tmpLimit;
      tmpLimit=-1;
    }
  }

  int saveMaxLimit=maxLimit;
  int saveMinLimit=minLimit;
  int saveTmpLimit=tmpLimit;

  place();
  while(cpt--) walk();

  maxLimit=saveMaxLimit;
  minLimit=saveMinLimit;
  tmpLimit=saveTmpLimit;
}


void TextFileModifier::goEndLine() throw()
{
  while(walk()) {
    if(current()=='\n') return;
  }
  return;
}

bool TextFileModifier::findCloseBrace() throw()
{
  int cptOpen=0;
  bool walkInString=false;
  while(walkIgnoring()) {
    if(current()=='\"')
      if(walkInString) walkInString=false;
      else walkInString=true;
    if(!walkInString) {
      if(current()=='{') cptOpen++;
      if(current()=='}') {
	if(cptOpen==0) return true;
	cptOpen--;
      }
    }
  }
  return false;
}
