/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "Triangle.h"

template <class T>
void swap(T &a,T &b) {
  T tmp=a;
  a=b;
  b=tmp;
}

Triangle::Triangle(Point2D a,Point2D b,Point2D c) throw()
:COEF(8)
{
  points[0]=a;
  points[1]=b;
  points[2]=c;
  
  // Ordonne les points Haut -> Bas

  if (points[1].getY()<points[0].getY())
    swap(points[0],points[1]);
  
  if (points[2].getY()<points[0].getY())
    swap(points[0],points[2]);
  
  if (points[2].getY()<points[1].getY())
    swap(points[1],points[2]);
  
  // Ordonne les points Gauche -> Droite

  if (points[0].getY()==points[1].getY())
    if (points[1].getX()<points[0].getX())
      swap(points[0],points[1]);
  
  if (points[1].getY()==points[2].getY())
    if (points[2].getX()<points[1].getX())
      swap(points[1],points[2]);
  
  int deltaX0=points[1].getX()-points[0].getX();
  int deltaX1=points[2].getX()-points[1].getX();
  int deltaX2=points[2].getX()-points[0].getX();
  
  int deltaY0=points[1].getY()-points[0].getY();
  int deltaY1=points[2].getY()-points[1].getY();
  int deltaY2=points[2].getY()-points[0].getY();
  
  int deltaS0=points[1].getS()-points[0].getS();
  int deltaS1=points[2].getS()-points[1].getS();
  int deltaS2=points[2].getS()-points[0].getS();
  
  int deltaT0=points[1].getT()-points[0].getT();
  int deltaT1=points[2].getT()-points[1].getT();
  int deltaT2=points[2].getT()-points[0].getT();
  
  int incX01=0;
  int incX12=0;
  int incX02=0;
  
  int incS01=0;
  int incT01=0;
  
  int incS12=0;
  int incT12=0;

  int incS02=0;
  int incT02=0;
  
  if (deltaY0!=0) {
    incX01=(deltaX0<<COEF)/deltaY0;
    incS01=(deltaS0<<COEF)/deltaY0;
    incT01=(deltaT0<<COEF)/deltaY0;
  }
  
  if (deltaY1!=0) {
    incX12=(deltaX1<<COEF)/deltaY1;
    incS12=(deltaS1<<COEF)/deltaY1;
    incT12=(deltaT1<<COEF)/deltaY1;
  }
  
  if (deltaY2!=0) {
    incX02=(deltaX2<<COEF)/deltaY2;
    incS02=(deltaS2<<COEF)/deltaY2;
    incT02=(deltaT2<<COEF)/deltaY2;
  }
  
  if (deltaY0!=0) {
    xGUp=xDUp=points[0].getX()<<COEF;
    sGUp=sDUp=points[0].getS()<<COEF;
    tGUp=tDUp=points[0].getT()<<COEF;
    if (incX01>incX02) {
      incXGUp=incX02;
      incXDUp=incX01;
	
      incXGDown=incX02;
      incXDDown=incX12;
      
      incSGUp=incS02;
      incSDUp=incS01;

      incSGDown=incS02;
      incSDDown=incS12;
      
      incTGUp=incT02;
      incTDUp=incT01;
      
      incTGDown=incT02;
      incTDDown=incT12;
    } else {
      incXGUp=incX01;
      incXDUp=incX02;
      
      incXGDown=incX12;
      incXDDown=incX02;
	
      incSGUp=incS01;
      incSDUp=incS02;

      incSGDown=incS12;
      incSDDown=incS02;

      incTGUp=incT01;
      incTDUp=incT02;

      incTGDown=incT12;
      incTDDown=incT02;
    }
  } else {
    if (points[0].getX()<points[1].getX()) {
      xGUp=points[0].getX()<<COEF;
      xDUp=points[1].getX()<<COEF;
      incXGDown=incX02;
      incXDDown=incX12;

      sGUp=points[0].getS()<<COEF;
      sDUp=points[1].getS()<<COEF;
      incSGDown=incS02;
      incSDDown=incS12;

      tGUp=points[0].getT()<<COEF;
      tDUp=points[1].getT()<<COEF;
      incTGDown=incT02;
      incTDDown=incT12;
    } else {
      xGUp=points[1].getX()<<COEF;
      xDUp=points[0].getX()<<COEF;
      incXGDown=incX12;
      incXDDown=incX02;

      sGUp=points[1].getS()<<COEF;
      sDUp=points[0].getS()<<COEF;
      incSGDown=incS12;
      incSDDown=incS02;

      tGUp=points[1].getT()<<COEF;
      tDUp=points[0].getT()<<COEF;
      incTGDown=incT12;
      incTDDown=incT02;
    }
  }
}

void Triangle::fill(PixelMatrix& image,const PixelMatrix& map) throw()
{
  int x0=xGUp;
  int x1=xDUp;

  int s0=sGUp;
  int s1=sDUp;

  int t0=tGUp;
  int t1=tDUp;

  int y0=points[0].getY();
  int y1=points[1].getY();
  int y2=points[2].getY();

  if (y0<0 && y2<0)
    return;

  int incS=0;
  int incT=0;

  if (incXDUp-incXGUp!=0) {
    incS=((incSDUp-incSGUp)<<COEF)/(incXDUp-incXGUp);
    incT=((incTDUp-incTGUp)<<COEF)/(incXDUp-incXGUp);
  }

  while (y0<0 && y0<y1) {
    x0+=incXGUp;
    x1+=incXDUp;
    s0+=incSGUp;
    s1+=incSDUp;
    t0+=incTGUp;
    t1+=incTDUp;
    y0++;
  }

  if (y1>=200)
    y1=199;

  for (int y=y0;y<y1;y++) {
    int s=s0;
    int t=t0;

    int xDebut=x0>>COEF;
    int xFin=x1>>COEF;

    if (xDebut<320) {
      if (xFin>=320)
	xFin=319;

      int x=xDebut;
      while (x<0) {
	s+=incS;
	t+=incT;
	x++;
      }

      for (;x<xFin;x++) {
	int divS=s>>COEF;
	int divT=t>>COEF;
	if (divS>=0 && divS<320 && divT>=0 && divT<200)
	  image[y][x]=map[divT][divS];
	  
	s+=incS;
	t+=incT;
      }
    }

    x0+=incXGUp;
    x1+=incXDUp;
    s0+=incSGUp;
    s1+=incSDUp;
    t0+=incTGUp;
    t1+=incTDUp;
  }

  if (incXDDown-incXGDown!=0) {
    incS=((incSDDown-incSGDown)<<COEF)/(incXDDown-incXGDown);
    incT=((incTDDown-incTGDown)<<COEF)/(incXDDown-incXGDown);
  }

  while (y1<0) {
    x0+=incXGDown;
    x1+=incXDDown;
    s0+=incSGDown;
    s1+=incSDDown;
    t0+=incTGDown;
    t1+=incTDDown;
    y1++;
  }

  if (y2>=200)
    y2=199;

  for (int y=y1;y<y2;y++) {
    int s=s0;
    int t=t0;

    int xDebut=x0>>COEF;
    int xFin=x1>>COEF;

    if (xDebut<320) {
      if (xFin>=320)
	xFin=319;

      int x=xDebut;
      while (x<0) {
	s+=incS;
	t+=incT;
	x++;
      }

      for (;x<xFin;x++) {
	int divS=s>>COEF;
	int divT=t>>COEF;
	if (divS>=0 && divS<320 && divT>=0 && divT<200)
	  image[y][x]=map[divT][divS];
	  
	s+=incS;
	t+=incT;
      }
    }

    x0+=incXGDown;
    x1+=incXDDown;
    s0+=incSGDown;
    s1+=incSDDown;
    t0+=incTGDown;
    t1+=incTDDown;
  }
}
