/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */

#include "Window.h"



void
Window::
copy(const Window& z) throw ()
{

  int jAux=0,iAux=0;
  if(z.w.savePos.j<0) jAux=-z.w.savePos.j;
  if(z.w.savePos.i<0) iAux=-z.w.savePos.i;

  int jPos=w.savePos.j+jAux,iPos=w.savePos.i+iAux;

  if(!w.print) return;
  if(!z.w.print) return;

  if(jPos>=w.pos.j+w.size.j) return;
  if(jPos+z.w.size.j<=w.pos.j) return;
  if(iPos>=w.pos.i+w.size.i) return;
  if(iPos+z.w.size.i<=w.pos.i) return;
  
  int jDecal=0,iDecal=0;
  int jSize=z.w.size.j;
  int iSize=z.w.size.i;
  clip(jPos,iPos,jSize,iSize,jDecal,iDecal);
  
  tab->copy(*(z.tab),
	    jDecal+z.w.pos.j,
	    iDecal+z.w.pos.i,
	    jPos,
	    iPos,
	    jSize,
	    iSize);
}



void
Window::
clip(int& jPos,int& iPos,
     int& jSize,int& iSize,
     int& jDecal,int& iDecal) const throw ()
{
  if(jPos<w.pos.j) {
    jSize-=w.pos.j-jPos;
    jDecal=w.pos.j-jPos;
    jPos=w.pos.j;
  }
  if(jPos+jSize>w.pos.j+w.size.j) {
    jSize=w.pos.j+w.size.j-jPos;
  }
  
  if(iPos<w.pos.i) {
    iSize-=w.pos.i-iPos;
    iDecal=w.pos.i-iPos;
    iPos=w.pos.i;
  }
  if(iPos+iSize>w.pos.i+w.size.i) {
    iSize=w.pos.i+w.size.i-iPos;
  }
}


Window::
Window(PixelMatrix& ref) throw ()
  :w(ref.nbLine(),ref.nbColumn()),
   tab(&ref)
{
  //  set(0,0,ref.nbLine(),ref.nbColumn());
}

void
Window::
set(int jPos,int iPos) throw ()
{
  set(jPos,iPos,w.saveSize.j,w.saveSize.i);
}


void
Window::
set(int jPos,int iPos,int jSize,int iSize) throw ()
{
  w.saveSize.j=jSize;
  w.saveSize.i=iSize;
  w.savePos.j=jPos;
  w.savePos.i=iPos;
  w.print=true;
  if(jPos<0) {
    jSize+=jPos;
    if(jSize<=0) w.print=false;
    else {
      if(jSize>tab->nbLine()) w.size.j=tab->nbLine();
      else w.size.j=jSize;
      w.pos.j=0;
    }
  } else
    if(jPos>=tab->nbLine()) {
      w.print=false;
    } else
      if(jPos+jSize>tab->nbLine()) {
	w.size.j=tab->nbLine()-jPos;
	w.pos.j=jPos;
      } else {
	w.pos.j=jPos;
	w.size.j=jSize;
      }
  //
  if(w.print)
    if(iPos<0) {
      iSize+=iPos;
      if(iSize<=0) w.print=false;
      else {
	if(iSize>tab->nbColumn()) w.size.i=tab->nbColumn();
	else w.size.i=iSize;
	w.pos.i=0;
      }
    } else
      if(iPos>=tab->nbColumn()) {
	w.print=false;
      } else
	if(iPos+iSize>tab->nbColumn()) {
	  w.size.i=tab->nbColumn()-iPos;
	  w.pos.i=iPos;
	} else {
	  w.pos.i=iPos;
	  w.size.i=iSize;
	}
}

void
Window::
get(int& jPos,int& iPos,int& jSize,int& iSize) const throw()
{
  jPos=w.savePos.j;
  iPos=w.savePos.i;
  jSize=w.saveSize.j;
  iSize=w.saveSize.i;
}


void Window::putSprite(const Sprite& a,int jPos,int iPos) throw()
{
  if(!w.print) return;
  if(jPos>=w.pos.j+w.size.j) return;
  if(jPos+a.nbLine()<=w.pos.j) return;
  if(iPos>=w.pos.i+w.size.i) return;
  if(iPos+a.nbColumn()<=w.pos.i) return;
  
  int jDecal=0,iDecal=0;
  int jSize=a.nbLine();
  int iSize=a.nbColumn();
  clip(jPos,iPos,jSize,iSize,jDecal,iDecal);
  
  tab->putSprite(*(a.getData()),
		 *(a.getMask()),
		 jDecal,iDecal,
		 jPos,iPos,
		 jSize,iSize);
}

void Window::putSpriteColor(Pixel color,const Sprite& a,int jPos,int iPos) throw()
{
  if(!w.print) return;
  if(jPos>=w.pos.j+w.size.j) return;
  if(jPos+a.nbLine()<=w.pos.j) return;
  if(iPos>=w.pos.i+w.size.i) return;
  if(iPos+a.nbColumn()<=w.pos.i) return;
  
  int jDecal=0,iDecal=0;
  int jSize=a.nbLine();
  int iSize=a.nbColumn();
  clip(jPos,iPos,jSize,iSize,jDecal,iDecal);
  
  tab->putSpriteColor(color,
		      *(a.getData()),
		      jDecal,iDecal,
		      jPos,iPos,
		      jSize,iSize);
}


void Window::putSpriteUnderMask(const Sprite& a,int jPos,int iPos,
				const UnderMaskMatrix& underMask,
				int jUnderMask,int iUnderMask) throw ()
{
  if(!w.print) return;
  if(jPos>=w.pos.j+w.size.j) return;
  if(jPos+a.nbLine()<=w.pos.j) return;
  if(iPos>=w.pos.i+w.size.i) return;
  if(iPos+a.nbColumn()<=w.pos.i) return;
  
  int jDecal=0,iDecal=0;
  int jSize=a.nbLine();
  int iSize=a.nbColumn();
  clip(jPos,iPos,jSize,iSize,jDecal,iDecal);

  if(jUnderMask+jDecal+jSize>underMask.nbLine())
    jSize=underMask.nbLine()-jUnderMask-jDecal;

  if(iUnderMask+iDecal+iSize>underMask.nbColumn())
    iSize=underMask.nbLine()-iUnderMask-iDecal;
  
  tab->putSpriteUnderMask(*(a.getData()),
			  *(a.getMask()),
			  jDecal,iDecal,
			  jPos,iPos,
			  jSize,iSize,
			  underMask,jUnderMask+jDecal,iUnderMask+iDecal);
}


void Window::putSpriteUnderMaskPalette(const Sprite& a,int jPos,int iPos,
				       const UnderMaskMatrix& underMask,
				       int jUnderMask,int iUnderMask,
				       const PixelPalette& pal) throw ()
{
  if(!w.print) return;
  if(jPos>=w.pos.j+w.size.j) return;
  if(jPos+a.nbLine()<=w.pos.j) return;
  if(iPos>=w.pos.i+w.size.i) return;
  if(iPos+a.nbColumn()<=w.pos.i) return;
  
  int jDecal=0,iDecal=0;
  int jSize=a.nbLine();
  int iSize=a.nbColumn();
  clip(jPos,iPos,jSize,iSize,jDecal,iDecal);

  if(jUnderMask+jDecal+jSize>underMask.nbLine())
    jSize=underMask.nbLine()-jUnderMask-jDecal;

  if(iUnderMask+iDecal+iSize>underMask.nbColumn())
    iSize=underMask.nbLine()-iUnderMask-iDecal;
  
  tab->putSpriteUnderMaskPalette(*(a.getData()),
				 *(a.getMask()),
				 jDecal,iDecal,
				 jPos,iPos,
				 jSize,iSize,
				 underMask,jUnderMask+jDecal,iUnderMask+iDecal,
				 pal);
}

void Window::clear() throw()
{
  *tab=toPixel(0,0,0,0);
}

