/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */

#include "WindowPipe.h"

WindowPipe::
WindowPipe(Window& aTo,Window& aFrom,const UnderMaskMatrix& underMask) throw ()
  :to(&aTo),from(&aFrom),mask(&underMask),windowToToClear(false)
{}

/*
WindowPipe::
WindowPipe(Window& aTo,
	   Window& aFromData,Window& aFromMask) throw ()
  :to(&aTo),fromData(&aFromData),fromMask(&aFromMask)
{}
*/

WindowPipe::
~WindowPipe() throw ()
{
  Zlist.place();
  while(Zlist.walk()) delete Zlist.get();
  ZlistOn.place();
  while(ZlistOn.walk()) delete ZlistOn.get();
  ZlistToUppest.place();
  while(ZlistToUppest.walk()) delete ZlistToUppest.get();
  ZlistTo.place();
  while(ZlistTo.walk()) delete ZlistTo.get();
  ZlistColorTo.place();
  while(ZlistColorTo.walk()) delete ZlistColorTo.get();
}


void WindowPipe::paintWindowFrom() throw()
{
  //vide la liste
  Zlist.place();
  while(Zlist.walk()) {
    delete Zlist.get();
    Zlist.suppr();
  }

  ZlistOn.place();
  while(ZlistOn.walk()) {
    delete ZlistOn.get();
    ZlistOn.suppr();
  }

  ZlistTo.place();
  while(ZlistTo.walk()) {
    delete ZlistTo.get();
    ZlistTo.suppr();
  }

  ZlistColorTo.place();
  while(ZlistColorTo.walk()) {
    delete ZlistColorTo.get();
    ZlistColorTo.suppr();
  }

  ZlistToUppest.place();
  while(ZlistToUppest.walk()) {
    delete ZlistToUppest.get();
    ZlistToUppest.suppr();
  }

}

void WindowPipe::put(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw ()
{
  T_Zlist *z=new T_Zlist;
  z->s=&s;
  z->j=jFromRelativePos;
  z->i=iFromRelativePos;
  z->pal=0;
  Zlist.put(z);
}

void WindowPipe::putWithPalette(const Sprite& s,int jFromRelativePos,int iFromRelativePos,const PixelPalette& pal) throw ()
{
  T_Zlist *z=new T_Zlist;
  z->s=&s;
  z->j=jFromRelativePos;
  z->i=iFromRelativePos;
  z->pal=&pal;
  Zlist.put(z);
}

void WindowPipe::putOn(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw ()
{
  T_Zlist *z=new T_Zlist;
  z->s=&s;
  z->j=jFromRelativePos;
  z->i=iFromRelativePos;
  z->pal=0;
  ZlistOn.put(z);
}

void WindowPipe::putTo(const Sprite& s,int jPos,int iPos,bool uppest) throw ()
{
  T_Zlist *z=new T_Zlist;
  z->s=&s;
  z->j=jPos;
  z->i=iPos;
  z->pal=0;
  if(uppest) ZlistToUppest.put(z);
  else ZlistTo.put(z);
}

void WindowPipe::putColorTo(Pixel color,const Sprite& s,int jPos,int iPos) throw ()
{
  T_ZColorlist *z=new T_ZColorlist;
  z->s=&s;
  z->j=jPos;
  z->i=iPos;
  z->color=color;
  ZlistColorTo.put(z);
}


void WindowPipe::refreshAll() throw()
{
  realPaintWindowFrom();
    
  Zlist.place();
  while(Zlist.walk()) {
    T_Zlist *z=Zlist.get();

    realPut(*(z->s),z->j,z->i,z->pal);
    
    delete z;
    Zlist.suppr();
  }
  
  ZlistOn.place();
  while(ZlistOn.walk()) {
    T_Zlist *z=ZlistOn.get();

    realPutOn(*(z->s),z->j,z->i);
    
    delete z;
    ZlistOn.suppr();
  }
  
  ZlistTo.place();
  while(ZlistTo.walk()) {
    T_Zlist *z=ZlistTo.get();

    realPutTo(*(z->s),z->j,z->i);
    
    delete z;
    ZlistTo.suppr();
  }

  ZlistColorTo.place();
  while(ZlistColorTo.walk()) {
    T_ZColorlist *z=ZlistColorTo.get();

    realPutColorTo(z->color,*(z->s),z->j,z->i);
    
    delete z;
    ZlistColorTo.suppr();
  }
  
  ZlistToUppest.place();
  while(ZlistToUppest.walk()) {
    T_Zlist *z=ZlistToUppest.get();

    realPutTo(*(z->s),z->j,z->i);

    delete z;
    ZlistToUppest.suppr();
  }

}

void WindowPipe::realPaintWindowFrom() throw()
{
  if(windowToToClear) {
    to->clear();
    windowToToClear=false;
  }
  else to->copy(*from);
}

void WindowPipe::realPut(const Sprite& s,int jFromRelativePos,int iFromRelativePos,const PixelPalette *pal) throw()
{
  int jPos=to->w.savePos.j-(from->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(from->w.savePos.i-iFromRelativePos);
  if(pal)
    to->putSpriteUnderMaskPalette(s,jPos,iPos,
				  *mask,jFromRelativePos,iFromRelativePos,
				  *pal);
  else
    to->putSpriteUnderMask(s,jPos,iPos,
			   *mask,jFromRelativePos,iFromRelativePos);
}


void WindowPipe::realPutOn(const Sprite& s,int jFromRelativePos,int iFromRelativePos) throw()
{ 
  int jPos=to->w.savePos.j-(from->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(from->w.savePos.i-iFromRelativePos);
  to->putSprite(s,jPos,iPos);
}

void WindowPipe::realPutTo(const Sprite& s,int jPos,int iPos) throw()
{ 
  int j=to->w.savePos.j+jPos;
  int i=to->w.savePos.i+iPos;
  to->putSprite(s,j,i);
}

void WindowPipe::realPutColorTo(Pixel color,const Sprite& s,int jPos,int iPos) throw()
{ 
  int j=to->w.savePos.j+jPos;
  int i=to->w.savePos.i+iPos;
  to->putSpriteColor(color,s,j,i);
}

void WindowPipe::putUnder(const Sprite& s,int jPos,int iPos) throw ()
{
  from->putSpriteUnderMask(s,jPos,iPos,
			   *mask,jPos,iPos);
}

/*
void
WindowPipe::
refresh() throw ()
{
  //  to->copy(*(fromData->tab));
  to->copy(*fromData);
}

void
WindowPipe::
put(const Array3<Pixel>& a,int jFromRelativePos,int iFromRelativePos) throw ()
{
  int jPos=to->w.savePos.j-(fromData->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(fromData->w.savePos.i-iFromRelativePos);
  if(fromMask) {
    to->andOrUnderMask(a,jPos,iPos,*(fromMask->tab),
		       jFromRelativePos,iFromRelativePos);
  }
  else to->andOr(a,jPos,iPos);
}

void
WindowPipe::
putOn(const Array3<Pixel>& a,
      int jFromRelativePos,int iFromRelativePos) throw ()
{
  int jPos=to->w.savePos.j-(fromData->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(fromData->w.savePos.i-iFromRelativePos);
  to->andOrWithoutShadow(a,jPos,iPos);
}

void
WindowPipe::
putUnder(const Array3<Pixel>& a,
	 int jPos,int iPos) throw ()
{
  if(fromMask) fromData->andOrUnderMask(a,jPos,iPos,*(fromMask->tab),
				    jPos,iPos);
  else fromData->andOr(a,jPos,iPos);
}


void
WindowPipe::
putTo(const Array3<Pixel>& a,
      int jFromRelativePos,int iFromRelativePos) throw ()
{
  int jPos=to->w.savePos.j+jFromRelativePos;
  int iPos=to->w.savePos.i+iFromRelativePos;
  to->andOrWithoutShadow(a,jPos,iPos);
}

void
WindowPipe::
putColor(const Array3<Pixel>& a,int color,
	 int jFromRelativePos,int iFromRelativePos) throw ()
{
  int jPos=to->w.savePos.j-(fromData->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(fromData->w.savePos.i-iFromRelativePos);
  to->andColor(a,color,jPos,iPos);
}


void
WindowPipe::
putOnColor(const Array3<Pixel>& a,int color,
	   int jFromRelativePos,int iFromRelativePos) throw ()
{
  //idem putColor
  int jPos=to->w.savePos.j-(fromData->w.savePos.j-jFromRelativePos);
  int iPos=to->w.savePos.i-(fromData->w.savePos.i-iFromRelativePos);
  to->andColor(a,color,jPos,iPos);
}

void
WindowPipe::
putUnderColor(const Array3<Pixel>& a,int color,
	      int jPos,int iPos) throw ()
{
  if(fromMask) fromData->andColorUnderMask(a,color,jPos,iPos,*(fromMask->tab),
					   jPos,iPos);
  else fromData->andColor(a,color,jPos,iPos);
}

void
WindowPipe::
putToColor(const Array3<Pixel>& a,int color,
	   int jFromRelativePos,int iFromRelativePos) throw ()
{
  int jPos=to->w.savePos.j+jFromRelativePos;
  int iPos=to->w.savePos.i+iFromRelativePos;
  to->andColor(a,color,jPos,iPos);
}
*/

void
WindowPipe::
moveFrom(int j,int i) throw ()
{
  from->set(j,i);
}

void
WindowPipe::
moveTo(int j,int i) throw ()
{
  to->set(j,i);
}

void
WindowPipe::
getFrom(int& jPos,int& iPos,int& jSize,int& iSize) const throw()
{
  from->get(jPos,iPos,jSize,iSize);
}

void
WindowPipe::
getTo(int& jPos,int& iPos,int& jSize,int& iSize) const throw()
{
  to->get(jPos,iPos,jSize,iSize);
}

void
WindowPipe::
clearTo() throw()
{
  windowToToClear=true;
}
