/* Bomb O Bomb - an entirely transformable game for Linux
 * David FAUTHOUX - July 1999 - email: david.fauthoux@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Modify this soft ! It is made to be original for each player who want it !!
 */



#include "display.h"
#include <vgakeyboard.h>
#include <vga.h>
#include <vgagl.h>
#include <string.h>
#include "Fifo.h"



bool graphicActif=false;
int oldMode=0;
bool keyboardActif=false;

Fifo<int> keyboardCode;
Fifo<bool> keyboardPressed;

#define NBKEY 255
bool switchKeyboardCode[NBKEY];//true if pressing

void keyInBuffer(int key,int press)
{
  keyboardCode.put(key);
  if(press==KEY_EVENTRELEASE) keyboardPressed.put(false);
  else keyboardPressed.put(true);
}

bool enableKeyboard()
{
  if(keyboard_init()!=0) return false;
  keyboard_seteventhandler(keyInBuffer);
  keyboard_translatekeys(DONT_CATCH_CTRLC);
  keyboardActif=true;
  for(int k=0;k<NBKEY;k++) switchKeyboardCode[k]=false;
  return true;
}

void disableKeyboard()
{
  if(keyboardActif) {
    keyboard_close();
    keyboardActif=false;
    while(!keyboardCode.empty()) keyboardCode.get();
    while(!keyboardPressed.empty()) keyboardPressed.get();
    for(int k=0;k<NBKEY;k++)
      if(switchKeyboardCode[k]) {
	switchKeyboardCode[k]=false;
	keyboardCode.put(k);
	keyboardPressed.put(false);
      }
  }
}

bool keyboardRefresh()
{
  if(keyboard_update()==0) return false;
  return true;
}

void getKeyboardCode(int& code,bool& pressed)
{
  //  keyboard_update();
  if(keyboardCode.empty()) {
    code=-1;
    return;
  }
  code=keyboardCode.get();
  pressed=keyboardPressed.get();
  if(pressed) switchKeyboardCode[code]=true;
  else switchKeyboardCode[code]=false;
}

//


PhysicPixel *video=new PhysicPixel[320*200];
Pixel *buffer=new Pixel[320*200];


bool enableGraphic()
{
  bool remettreKeyboard=false;
  if(graphicActif) return false;
  if(keyboardActif) {
    remettreKeyboard=true;
    disableKeyboard();
  }

  /*  vga_disabledriverreport(); */

  bool ret=true;

  oldMode=vga_getcurrentmode();
  switch(PIXEL_NUMBER_OF_COLOR) {
  case (64*1024):
    if(vga_setmode(G320x200x64K)==0
       && gl_setcontextvga(G320x200x64K)==0) ret=true;
    else ret=false;
    break;
  case (32*1024):
    if(vga_setmode(G320x200x32K)==0
       && gl_setcontextvga(G320x200x32K)==0) ret=true;
    else ret=false;
    break;
  case (256):
    if(vga_setmode(G320x200x256)==0
       && gl_setcontextvga(G320x200x256)==0) {
      ret=true;
      for(int k=0;k<256;k++)
	//	vga_setpalette(k,toRed(k),toGreen(k),toBlue(k));
	vga_setpalette(k,
		       getRedFromPhysicPixel(k)>>2,
		       getGreenFromPhysicPixel(k)>>2,
		       getBlueFromPhysicPixel(k)>>2);
    }
    else ret=false;
    break;
  default:
    ret=false;
    break;
  }

  if(remettreKeyboard) enableKeyboard();
  graphicActif=true;
  return ret;
}

void disableGraphic()
{
  bool remettreKeyboard=false;
  if(graphicActif) {
    if(keyboardActif) {
      remettreKeyboard=true;
      disableKeyboard();
    }
    //    vga_setmode(oldMode);
    vga_setmode(TEXT);
    if(remettreKeyboard) enableKeyboard();
    graphicActif=false;
  }
}

int graphicWidth()
{
  return 320;
}

int graphicHeight()
{
  return 200;
}


void graphicRefresh(const Pixel *buffer)
  //buffer size=sizeof(Pixel)*graphicWitdth()*graphicHeight();
{
  if(!graphicActif) return;
  for(int k=0;k<320*200;k++)
    *(video+k)=(PhysicPixel)*(buffer+k);
  gl_putbox(0,0,320,200,video);
}

/*
//

extern void setAllPalette(int *tab)//sizePalette()*3 int
{
  vga_setpalvec(0,256,tab);
}

extern void setPalette(int k,int r,int g,int b)
{
  vga_setpalette(k,r,g,b);
}

int sizePalette()
{
  return 256;
}

int rgbMax()
{
  return 63;
}

*/
